GLOBAL.setmetatable(env,{__index=function(t,k) return GLOBAL.rawget(GLOBAL,k) end})

local PURIFY = Action({mount_enable = true})    
PURIFY.id = "PMMM_PURIFY"  PURIFY.str = GLOBAL.PUELLA_MOD_SETTINGS.LANGUAGE and "Purify" or "净化"
PURIFY.fn = function(act) 
    act.doer.components.soulgem:Purify(act.invobject) 
    return true  
end
AddAction(PURIFY)
AddStategraphActionHandler("wilsonboating",ActionHandler(PURIFY, "pmmm_doperifyaction"))
AddStategraphActionHandler("wilson",ActionHandler(PURIFY, "pmmm_doperifyaction"))

local PerifyState = State({
    name = "pmmm_doperifyaction", 
    tags = {"notalking","doing","busy"},
    onenter = function(inst)
        local targ = inst:GetBufferedAction() and inst:GetBufferedAction().target or nil
        if targ then targ:PushEvent("startlongaction") end
        local timeout = inst.components.soulgem.current < 20 and 10 or 5
        inst.sg.statemem.time = timeout
        inst.sg:SetTimeout(timeout)
        inst.components.locomotor:Stop()
        inst.SoundEmitter:PlaySound("dontstarve/wilson/make_trap", "make")  
        inst.AnimState:PlayAnimation("build_pre")
        inst.AnimState:PushAnimation("build_loop", true)

        inst.sg.statemem.purifyring = SpawnPrefab("magic_purify_fx")
        inst.sg.statemem.purifyring:InFn(inst:GetPosition())
    end,
    timeline=
        {
            TimeEvent(4*FRAMES, function(inst) inst.sg:RemoveStateTag("busy") end),
        },
    onupdate= function(inst)
        local p = 1-inst.sg.timeout/inst.sg.statemem.time
        inst.sg.statemem.purifyring.AnimState:SetPercent('progress',p)
    end,
    ontimeout= function(inst)
        inst.AnimState:PlayAnimation("build_pst")
        inst:PerformBufferedAction()
        inst.sg.statemem.out = true
        inst.sg.statemem.purifyring:OutFn()
        inst.sg:GoToState("idle", false)
    end,
    onexit= function(inst)
        inst.SoundEmitter:KillSound("make")
        if not inst.sg.statemem.out then ---被打断
            inst.sg.statemem.purifyring:Remove()
        end
    end,
})
AddStategraphState("wilson", PerifyState)
AddStategraphState("wilsonboating",PerifyState)

local DarkStunnedState = State({
    name = "pmmm_sd_darkness_stun",
    tags = { "busy" },
        
    onenter = function(inst,a)
        inst.sg:SetTimeout(a)
        inst.components.locomotor:Stop()
        inst.AnimState:PlayAnimation("idle_sanity_pre", false)
        inst.AnimState:PushAnimation("idle_sanity_loop", true)
    end,

    ontimeout = function(inst) inst.sg:GoToState('idle') end,
})
AddStategraphState("wilson",DarkStunnedState)
AddStategraphState("wilsonboating",DarkStunnedState)

local DarkHitState = State({
    name = 'pmmm_sd_darkattack',
    tags = {'busy','hit_darkness'},

    onenter = function(inst)

        inst.AnimState:PlayAnimation('hit_darkness')
        inst.SoundEmitter:PlaySound("dontstarve/charlie/attack")
        inst.components.locomotor:Stop()
        inst:ClearBufferedAction()

        inst.sg:SetTimeout(24 * FRAMES)
    end,

    ontimeout = function(inst)inst.sg:GoToState("idle")end,
})
AddStategraphState("wilson",DarkHitState)
AddStategraphState("wilsonboating",DarkHitState)

local ADD_WEAPON_BUFF = Action({mount_enable = true}) --加载配件
ADD_WEAPON_BUFF.id = 'HOMURA_ADDBUFF' ADD_WEAPON_BUFF.str =  GLOBAL.PUELLA_MOD_SETTINGS.LANGUAGE and "modify" or "添加配件"
ADD_WEAPON_BUFF.fn = function(act)
    act.invobject.components.container:Open(act.doer)
    return true
end
AddAction(ADD_WEAPON_BUFF)
AddStategraphActionHandler("wilsonboating",ActionHandler(ADD_WEAPON_BUFF, "give"))
AddStategraphActionHandler("wilson",ActionHandler(ADD_WEAPON_BUFF, "give"))

local Add_BUFF_INV = Action({mount_valid = true}) --物品栏加载配件
Add_BUFF_INV.id = 'HOMURA_ADDBUFF_INV' Add_BUFF_INV.str = ADD_WEAPON_BUFF.str
Add_BUFF_INV.fn = function (act)
    if act.target and act.target.components.container:IsOpen() then
        return ACTIONS.STORE.fn(act)
    else
        local workdesk = FindEntity(act.doer, 3, function(inst) return inst.prefab == 'homura_workdesk' end)
        if workdesk then
            act.target.components.container:Open(act.doer)
            return ACTIONS.STORE.fn(act)
        else
            act.doer.components.talker:Say(L and 'I need a work desk' or '材料齐全, 现在只缺一个工作台')
            return true
        end
    end
end

AddAction(Add_BUFF_INV)
AddStategraphActionHandler("wilsonboating",ActionHandler(Add_BUFF_INV, "give"))
AddStategraphActionHandler("wilson",ActionHandler(Add_BUFF_INV, "give"))

local ADD_AMMO = Action({mount_enable=true}) --添加子弹
ADD_AMMO.id = "HOMURA_TAKEAMMO" ADD_AMMO.str = GLOBAL.PUELLA_MOD_SETTINGS.LANGUAGE and "Reload" or "填弹"
ADD_AMMO.fn = function (act)
    act.target.components.homura_weapon:OnAccept(act.invobject,act.doer)
    return true
end
AddAction(ADD_AMMO)
AddStategraphActionHandler("wilsonboating",ActionHandler(ADD_AMMO, "give"))
AddStategraphActionHandler("wilson",ActionHandler(ADD_AMMO, "give"))

local FOCUS = Action({mount_enable=true},3,true)
FOCUS.id = "HOMURA_FOCUS" FOCUS.str = GLOBAL.PUELLA_MOD_SETTINGS.LANGUAGE and 'Aim at' or '瞄准'
FOCUS.fn = function(act)
    act.doer.components.combat:SetTarget(act.target)
    --act.doer.sg.statemem.focusactionpush = true
    return true
end
AddAction(FOCUS)

local function SGWilsonBoatingPostInit(sg)
    local old_onattacked = sg.events['attacked'].fn
    sg.events['attacked'] = EventHandler('attacked', function(inst,data,...)
        if inst.sg:HasStateTag('hit_darkness') then
            return
        end
        if inst.sg:HasStateTag('ignorehitstate') then
            return
        end
        old_onattacked(inst,data,...)
    end)

    sg.events['sd_darkattack'] = EventHandler('sd_darkattack', function(inst,data,...)
        if not inst.sg:HasStateTag('death') then
            inst.sg:GoToState('pmmm_sd_darkattack')
        end
    end)

    --部分sg的新标记
    --上下船移动期间不能进入停止移动的sg
    local invincibleSG = 
    {
        'jumpboatland','jumpboatstart','jumponboatstart_mounted','jumponboatstart','jumpoffboatstart','jumpoffboatland',
    }
    for _,v in pairs(invincibleSG)do
        if sg.states[v] then
            sg.states[v].tags['pmmm_invinvible'] = true
        end
    end
    --[[
    local blockSG = {}
    for _,v in pairs(blockSG)do
        if sg.states then
            table.insert(sg.states.tags, 'pmmm_block')
        end
    end
    --]]
end

--修正hair和torso图层的顺序
--p.s.克雷官方人物动画真的有毒!!!!!
---[[
local function SGWilsonPostInit_Run(sg)
    
    local old_enter = sg.states["run_start"].onenter 
    sg.states["run_start"].onenter = function(inst)
        old_enter(inst)
        if inst.prefab == 'kyouko' or inst.prefab == 'sayaka' then
            if inst.AnimState:IsCurrentAnimation('run_pre') then
                inst.AnimState:PlayAnimation('pmmm_run_pre')
            end
        end
    end
    local old_enter = sg.states["run"].onenter 
    sg.states["run"].onenter = function(inst)
        old_enter(inst)
        if inst.prefab == 'kyouko' or inst.prefab == 'sayaka' then
            if inst.AnimState:IsCurrentAnimation('run_loop') then
                inst.AnimState:PlayAnimation('pmmm_run_loop')
            end
        end
    end
    local old_enter = sg.states["run_stop"].onenter 
    sg.states["run_stop"].onenter = function(inst)
        old_enter(inst)
        if inst.prefab == 'kyouko' or inst.prefab == 'sayaka' then
            if inst.AnimState:IsCurrentAnimation('run_pst') then
                inst.AnimState:PlayAnimation('pmmm_run_pst')
            end
        end
    end
end
--]]
AddStategraphPostInit("wilson", SGWilsonPostInit_Run)
AddStategraphPostInit("wilson", SGWilsonBoatingPostInit)
AddStategraphPostInit("wilsonboating", SGWilsonBoatingPostInit)
