--[[
	这是一个简单的添加可选择配方的教程
	因为我只打算让雷管具有这个特点，所以没有把它做的非常通用性
]]

--组件api
AddComponentPostInit('builder', function(self)

	--如果要制作的物品是雷管，则采用特殊的判定方法
	local old_can = self.CanBuild
	function self:CanBuild(recname, ...)
		if recname == 'homura_detonator' and not self.freebuildmode then
            return old_can(self, recname .. (self.pmmm_detonator_craftmode or 1), ...)
    	else
    		return old_can(self, recname, ...)
    	end
    end

    --消耗的材料也需要修改
    local old_build = self.DoBuild
    function self:DoBuild(recname, ...)
    	if recname == 'homura_detonator' then
    		return old_build(self, recname .. (self.pmmm_detonator_craftmode or 1), ...)
    	else
    		return old_build(self, recname, ...)
    	end
    end

    --象征性的保存一下
    local old_save = self.OnSave
    function self:OnSave()
    	local data = old_save(self)
    	data.pmmm_detonator_craftmode = self.pmmm_detonator_craftmode
    	return data
    end

    local old_load = self.OnLoad
    function self:OnLoad(data)
    	if data then
    		self.pmmm_detonator_craftmode = data.pmmm_detonator_craftmode
    	end
    	old_load(self, data)
    end

end)

--修改界面
--顶部配方图标可点击来选择具体的合成材料
--如果这个配方暂时用不到，则显示为半透明
AddClassPostConstruct('widgets/recipepopup', function(self) --鼠标移动到配方tab时，显示出的面板

	--如果是雷管，则魔改界面
	local old_set = self.SetRecipe

	function self:SetRecipe(rec, owner)

		old_set(self, rec, owner)
		if not owner then
			return false
		end

		if rec and rec.name == 'homura_detonator' then
			local mode = owner.components.builder.pmmm_detonator_craftmode or 1
			local can_build = owner.components.builder:CanBuild('homura_detonator')

			self.desc:SetString(STRINGS.RECIPE_DESC['HOMURA_DETONATOR'..mode])

			local ing1, ing2
			for k, v in pairs(self.ing)do --这里存放了配方图标ui
				print(v.ing.texture)
				if v.ing.texture == 'gunpowder.tex' then --获取火药图标
					ing1 = v
				elseif v.ing.texture == 'powcake.tex' then --获取火药饼图标
					ing2 = v
				end
			end

			local function SetUnused(self) --变得半透明
				self.pmmm_unused = true
				self.ing:SetTint(1,1,1,0.2)

				self.bg.oldtex = self.bg.texture
				self.bg:SetTexture(resolvefilepath("images/hud.xml"), "inv_slot.tex")
				self.bg:SetTint(1,1,1,0.2)

				self.quant.oldtext = self.quant.string
				self.quant:SetString("-/-")
				self.quant:SetColour(1,1,1,0.2)
			end

			local function SetUsed(self) --恢复原样
				if not self.pmmm_unused then
					return
				else
					self.pmmm_unused = nil
				end

				self.ing:SetTint(1,1,1,1)

				self.bg:SetTexture(resolvefilepath("images/hud.xml"), self.bg.oldtex)
				self.bg:SetTint(1,1,1,1)

				self.quant:SetString(self.quant.oldtext)
				self.quant:SetColour(self.bg.oldtex == "inv_slot.tex" and {1,1,1,1} or {1,155/255,155/255,1})
			end

			if mode == 1 then
				SetUnused(ing2)
			else
				SetUnused(ing1)
			end

			local ing = {[1]=ing1,[2]=ing2} --批量处理
			local this = self --防止被参数覆盖
			for i = 1, 2 do

				local old_control = ing[i].OnControl
				ing[i].OnControl = function(self, control, down, ...)
					if old_control then
						old_control(self, control, down, ...)
					end
					if not self:IsEnabled() or not self.focus then return end
					if control == CONTROL_ACCEPT then
						if down then
							TheFrontEnd:GetSound():PlaySound("dontstarve/HUD/click_move")
							self:ScaleTo(1.1,0.9,0.1)
							self.down = true
						else
							if self.down then
								self.down = false
								self:ScaleTo(1,1,0)
								owner.components.builder.pmmm_detonator_craftmode = i
								if owner.components.builder:CanBuild('homura_detonator') then
									this.button:Enable()
								else
									this.button:Disable()
								end
								this.desc:SetString(STRINGS.RECIPE_DESC['HOMURA_DETONATOR'..i])
								SetUnused(ing[3-i])
								SetUsed(ing[i])
							end
						end
						return true
					end
				end

				ing[i].OnGainFocus = function(self)
					self:ScaleTo(1,1.1,0.1)
				end

				ing[i].OnLoseFocus = function(self)
					self:ScaleTo(1,1,0)
				end
			end
		end
	end
end)

AddPrefabPostInit('greenstaff', function(inst) --禁止绿杖分解雷管
	local old_can = inst.components.spellcaster.spelltest
	inst.components.spellcaster.spelltest = function(staff, caster, target, ...)
		if target and target.prefab == 'homura_detonator' then
			return false
		else
			return old_can(staff, caster, target, ...)
		end
	end
end)

