GLOBAL.setmetatable(env,{__index=function(t,k) return GLOBAL.rawget(GLOBAL,k) end})

GLOBAL.PUELLA_MOD_MATH = {}

--圆
GLOBAL.PUELLA_MOD_MATH.CRICLE = Class(function(self,ori,radius)
    self.ori = ori
    self.radius = radius

    function self:PointIs(inst_pos,r)
        inst_pos = inst_pos.GetPosition and inst_pos:GetPosition() or inst_pos
        r = r or self.radius
        local dist = self.ori:Dist(inst_pos)
        if dist < r then
            return "in"
        else
            return "out"
        end
    end

    function self:GetTracePosition(faceangle,num,internal,r)  --faceangle：其中一个点与圆心的连线必须朝向这个位置
        r = r or self.radius
        num = internal and r * 6.283 / internal or num
        local angle = faceangle or 0
        local postable = {}
        for i = 1,num do 
            local offset = Vector3(math.cos(angle)*r,0,math.sin(angle)*r)
            postable[i]=offset+self.ori
            angle = angle + 6.283 / num
        end
        return postable
    end
end)

--正多边形
GLOBAL.PUELLA_MOD_MATH.POLYGON = Class(function (self,ori,radius,side,faceangle) --正多边形的半径在此定义为：中心点至顶点的距离
    self.ori = ori
    self.radius = radius
    self.side = side 
    self.faceangle = faceangle or 0
    self.T = 6.283/side

    function self:PolarCoordinates(theta,r)  --theta为与半径线的夹角，输入角度，返回和原点的距离
        r = r or self.radius
        local angle1 = 1.571*(self.side-2)/self.side  ---这个角为定值
        local angle2 = 3.142-angle1-theta  ---这个角对应半径
        return math.sin(angle1)/math.sin(angle2)*r
    end

    function self:ReversePolarCoordinates(dist,r)  --dist为与顶点的距离，返回和原点的角度与距离
        r = r or self.radius
        local angle1 = 1.571*(self.side-2)/self.side  --同上
        local dist2 = (dist^2+r^2)-math.cos(angle1)*2*dist*r --余弦定理
        dist2 = math.sqrt(dist2)
        local sin2 = math.sin(angle1)*dist/dist2
        if dist^2 < r^2 + dist2^2 then --锐角三角形
            return math.asin(sin2),dist2
        else
            return 3.1416 - math.asin(sin2),dist2
        end
    end

    function self:PointIs(inst_pos,r)
        inst_pos = inst_pos.GetPosition and inst_pos:GetPosition() or inst_pos
        r = r or self.radius
        local vec = inst_pos - self.ori 
        local pointangle = math.atan2(vec.z,vec.x)+2*PI  --目标和原点的角度
        --print(self.faceangle/DEGREES)
        --print(self.T/DEGREES)
        --print((pointangle-self.faceangle)/DEGREES)
        local angle = math.mod(pointangle-self.faceangle,self.T)
        --print(angle/DEGREES) --这个角度应该比2PI/边数小
        local dist = vec:Length()  --距离
        local boundary = self:PolarCoordinates(angle,r)
        if dist < boundary then
            return "in"
        else
            return "out"
        end
    end

    function self:GetPointPosition()
        local postable = {}
        for period = 1,self.side do
            local base_angle = self.faceangle + self.T*period
            local offset = Vector3(self.radius*math.cos(base_angle),0,self.radius*math.sin(base_angle))
            postable[period]= self.ori + offset
        end
        return postable
    end
    
    function self:GetTracePosition(internal,r)
        r = r or self.radius
        local angle1 = 1.571*(self.side-2)/self.side
        local sidelength = 2*math.cos(angle1)*r
        local sidenum = math.ceil(sidelength / internal)
        internal = sidelength/sidenum
        local postable = {}
        for period = 1,self.side do
            local base_angle = self.faceangle + self.T*period
            for num = 1,sidenum do
                local _theta,_radius = self:ReversePolarCoordinates(internal*num)
                postable[#postable+1]= self.ori + Vector3(math.cos(_theta+base_angle)*_radius,0,math.sin(_theta+base_angle)*_radius)
            end
        end
        return postable
    end
end)

GLOBAL.PUELLA_MOD_MATH.OVAL = Class(function (self,ori,a,b) --在ui中使用
    self.ori = ori or Point(0,0,0)
    self.radius = radius
    self.side = side 
    self.faceangle = faceangle or 0
    self.T = 6.283/side
end)

