
local LootPack = Class(function(self, name, slots, loot, weight)
	self.name = name
	self.slots = slots
	self.loot = loot
	self.weight = weight
end)

local Ran = function(a, b) 
	if b > 1 then
		return function() 
			return math.random(a, b)
		end
	else
		return function()
			return GetRandomMinMax(a, b) 
		end 
	end
end

local loottable = {
	LootPack('Pistol Pack', 2, {homura_pistol = Ran(0.3,0.7), homura_gun_ammo1 = Ran(7,15)}, 10),
	LootPack('Pistol', 1, {homura_pistol = Ran(0.5,1)}, 20),
	LootPack('Pistol Ammo', 1, {homura_gun_ammo1 = Ran(10,20)}, 20),
	LootPack('Ori', 1, {homura_gun_ammo3 = Ran(7,15)}, 5),

	LootPack('Gun Pack', 2, {homura_gun_ammo1 = Ran(20,30), homura_gun = Ran(0.3,0.7)}, 5),
	LootPack('Gun', 1, {homura_gun = Ran(0.5,0.1)}, 10),
	LootPack('Gun Ammo', 2, {homura_gun_ammo1 = Ran(30,50)}, 10),

	LootPack('HMG Pack', 3, {homura_hmg = Ran(0.3,0.7), homura_gun_ammo2 = Ran(40,60)}, 5),
	LootPack('HMG', 1, {homura_hmg = Ran(0.5,1)}, 10),
	LootPack('HMG Ammo', 3, {homura_gun_ammo2 = Ran(70,100)}, 10),

	LootPack('Gun Powder', 1, {gunpowder = Ran(4,10)}, 20),
	LootPack('Bomb', 1, {homura_bomb_bomb = Ran(3,5)}, 10),
	LootPack('Flash Bomb', 1, {homura_bomb_flash = Ran(4,6)}, 10),
	LootPack('Fire Bomb', 1, {homura_bomb_fire = Ran(3,5)}, 10),

	LootPack('Missle 1', 1, {homura_rpg_ammo1 = Ran(1,3)}, 10),
	LootPack('Missle 2', 1, {homura_rpg_ammo2 = Ran(1,5)}, 10),
	LootPack('RPG', 1, {homura_rpg = Ran(1,1)}, 5),

	--LootPack('稀有配件1', 1, {homura_weapon_buff_whitehole = function()return 1 end}, 1),
	LootPack('彩蛋', 1, {homura_pistol_sn = Ran(1,1)}, 1),

}

local function PickHomuraLoot(left_slots)
	if left_slots == 0 then
		return {}
	end
	local t = {}
	local totalweight = 0
	for _, v in pairs(loottable)do
		if v.slots <= left_slots then
			table.insert(t, v)
			totalweight = totalweight + v.weight
		end
	end

	local sel = totalweight * math.random()
	for _, v in pairs(t)do
		sel = sel - v.weight
		if sel <= 0 then
			return v.loot
		end
	end
end

local function AddHomuraLoot(inst, rate)
	if math.random() > rate then
		return
	end
	local loot = PickHomuraLoot(inst.components.container.numslots - inst.components.container:NumItems())
	assert(loot)
	for prefabname, init in pairs(loot)do
		local item = SpawnPrefab(prefabname)
		assert(item)
		local number = init() or 1
		if number >= 1 and item.components.stackable then
			item.components.stackable:SetStackSize(number)
		end
		if number <= 1 and item.components.homura_weapon then
			item.components.homura_weapon:SetPercent(number)
		end

		print(item)
		inst.components.container:GiveItem(item)
	end
end

---哈姆雷特小偷
AddComponentPostInit('banditmanager', function(banditmanager)
    local old_get = banditmanager.GetLoot
    function banditmanager:GetLoot(...)
        local loots = old_get(self, ...)
        if GetPlayer():HasTag('homura') and math.random() < 1.33 then
        	local special_loot = PickHomuraLoot(GetTableSize(loots))
       		for prefabname, init in pairs(special_loot)do
       			loots[prefabname] = math.max(1, init())
       		end
        end
        return loots
    end
end)

--远古箱子/ 迷宫区箱子/ 海妖箱子
AddComponentPostInit('scenariorunner', function(scenariorunner)
	local old_run = scenariorunner.Run
	function scenariorunner:Run(...)
		local hasrunonce = false --储存当前状态
		if self.hasrunonce then
			hasrunonce = true
		end
		old_run(self, ...)
		if hasrunonce or not GetPlayer():HasTag('homura') then
			return
		end
		if self.inst.prefab == 'minotaurchest' and self.scriptname == 'chest_minotaur' then
			AddHomuraLoot(self.inst, 1)
			AddHomuraLoot(self.inst, 1)
		elseif self.inst.prefab == 'pandoraschest' and self.scriptname == 'chest_labyrinth' then
			AddHomuraLoot(self.inst, 0.33)
		elseif self.inst.prefab == 'krakenchest' and self.scriptname == 'chest_kraken' then
			AddHomuraLoot(self.inst, 1)
			AddHomuraLoot(self.inst, 1)
		end
	end
end)

--海难藏宝图
AddSimPostInit(function()
	if not rawget(_G, 'SpawnTreasureChest') then
		return
	end
	local old_spawn = GLOBAL.SpawnTreasureChest
	function GLOBAL.SpawnTreasureChest(...)
		local r = {old_spawn(...)}
		local maxGUID = 0  --在大部分情况下, 后生成的实体的GUID值更大
		local inst = nil
		for k, v in pairs(Ents)do
			if k > maxGUID and v:IsValid() and v.prefab == 'treasurechest' and v.components.container then
				maxGUID = k
				inst = v
			end
		end
		if inst then
			AddHomuraLoot(inst, 0.33)
		end
		return unpack(r)
	end
end)

--猪人武器商店
local pig_shop_weapons_homura_weight = {
    { "homura_gun_ammo1",       "oinc", 5  ,3}, 	--20
    { "homura_gun_ammo2",       "oinc", 10 ,2},		--40
    { "homura_pistol",        	"oinc", 50 ,2},
    { "homura_gun",     		"oinc", 100,1},
    { "homura_hmg",      		"oinc", 100,1},
    { "gunpowder",              "oinc", 5  ,3},
    { "homura_rpg",				"oinc", 200,1},
    { "homura_rpg_ammo1",		"oinc", 20, 1},
    { "homura_rpg_ammo2",		"oinc", 50, 1},
}
local pig_shop_weapons_homura = {}
for k,v in pairs(pig_shop_weapons_homura_weight)do
	for i = 1, v[4] or 1 do
		table.insert(pig_shop_weapons_homura, v)
	end
end
AddComponentPostInit('shopinterior', function(self)
	local old_get = self.GetNewProduct
	function self:GetNewProduct(shoptype, ...)
		local product = old_get(self, shoptype, ...)
		if GetPlayer():HasTag('homura') and math.random() < 0.2 and shoptype == 'pig_shop_weapons' then
			product = GetRandomItem(pig_shop_weapons_homura)
			if math.random() < 0.01 then
				product = {'homura_pistol_sn', 'oinc', 1}
			end
		end
		return product
	end
end)

AddPrefabPostInit('homura_gun_ammo1', function(inst)
	inst.OnBought = function(inst)
		inst.components.stackable:SetStackSize(20)
	end
end)
AddPrefabPostInit('homura_gun_ammo2', function(inst)
	inst.OnBought = function(inst)
		inst.components.stackable:SetStackSize(40)
	end
end)

GLOBAL.PUELLA_MOD_FN.AddModItem('homura_gun_ammo1','homura_gun_ammo2','homura_gun_ammo3')
GLOBAL.PUELLA_MOD_FN.AddModItem('homura_rpg_ammo1','homura_rpg_ammo2')
GLOBAL.PUELLA_MOD_FN.AddModItem('homura_rpg','homura_pistol','homura_gun','homura_hmg')
--图标修复
AddPrefabPostInitAny(function(inst)
	if inst:HasTag('shop_pedestal') or (inst:HasTag("NOCLICK")and inst:HasTag("wallsection")and inst:HasTag("furniture"))then
		local old_setimage = inst.SetImage
		if old_setimage then
			inst.SetImage = function(inst, ent, slot, ...) 
				slot = slot or 'SWAP_SIGN'
				old_setimage(inst, ent, slot, ...)
				if ent and GLOBAL.PUELLA_MOD_FN.IsModItem(ent.prefab) then
					local texname = ent.prefab
					inst.AnimState:ClearOverrideSymbol(slot)
					inst.AnimState:OverrideSymbol(slot, 
						resolvefilepath('images/inventoryimages/'..texname..'.xml'), texname..'.tex')
				end
			end
		end

		local old_setfromname = inst.SetImageFromName
		if old_setfromname then
			inst.SetImageFromName = function(inst, name, slot, ...)
				slot = slot or 'SWAP_SIGN'
				old_setfromname(inst, name, slot, ...)
				if name and GLOBAL.PUELLA_MOD_FN.IsModItem(name) then
					local texname = name
					inst.AnimState:ClearOverrideSymbol(slot)
					inst.AnimState:OverrideSymbol(slot, 
						resolvefilepath('images/inventoryimages/'..texname..'.xml'), texname..'.tex')
				end
			end
		end

		local old_load = inst.OnLoad
		if old_load then
			inst.OnLoad = function(inst, data)
				old_load(inst, data)
				if data.imagename then
            		inst:SetImageFromName(data.imagename)
            	end
            end
        end
	end
end)

AddComponentPostInit('shopper', function(self)
	local old_IsWatching = self.IsWatching
	function self:IsWatching(...)
		if GetWorld():HasTag('homuraTag_pause') then
			return false
		else
			return old_IsWatching(self, ...)
		end
	end
end)

