local HomuraProjectile = Class(function(self, inst)
    self.inst = inst

    ---击中后有几率miss
    local old_hit = inst.components.projectile.Hit
    function inst.components.projectile:Hit(target)
        local current = self.inst:GetPosition()
        local coveredDistsq = distsq(self.start, current) --飞行距离
        local missrate = self.inst.components.homura_projectile:CalcMissRate(coveredDistsq)
        if math.random() < missrate then
            self:Miss(target)
        else
            old_hit(self,target)
        end
    end

    inst:ListenForEvent('onremove',function()self:SpawnTornado()end)
end)

function HomuraProjectile:CalcMissRate(distsq)
    --当距离小于必中射程时, 返回0
    --当距离过大时, 子弹偏角越大, 命中率越低
    if distsq < self.hitrange * self.hitrange then
        return 0
    else
        return 1-Remap(math.abs(self.projectiletilt),0,self.variety,1,self.min_hit_rate)
    end
end


function HomuraProjectile:SpawnTornado()
    if self.wind_sp then  --击中后几率召唤龙卷风
        local rate = 0 
        if self.wind_sp > 2 then
            rate = 0.06
        elseif self.wind_sp > 1 then
            rate = 0.03
        end
        if math.random() <= rate then
            local tornado = SpawnPrefab("tornado")
            tornado.WINDSTAFF_CASTER = self.owner or GetPlayer()
            local spawnPos = self.inst:GetPosition() + TheCamera:GetDownVec()
            local totalRadius = tornado.Physics:GetRadius() + 1
            local targetPos = self.inst:GetPosition() + (TheCamera:GetDownVec() * totalRadius)
            tornado.AnimState:SetTime(10/30)
            tornado.Transform:SetPosition(spawnPos:Get())
            tornado.components.knownlocations:RememberLocation("target", targetPos)
        end
    end
end

local function IsDead(inst)
    return inst and inst.components.health and inst.components.health.currenthealth <= 0
end
--原来的组件IsDead函数已被修改, 所以这里直接用定义来判定

function HomuraProjectile:OnHit(inst, owner, target)
    if not (target and target.components.health) then
        return
    end
    --消音
    if self.silent then
        owner:AddTag('homura: has silencer')
    end

    if not IsDead(target) then --物理伤害在这里给, 而不是直接通过武器组件, 以防触发坎普斯bug
        target.components.combat:GetAttacked(owner, self.hitdamage, inst)
    end
    
    if self.magic then --魔法伤害(真伤)
        owner:AddTag('canignorearmor')
        if not IsDead(target) then
            target.components.combat:GetAttacked(owner, self.magic_sp and 20 or 10, inst)
        end
        owner:RemoveTag('canignorearmor')
    end

    if self.inst.prefab == 'homura_projectile3' then --起源弹附带的对魔女100%穿透伤害
        if target:HasTag('witch') and not IsDead(target) then
            target.components.combat:GetAttacked(owner, 30/0.85, inst)
        end
    end
    
    if self.ice then
        if target.components.freezable and not IsDead(target) then
            target.components.freezable:AddColdness(self.ice_sp and 0.34 or 0.1)
        end
    end

    owner:RemoveTag("homura: has silencer")
end

function HomuraProjectile:OnTrow(inst, owner, target)
    local buffed_data = owner.components.homura_weapon.buffed_data
    self.hitrange = buffed_data.hitrange
    self.maxrange = owner.components.weapon.hitrange
    self.min_hit_rate = buffed_data.min_hit_rate
    self.variety = buffed_data.variety
    self.hitdamage = buffed_data.damage1 --伤害
    self.speed = buffed_data.speed
    self.magic = buffed_data.magic
    self.time = buffed_data.time 
    self.homing = buffed_data.homing
    self.ice = buffed_data.ice
    self.silent = buffed_data.silent

    self.magic_sp = buffed_data.magic_sp
    self.time_sp = buffed_data.time_sp
    self.wind_sp = buffed_data.wind_sp
    self.ice_sp = buffed_data.ice_sp

    self.owner = owner

    local projectile = inst.components.projectile
    --重定向
    if self.variety then
        local angle = inst:GetRotation()
        self.projectiletilt = GetRandomMinMax(-self.variety, self.variety) --子弹偏角线性分布
        inst.Transform:SetRotation(angle + self.projectiletilt)
    end
    --重设速度
    if self.speed then
        inst.Physics:SetMotorVel(self.speed,0,0)
    end
    --重设距离
    if self.maxrange and not self.infinite_range then
        projectile:SetRange(self.maxrange+1)
    end
    --贴图
    if self.ice then
        inst.AnimState:PlayAnimation('anim_ice')
    elseif self.ice_sp then
        inst.AnimState:PlayAnimation('anim_ice') --@--
    end
    if self.magic_sp then
        inst.AnimState:PlayAnimation('anim_magic_sp')
        inst.AnimState:SetBloomEffectHandle("shaders/anim.ksh")
        inst.AnimState:SetLightOverride(1)
        if inst.updatetailfn then
            inst:updatetailfn()
        end
    elseif self.magic then
        inst.AnimState:PlayAnimation('anim_magic')
        inst.AnimState:SetBloomEffectHandle("shaders/anim.ksh")
        inst.AnimState:SetLightOverride(1)
    end
    --免疫时停
    if self.time_sp then
        inst:AddTag('ignoretimemagic')
    elseif self.time then
        inst.components.ignoretimemagic:AddResistance(math.random(2,3))
    end
    --追踪
    if self.homing then
        projectile:SetRange(50) 
        local maxturningangle = 15
        local rate = 0.5
        local function Homing(pro)
            local current = inst:GetPosition()
            local dest = inst.components.projectile.target and inst.components.projectile.target:GetPosition()
            if not dest then return end
            local destangle = inst:GetAngleToPoint(dest)
            local oldangle = inst.Transform:GetRotation()
            local deltaangle = PUELLA_MOD_FN.DeltaAngle(oldangle,destangle) --0~360

            maxturningangle = maxturningangle + 1
            rate = rate + 0.005
            if deltaangle < 180 then
                deltaangle = math.min(maxturningangle,deltaangle * rate)
            elseif deltaangle >= 180 then
                deltaangle = math.max(-maxturningangle,(deltaangle-360) * rate)
            end
            --print(deltaangle)
            inst.Transform:SetRotation(oldangle+deltaangle)
        end
        inst:DoPeriodicTask(1/10, Homing)

        inst:DoTaskInTime(10, inst.Remove)
    end
end

return HomuraProjectile

