

local function GiveDefaultWeapon(inst,self)
    if self.state < 3 then
        if self.checkweapontask then
            self.checkweapontask:Cancel()
            self.checkweapontask = nil
            return
        end
    end
    local t = {
        kyouko = "kyouko_spear",
        sayaka = "sayaka_sword",
        homura_1 = "homura_magicwave_launcher",
    }
    if not inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) then
        local weapon = SpawnPrefab(t[inst.prefab])
        inst.components.inventory:Equip(weapon)
    end
end

local Charge = 
{
    kyouko = function(inst)
        inst.components.health:SetAbsorptionAmount(.3)

        local cloth = SpawnPrefab("kyouko_clo")
        inst.components.inventory:Equip(cloth)
        local spear = SpawnPrefab("kyouko_spear")
        inst.components.inventory:Equip(spear)
        inst.components.inventory:DropActiveItem() 
    end,

    sayaka = function(inst)
        inst.components.locomotor:AddSpeedModifier_Mult("CHARGED", .05)
        inst.components.health:SetAbsorptionAmount(.3)

        local cloth = SpawnPrefab("sayaka_clo")
        inst.components.inventory:Equip(cloth)
        local sword = SpawnPrefab("sayaka_sword")
        inst.components.inventory:Equip(sword)
        inst.components.inventory:DropActiveItem() 
        
    end,

    homura_1 = function(inst)
        inst.components.health:SetAbsorptionAmount(.1)

        local cloth = SpawnPrefab("homura_clo")
        inst.components.inventory:Equip(cloth)
        local weapon = SpawnPrefab("homura_magicwave_launcher")
        inst.components.inventory:Equip(weapon)
        inst.components.inventory:DropActiveItem() 
    end,
}
local function StandardChargeFn(self, nofx)
    local inst = self.inst
    local name = inst.prefab

    self.Light:ReturnToScene()
    self.state = 2

    inst.SoundEmitter:PlaySound("dontstarve/common/staff_star_create")
    inst.components.health:SetInvincible(true)
    inst.AnimState:SetLightOverride(0.6)

    inst:StartThread(function ()  
        Sleep(0)
        --inst.AnimState:PlayAnimation("research")  
        inst.components.magicalcharge.state = 3 
        inst.AnimState:SetBuild(name .. "_c")

        inst:PushEvent("charge")
        
        inst.components.soulgem:AddBurnRateModifier("CHARGED", 2)
        inst.components.health:SetInvincible(false)

        if nofx == nil then   
            local fx = SpawnPrefab("magicalboom_fx")
            fx.Transform:SetPosition((inst:GetPosition()+Vector3(0,.5,0)):Get())
            fx.Transform:SetScale(3,3,3)
            fx.AnimState:SetMultColour(inst.themecolour[1],inst.themecolour[2],inst.themecolour[3],inst.themecolour[4])
        end

        Charge[name](inst)
        
        if not self.checkweapontask then
            self.checkweapontask = inst:DoPeriodicTask(.199,GiveDefaultWeapon,nil,self)
        end
        --inst:Show()
        return true
    end)
    
end


local Uncharge = 
{
    kyouko = function(inst)
        if inst.components.skill_kyouko.shield then
            inst.components.skill_kyouko.shield:fadefn()
        end
    end,
    sayaka = function(inst)

    end,
    homura_1 = function(inst)
    end,
}

local function StardardUnchargeFn(self, nofx)
    local inst = self.inst
    local name = inst.prefab
    inst:PushEvent("uncharge")
    --inst.Light:Enable(false)
    --inst:Show()
    self.state = 1
    self.Light:RemoveFromScene()

    inst.AnimState:SetBuild(name)
    inst.AnimState:SetLightOverride(0)

    inst.components.locomotor:RemoveSpeedModifier_Mult("CHARGED")
    inst.components.soulgem:RemoveBurnRateModifier("CHARGED")
    inst.components.health:SetAbsorptionAmount(0)
    inst.components.health:SetInvincible(false)

    for k,v in pairs(inst.components.inventory.equipslots)do
        if v:HasTag("magicweapon")then
            v.components.equippable:Unequip(inst)
        end
    end

    if nofx == nil then
        local fx = SpawnPrefab("collapse_small")
        fx.Transform:SetPosition(inst:GetPosition():Get())
        fx.Transform:SetScale(1,1,1)
    end

    Uncharge[name](inst)
    
end


-------------------------------------------------
local magicalcharge = Class(function(self, inst)
    
    inst:ListenForEvent("cantequip", function(_, data)
        local line = data and data.olditem and data.olditem.forceequipstr
        if line then
            inst.components.talker:Say(line[math.random(#line)])
        end
    end)

    self.inst = inst
    self.state = 1

    self.Light = CreateEntity()
    self.Light:AddTag('FX','NOCLICK','NOBLOCK')
    self.Light.entity:AddTransform()
    self.inst:AddChild(self.Light)
    
    local light = self.Light.entity:AddLight()
    light:SetColour(unpack(inst.themecolour))
    light:SetRadius(1.5)
    light:SetIntensity(.75)
    light:SetFalloff( 0.5 )
    light:Enable(false)


    self.chargefn = StandardChargeFn
    self.unchargefn = StardardUnchargeFn
end)

function magicalcharge:OnTrigger()
    if self.state == 1 then
        self:TryCharge()
    end
    if self.state == 3 then
        self:TryUncharge()
    end
end

function magicalcharge:OnSave() 
   return {state = self.state} 
end

function magicalcharge:OnLoad(data)
    self.state = data.state
    if self.state ~= 1 then
        self:chargefn(true)
    end
end

function magicalcharge:TryCharge()
    if self:CanCharge() then
        self:SetCharge()
    end
end

function magicalcharge:TryUncharge()
    if self:CanUncharge() then
        self:SetUncharge()
    else
        local player = self.inst.prefab
        local BATTLE = PUELLA_MOD_STRINGS:Get(player,"FORCECHARGE")
        self.inst.components.talker:Say(BATTLE)
    end
end

function magicalcharge:SetCharge()
    self:chargefn()
end

function magicalcharge:SetUncharge()
    self:unchargefn()
end

function magicalcharge:CanCharge()
    return not self.inst.XXX 
end

function magicalcharge:CanUncharge()
    if self.inst:HasTag("puella_skill") then 
        return false 
    end
    if self.inst.XXX then --活体机械
        return false
    end
    do 
        return true 
    end
end  

return magicalcharge
