local MagicBallManager = Class(function(self, inst)
    self.inst = inst

    self.radius = 1
    self.angle = 0
    self.base_angle = 0
    self.speed = 30 --旋转一周的秒数
    self.clockwise = true
    self.circle = true

    self.behavior = {constant=true}
    
    self.inst:StartUpdatingComponent(self)
    self.speed_start = 0 self.speed_end = 0 self.radius_start = 0 self.radius_end = 0

end)

function MagicBallManager:SpeedTo(targspeed,time,delay,offset)
    self.targspeed = targspeed
    self.oldspeed = self.speed
    self.speed_start = GetTime()+(delay or 0)
    self.speed_end = GetTime()+(delay or 0)+(time or 3)
    self.behavior.speedup = true
    self.behavior.constant = false
    ----------------------------------偏移修正
    local deltaspeed = 6.283/targspeed - 6.283/self.oldspeed
    local timedelta = offset or 0    --相邻两颗球加速的时间差
    local offsetvalue = deltaspeed * timedelta * 1 --顺时针
    return offsetvalue
end

function MagicBallManager:RadiusTo(targradius,time,delay)
    self.targradius = targradius
    self.oldradius = self.radius
    self.radius_start = GetTime()+(delay or 0)
    self.radius_end = GetTime()+(delay or 0)+(time or 3)
    self.behavior.radiusup = true
    self.behavior.constant = false
end

function MagicBallManager:UpdateSpeed(dt)
    if self.speed_start > GetTime() then
        return
    end
    if self.speed_end < GetTime() then
        self.speed = self.targspeed
        self.behavior.speedup = false
        return
    end
    self.speed = Remap(GetTime(),self.speed_start,self.speed_end,self.oldspeed,self.targspeed)
end

function MagicBallManager:UpdateRadius(dt)
    if self.radius_start > GetTime() then
        return
    end
    if self.radius_end < GetTime() then
        self.radius = self.targradius
        self.behavior.radiusup = false
        return
    end
    self.radius = Remap(GetTime(),self.radius_start,self.radius_end,self.oldradius,self.targradius)
end

function MagicBallManager:CheckCanShoot()
    if not self.behavior.constant then
        return
    end
    local center, targ = self.inst.center
    if not center then
        return 'true'
    elseif center.sg:HasStateTag('teleport') then
        return false
    else
        targ = center.components.combat.target
    end
    if not targ then
        return 'true'
    end
    
    local dist = targ:GetPosition():Dist(self.inst:GetPosition())
    if  dist <= 3 then --站在圆内的情况
        return true
    elseif dist >= 40 then
        return false
    end
    local targangle = self.inst:GetAngleToPoint(targ:GetPosition():Get())    
    local faceangle = self.inst.Transform:GetRotation()
    local delta = math.abs(math.mod((targangle - faceangle),360))
    if delta < 10 then
        return true
    end
end

function MagicBallManager:Launch()
    self.launchspeed = self:GetCircleSpeed()
    self.circle = false
    self.inst.center.components.tv_ballspawner.minion[self.inst]=nil
    self.inst.components.acceleratingprojectile:Launch(self.inst.center,self.inst.center.components.combat.target)
    if self.inst.sound then
        self.inst.SoundEmitter:PlaySound(self.inst.sound, nil, 0.7)
    end
    self.inst:DoTaskInTime(10,function()
        self.inst:Remove()--防止游戏卡顿
        self.inst:PushEvent("forceremove")
    end) 
end

function MagicBallManager:ForceLaunch()
    self.launchspeed = self:GetCircleSpeed()
    self.circle = false
    self.inst.center.components.tv_ballspawner.minion[self.inst]=nil
    self.inst.components.acceleratingprojectile:StartUpdate()
    self.inst.Physics:SetMotorVel(self.launchspeed,0,0)
    self.inst:DoTaskInTime(10,function()
        self.inst:Remove()
        self.inst:PushEvent("forceremove")
    end)
end
--[[
function MagicBallManager:OnRemoveEntity() --由于玩家远离而移除魔法球时, 计数器-1
    if self.inst.center and self.inst.center.components.tv_ballspawner then
        self.inst.center.components.tv_ballspawner.minion[self.inst] = nil
    end
end
--]]

function MagicBallManager:SetBaseAngleRate(rate)
    self.angle = rate*6.283
    self.base_angle_rate = rate
end

function MagicBallManager:Hide()
    self.inst:Hide()
    self.hiden = true
end

function MagicBallManager:Show()
    self.inst:Show()
    self.inst:PushEvent("startperiodictask")
    self.hiden = false
end

function MagicBallManager:StartFocusing()
    self.focusing = true
end

function MagicBallManager:Initialize(r,speed,base_angle,clockwise)
    self.radius = r or self.radius
    self.speed = speed or self.speed
    self.base_angle = base_angle or self.base_angle
    self.clockwise = clockwise or self.clockwise
end

function MagicBallManager:Rotate(dt)
    local old_pos = self.inst:GetPosition()
    local angledelta = 2*PI*dt / self.speed
    self.angle = self.angle + angledelta * (self.clockwise and -1 or 1)
    local pos = self.inst.center:GetPosition()
    local new_pos = pos + Vector3(math.cos(self.angle)*self.radius,0,math.sin(self.angle)*self.radius)
    self.inst.Transform:SetPosition(new_pos.x,old_pos.y,new_pos.z)
    local faceangle = math.atan2(old_pos.z - new_pos.z,new_pos.x - old_pos.x) /DEGREES
    self.inst.Transform:SetRotation(faceangle)
end

function MagicBallManager:OnUpdate(dt)
    if self.inst:HasTag('homuraTag_pause') then
        return
    end
    if self.behavior.radiusup then
        self:UpdateRadius(dt)
    end
    if self.behavior.speedup then
        self:UpdateSpeed(dt)
    end
    if not self.behavior.radiusup and not self.behavior.speedup then
        self.behavior.constant = true
    end
    if self.focusing then
        if self:CheckCanShoot() and self.circle == true then
            if self:CheckCanShoot() == true then
                self:Launch()
            else
                self:ForceLaunch()
            end
        end
    end
    if self.circle then
        self:Rotate(dt)
    end
end

function MagicBallManager:lw_timemagic_onupdate(dt) --时停补偿
    self.radius_start = self.radius_start+dt
    self.radius_end = self.radius_end+dt
    self.speed_start = self.speed_start+dt
    self.speed_end = self.speed_end+dt
end

function MagicBallManager:GetCircleSpeed()
    if self.circle then
        return 2*PI / self.speed * self.radius
    end
    return self.launchspeed
end

return MagicBallManager
