local bigcircleR = 16

local function GetPhase()
    return GetWorld():IsCave() and 'night' or GetClock():GetPhase()
end

local function CountChild(t)
    local num = 0
    for k,v in pairs(t)do
        if v and type(k)=='table' and k.IsValid and k:IsValid() then
            num = num+1
        end
    end
    return num
end

local sd_shadowspawner = Class(function(self, inst)
	
    self.inst = inst
    
    self.inst:ListenForEvent('dusktime', function()self:OnPhaseChanged()end,GetWorld())
    self.inst:ListenForEvent('daytime' , function()self:OnPhaseChanged()end,GetWorld())
    self.inst:ListenForEvent('nighttime',function()self:OnPhaseChanged()end,GetWorld())
    self.inst:DoTaskInTime(0,function()self:OnPhaseChanged()end)
    self.inst:ListenForEvent('onremove',function()self:KillAllMinions()end)

    self.bigtentacle = {canspawn=true,lasttime=0,period=15,max=18,ID={},}
    self.midtentacle = {canspawn=true,lasttime=0,period=10,max=4,dist=2}
    self.smalltentacle = {canspawn=true,lasttime=0,max=10}
    self.smallroot = {canspawn=true,lasttime=0,period=3,}
    self.skittish = {canspawn=true,lasttime=0,period=10,max=5,dist=2,}

    self.enabled = true
    self.reenabled_delay = 0

    self.inst:StartUpdatingComponent(self)

    self.inst:DoPeriodicTask(1,function()
        local skittishnum = CountChild(self.skittish)
        if not inst.components.health:IsDead() then
            inst.components.health:DoDelta(skittishnum*3)
            --print('heal: '..tostring(skittishnum*5))
        end
    end)
end)
function sd_shadowspawner:KillAllMinions()
    for _,v in pairs({"bigtentacle","midtentacle","smalltentacle","smallroot","skittish"})do
        for k,_ in pairs(self[v])do
            if type(k)== 'table' and k.components and k.components.health and not k.components.health:IsDead() then
                k:DoTaskInTime(math.random(),function()k.components.health:Kill("nopower") end) 
            end
        end
    end
end

local function CheckTile(pos)
    local tile = GetVisualTileType(pos:Get())
    for _,v in pairs({255,1,50,55,56,57,58,59,60,61,62,63,64,128}) do if v == tile then return false end end
    return true 
end

local function IsNear(pos,r,prefab)
    local ents = TheSim:FindEntities(pos.x,pos.y,pos.z,r)
    for _,v in pairs(ents)do
        if v:IsValid() and v.prefab == prefab then
            return true
        end
    end
end

function sd_shadowspawner:OnPhaseChanged()
    local phase = GetPhase()
    if phase == 'night' then
        self.bigtentacle.canspawn = true

        self.midtentacle.canspawn = true
        self.midtentacle.max = 6
        self.midtentacle.period = 10

        self.smallroot.canspawn = true
        self.smallroot.period = 6

        self.skittish.canspawn = true
        self.skittish.max = 5
        self.skittish.aspawnmax = 1
        self.skittish.minr = nil
        self.skittish.period = 20

    elseif phase == 'dusk' then
        self.bigtentacle.canspawn = true

        self.midtentacle.canspawn = true
        self.midtentacle.max = 4
        self.midtentacle.period = 10

        self.smallroot.canspawn = true
        self.smallroot.period = 8

        self.skittish.canspawn = true
        self.skittish.max = 7
        self.skittish.aspawnmax = 2
        self.skittish.minr = nil
        self.skittish.period = 12
        
    elseif phase == 'day' then
        self.bigtentacle.canspawn = false
        self.midtentacle.canspawn = false

        self.smallroot.canspawn = true
        self.smallroot.period = 10

        self.skittish.canspawn = true
        self.skittish.period = 5
        self.skittish.max = 12
        self.skittish.minr = bigcircleR/3
        self.skittish.aspawnmax = 3
        self.skittish.period = 10
    end
end

function sd_shadowspawner:SummonBigTentacleRing(center) --默认圆心:塔
    local pos = self.inst:GetPosition()
    if self.inst.tower then
        pos = self.inst.tower:GetPosition()
    end
    if center then
        pos = center
    end

    local r = bigcircleR --半径
    local num = self.bigtentacle.max --数量
    for i = 1,num do
        local theta = i*360/num*DEGREES
        local spawnpos = pos + Vector3(math.cos(theta)*r,0,math.sin(theta)*r)
        self.inst:DoTaskInTime(2*math.random(),function()
            self:SpawnBigTentacle(spawnpos,i)
        end)
    end
    self.bigtentacle.lasttime = GetTime()
end

function sd_shadowspawner:SpawnBigTentacle(pos,i) --单个海妖触手
    if CheckTile(pos) and self.bigtentacle.canspawn and self.enabled then
        if self.bigtentacle.ID[i]then
            return
        end
        local tentacle = SpawnPrefab('sd_tentacle_big')
        tentacle.Transform:SetPosition(pos:Get())
        tentacle.sg:GoToState('spawn')
        self.bigtentacle[tentacle] = true
        self.bigtentacle.ID[i] = true

        self.inst:ListenForEvent('onremove',function()
            self.bigtentacle[tentacle]=nil 
            self.bigtentacle.ID[i]=nil
        end, tentacle)
    end
end

function sd_shadowspawner:SpawnMidTentacle(pos) --单个触手
    if CheckTile(pos) and self.midtentacle.canspawn and self.enabled then
        local tentacle = SpawnPrefab('sd_tentacle_mid')
        tentacle.Transform:SetPosition(pos:Get())
        tentacle.sg:GoToState('attack_pre')
        self.midtentacle[tentacle] = true
        self.midtentacle.lasttime = GetTime()

        self.inst:ListenForEvent('onremove',function()self.midtentacle[tentacle]=nil end, tentacle)
    end
end

function sd_shadowspawner:SpawnRoot(pos) --攻击性根系,pos不使用
    --if not CheckTile(pos) then
    --    return
    --end
    local target = self.inst.components.combat.target
    if not target then 
        return
    end
    local rootpos = target:GetPosition()
    if not CheckTile(rootpos) or not self.enabled or not self.smallroot.canspawn then
        return
    end
    local root = SpawnPrefab('sd_attackroot')
    local instpos = self.inst:GetPosition()
    local angle = self.inst:GetAngleToPoint(rootpos)*DEGREES
    local targdist = 10 --最远攻击距离
    if rootpos:Dist(instpos) > targdist then
        rootpos = instpos + Vector3(math.cos(angle) * targdist, 0, -math.sin(angle) * targdist)
    end
    local offset = Vector3(math.cos(angle) * 1.75, 0, -math.sin(angle) * 1.75)
    root.Transform:SetPosition((instpos+offset):Get())
    root:PushEvent("givetarget", {target=target, targetpos=rootpos, targetangle=angle, owner=self.inst})

    self.smallroot[root] = true
    self.smallroot.lasttime = GetTime()

    --self.inst:ListenForEvent("onremove",function()end)
end

function sd_shadowspawner:SpawnSmallTentacle(pos) --小触手 (sg比较奇怪,暂不使用)
    if CheckTile(pos) and self.smalltentacle.canspawn and self.enabled then
        local tentacle = SpawnPrefab('sd_tentacle_small')
        tentacle.Transform:SetPosition(pos:Get())
        --tentacle.sg:GoToState("")
        self.smalltentacle[tentacle] = true
        self.smalltentacle.lasttime = GetTime()

        self.inst:ListenForEvent('onremove',function()self.smalltentacle[tentacle]=nil end, tentacle)
    end
end

function sd_shadowspawner:SpawnSkittish(pos)
    if CheckTile(pos) and self.skittish.canspawn and self.enabled then
        local skittish = SpawnPrefab('sd_skittish')
        skittish.Transform:SetPosition(pos:Get())
        self.skittish[skittish] = true
        self.skittish.lasttime = GetTime()

        self.inst:ListenForEvent('onremove', function()self.skittish[skittish] = nil end, skittish)
    end
end

function sd_shadowspawner:TempDisable(time) --time 在魔女吟唱技能期间,暂时停止召唤/ 闪光弹额外效果
    self.enabled = false
    self.reenabled_delay = time or 1
end

function sd_shadowspawner:UpdateRoot(dt)
    --if self.smallroot.canspawn and GetTime()-self.smallroot.lasttime > 2 then
    --    self:SpawnRoot()
    --end
end

function sd_shadowspawner:UpdateTime(dt)
    if self.reenabled_delay > 0 then
        self.reenabled_delay = self.reenabled_delay-dt
        if self.reenabled_delay <= 0 then
            self.enabled = true
        end
    end
    for _,v in pairs({'smallroot','smalltentacle','midtentacle','bigtentacle','skittish'})do
    --for _,v in pairs({'midtentacle','bigtentacle'})do
    --for _,v in pairs({})do
        if self[v].period then
            if GetTime()-self[v].lasttime > self[v].period then
                if self[v].aspawnmax then
                    for i = 1,math.random(1,self[v].aspawnmax)do
                        self:TrySpawn(v)
                    end
                else
                    self:TrySpawn(v)
                end
            end
        end
    end
end

local function GetRandomPos(r,min,max)
    local theta = 6.283*math.random()
    local r = r*math.random()
    if min then
        r = GetRandomMinMax(min,r)
    end
    return Vector3(math.cos(theta)*r,0,math.sin(theta)*r)
end

function sd_shadowspawner:TrySpawn(spawntype)
    if spawntype == 'smallroot' then
        self:SpawnRoot()
    end
    if spawntype == 'midtentacle' and CountChild(self.midtentacle)<= self.midtentacle.max then
        for i = 1,10 do
            local pos = self.inst:GetPosition()+GetRandomPos(bigcircleR-1)
            if not IsNear(pos,self.midtentacle.dist,'sd_tentacle_mid')  then
                self:SpawnMidTentacle(pos)
                return
            end
        end
    end
    if spawntype == 'skittish' then
        for i = 1,10 do
            local pos = self.inst:GetPosition()+GetRandomPos(bigcircleR+4,bigcircleR/3)
            if not IsNear(pos,self.skittish.dist,'sd_skittish')then
                self:SpawnSkittish(pos)
                return
            end
        end
    end
    if spawntype == 'bigtentacle' and CountChild(self.bigtentacle)<= self.bigtentacle.max then
        self:SummonBigTentacleRing()
        self.bigtentacle.period = 15
    end

end

function sd_shadowspawner:GetRedirectTarget()
    print('Run -> GetRedirectTarget')
    for k in pairs(self.skittish)do
        if type(k)== 'table' and k.components and k.components.health and not k.components.health:IsDead() then
            print(k)
            return k
        end
    end
    for k in pairs(self.midtentacle)do
        if type(k)== 'table' and k.components and k.components.health and not k.components.health:IsDead() then
            print(k)
            return k
        end
    end
    print('return default')
    return self.inst
end

function sd_shadowspawner:OnUpdate(dt)
    self:UpdateTime(dt)
    self:UpdateRoot(dt)
end 



function sd_shadowspawner:OnSave()
   local data = {}
   for _,v in pairs({'bigtentacle','midtentacle','smalltentacle','smallroot','skittish'})do
        data[v] = {}
        for ent,vv in pairs(self[v])do
            if vv and type(ent) == 'table' and ent.IsValid and ent:IsValid() then
                data[v][ent.GUID] = true
            end
        end
    end
    return data
end

function sd_shadowspawner:OnLoad(data)

end

function sd_shadowspawner:LoadPostPass(ents, data)
    for k,v in pairs(data)do
        for guid,_ in pairs(v) do
            if ents[guid] then
                self[k][ents[guid]] = true
            end
        end
    end
end

function sd_shadowspawner:GetDebugString()
    local str = ''
    str = str..'\nroot:  '..tostring(self.smallroot.canspawn)..'  '
    str = str..'\nbig:  '..tostring(self.bigtentacle.canspawn)..'  '..self.bigtentacle.period..' /has child:'..CountChild(self.bigtentacle)
    str = str..'\nmid:  '..tostring(self.midtentacle.canspawn)..'  '..self.midtentacle.period

    return str
end

return sd_shadowspawner
