local soulgem = Class(function(self, inst)
    self.inst = inst
    self.max = 200
    self.current = 0

    self.basicrate =  8  --每天（480秒）默认消耗值

    self.rate = 1
    
    self.burning = true
    
    self.burn_rate_modifiers = {}
    self.period =  5
    
    self.task = self.inst:DoPeriodicTask(self.period, function() self:DoDec(self.period) end) 

    self.inst:ListenForEvent('soulgemdelta',function(inst,data)self:OnDelta(data)end)

end)
function soulgem:OnDelta(data)
    if data and data.newpercent >= 0.9 and data.oldpercent < 0.9 then
        if not self.inst.components.health:IsDead() then
            self.inst.components.talker:Say(PUELLA_MOD_STRINGS:Get(self.inst.prefab,"SOULGEM_WARNING"))
        end
    end
end


function soulgem:AddBurnRateModifier(key, mod)
    self.burn_rate_modifiers[key] = mod
end

function soulgem:RemoveBurnRateModifier(key)
    self.burn_rate_modifiers[key] = nil
end

function soulgem:GetBurnRate()
    local mod = 1
    for k,v in pairs(self.burn_rate_modifiers) do
        mod = mod + v
    end
    return mod
end

function soulgem:OnSave() 
    return {current = self.current}
end

function soulgem:OnLoad(data)
    if data.current then
        self.current = data.current
        self:DoDelta(0)
    end
end

function soulgem:LongUpdate(dt)
	self:DoDec(dt, true)
end

function soulgem:Pause()
    self.burning = false
end

function soulgem:Resume()
    self.burning = true
end

function soulgem:SetMax(amount)
    self.max = amount
end


function soulgem:IsDark() 
    return self.current == self.max
end

function soulgem:CanRevive()
    return (self.max - self.current)>5  
end

function soulgem:CanLock(num)
   return (self.max - self.current)>num 
end

function soulgem:DoRevive(delta)
    if self.max - self.current - delta > 5 then
        self:DoDelta(delta,true)
    else
        self.current = self.max - 5
        self:DoDelta(0,true)
    end
end


function soulgem:DoDelta(delta, overtime)
    local old = self.current
    self.current = self.current + delta

    if self.current <= 0 and old > 0 then 
        if self.ondark then 
            self.ondark(self.inst)
        end 
    end 

    if self.current < 0 then 
        self.current = 0
    elseif self.current > self.max then
        self.current = self.max
    end
    
    self.inst:PushEvent("soulgemdelta", {oldpercent = old/self.max, newpercent = self.current/self.max, overtime = overtime})
end

function soulgem:GetPercent()
    return self.current / self.max
end

function soulgem:GetVigIntensity()
    local p = self:GetPercent()
    return math.max(0,p*5-4)
end

function soulgem:SetPercent(p)
    local old = self.current
    self.current  = p*self.max
    self.inst:PushEvent("soulgemdelta", {oldpercent = old/self.max, newpercent = p})
end

function soulgem:DoDec(dt)  
    local delta = self.basicrate / 480 * dt
    if self.burning then self:DoDelta(self:GetBurnRate()*delta*self.rate, true) end
end

function soulgem:LongUpdate(dt)
    self:DoDec(dt)
end

function soulgem:Purify(item)
    if item.components.finiteuses then
        item.components.finiteuses:Use(1) 
    else 
        --item.components.stackable
    end
    self:DoDelta(item.components.purifier.value,false)
    if item.components.purifier.onusedfn then
        item.components.purifier:onusedfn(self.inst)
    end        
    --self.inst.SoundEmitter:PlaySound("dontstarve/common/staff_star_dissassemble","busy") 
end

return soulgem
