local balls = 
{
    "witch_tv_red",
    "witch_tv_blue",
    "witch_tv_zap",
    --"witch_tv_simply",
}
if  SaveGameIndex:IsModeShipwrecked() or SaveGameIndex:IsModePorkland() then
    table.insert(balls ,"witch_tv_green")
end

local tv_ballspawner = Class(function(self, inst)
	
    self.inst = inst
    self.timer = 0   
    self.type = 0
    self.state = "idle"
    self.idletime = 1
    self.minion = {}
    self.minionnum = 0

    self.circletime = 0
    self.summoninterval = 0

    self.minh = 1
    self.maxh = 2
    self.x = 0

    self.inst:StartUpdatingComponent(self)
end)

function tv_ballspawner:SpawnBall(num,base_angle)
    local spawntype = num
    local ball = SpawnPrefab(balls[spawntype])
    ball.center = self.inst
    ball:AddComponent("magicballmanager")
    ball.components.magicballmanager:SetBaseAngleRate(base_angle)
    ball.components.magicballmanager:Hide()
    self.minion[ball]=true
    self.inst:ListenForEvent('onremove', function()
        self.minion[ball]=nil
    end, ball)
end

function tv_ballspawner:GetIdleTime(state)
    local t = {5,3,0}
    return t[state]
end

function tv_ballspawner:StartIdle()
    self.timer = 0
    self.state = "idle"
    self.idletime = self:GetIdleTime(self.inst.state)
end

function tv_ballspawner:IdleUpdate()
    if self.timer > self.idletime and self.inst.components.combat.target then
        self:StartSummon()
    end
end

function tv_ballspawner:GetSpawnState()
    local state = self.inst.state
    if state == 1 then
        return 4,3
    elseif state == 2 then
        return 2,4
    elseif state == 3 then
        return 1,6
    end
    return 0,1
end

function tv_ballspawner:StartSummon()
    --print("summon")
    self.timer = 0
    self.havespawnednum = 0
    self.state = "summon"
    self.inst:PushEvent("attack")
    self.summoninterval,self.maxminionnum = self:GetSpawnState()
    self.type = math.random(#balls)
    for i = 0,self.maxminionnum-1 do
        self:SpawnBall(self.type,i/self.maxminionnum)
    end
end

function tv_ballspawner:SummonUpdate()
    if self.timer > self.summoninterval then
        self.timer = 0
        if self.havespawnednum < self.maxminionnum then
            local ball = self:GetNextBall()
            if not ball then
                --print("getnoball")
                self.havespawnednum = self.havespawnednum + 1
                return
            end
            ball.components.magicballmanager:RadiusTo(2.5,2,0)
            local offset = ball.components.magicballmanager:SpeedTo(6,3,1,self.summoninterval) * self.havespawnednum
            ball.components.magicballmanager.angle = ball.components.magicballmanager.angle - offset
            ball.components.magicballmanager:Show()
            self.havespawnednum = self.havespawnednum + 1
        else
            self:StartCircling()
        end
    end
end

function tv_ballspawner:GetNextBall()
    for k,_ in pairs(self.minion)do
        if k.components.magicballmanager.base_angle_rate == self.havespawnednum / self.maxminionnum then
            return k
        end
    end
end

function tv_ballspawner:GetCircleTime(state)
    local t = {8,4,0}
    return t[state]
end

function tv_ballspawner:StartCircling()
    --print("circle")
    self.timer = 0
    self.state = "attack_pre"
    self.circletime = self:GetCircleTime(self.inst.state)     
end

function tv_ballspawner:CircleUpdate(dt)
    local targ = self.inst.components.combat.target
    if targ and targ:IsValid() then
        local dist = targ:GetPosition():Dist(self.inst:GetPosition())
        if dist < 3 then
            self.circletime = self.circletime + dt * (self.inst.Angry and .2 or .5)
        end
    end
    if self.timer > self.circletime then
        self:StartFocusing()
    end
end

function tv_ballspawner:StartFocusing() 
    local teleport_rate = {0.1,0.33,1}
    if math.random() < teleport_rate[self.inst.state] then
        self:Teleport()
    end
    self.timer = 0
    self.state = "attacking"
    for k,_ in pairs(self.minion)do
        k.components.magicballmanager:SpeedTo(3,2)
        k.components.magicballmanager:StartFocusing()
    end
end

function tv_ballspawner:FocusUpdate()
    if self:GetMinionNum() == 0 then
        self:StartIdle()
    end
end

function tv_ballspawner:ForceShoot()
    if self:GetMinionNum() > 1 and self.state ~= "attacking" then
        self:StartFocusing()
    end
end

function tv_ballspawner:Teleport()
    local pos = self.inst:GetPosition()
    if self.inst:GetIsOnWater() then
        local offset = FindWaterOffset(self.inst:GetPosition(),math.random()*6.28,12,5)
        if offset then
            self.inst:PushEvent('teleport', {pos = pos+offset})
        end
    elseif self.inst:GetIsOnLand() then
        local offset = FindGroundOffset(self.inst:GetPosition(),math.random()*6.28,12,5)
        if offset then
            self.inst:PushEvent('teleport', {pos = pos+offset})
        end
    end
end

function tv_ballspawner:OnUpdate(dt)
    self.timer = self.timer + dt
    if self.state == "idle" then
        self:IdleUpdate(dt)
    elseif self.state == "summon" then
        self:SummonUpdate(dt)
    elseif self.state == "attack_pre" then
        self:CircleUpdate(dt)
    elseif self.state == "attacking" then
        self:FocusUpdate(dt)
    end
    --self:UpdateHeight()
end 

function tv_ballspawner:UpdateHeight()
    self.x = self.x + 1
    local h = math.sin(self.x * .01)*(self.maxh - self.minh)/2 + (self.minh+self.maxh)/2
    local pos = self.inst:GetPosition()
    self.inst.Transform:SetPosition(pos.x,h,pos.z)
end

function tv_ballspawner:GetMinionNum()
    local num = 0
    for k,v in pairs(self.minion)do
        num = num + 1
    end
    return num
end

function tv_ballspawner:Hide()
    for k in pairs(self.minion)do
        k:Hide()
    end
end

function tv_ballspawner:Show()
    for k in pairs(self.minion)do
        k:Show()
    end
end

function tv_ballspawner:OnSave()
    return {state=self.state, timer=self.timer, ["type"]=self.type}
end

function tv_ballspawner:OnLoad(data)
    if data then
        self.state = data.state
        self.timer = data.timer
        self.type = data.type
    end
    if self.state == "summon" then
        self:StartSummon()
    elseif self.state == "attack_pre" then
        self:StartCircling()
    elseif self.state == "attacking" then
        self:StartFocusing()
    end
end

return tv_ballspawner
