local assets =  
{ 
    Asset("ANIM", "anim/griefseed_fx.zip"),
    Asset("SOUNDPACKAGE", "sound/lw_griefseed.fev"),
    Asset("SOUND", "sound/lw_griefseed.fsb"), 
}
local function Unpack(colour )
    return colour[1],colour[2],colour[3],colour[4]
end

local function Tags(inst)
    inst:AddTag("FX")
    inst:AddTag("NOCLICK")
    inst:AddTag("ignoretimemagic")
end

local function SetAnim(inst,build,anim,loop)
    inst.AnimState:SetBank(build)
    inst.AnimState:SetBuild(build)
    inst.AnimState:PlayAnimation(anim,loop)
end

--------------------------------------------------------------------

local function CreateFraction()
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local snd = inst.entity:AddSoundEmitter()
    MakeInventoryPhysics(inst)

    SetAnim(inst,"griefseed_fx","fraction_pst")
    anim:OverrideSymbol("dfs0","griefseed_fx","dfs"..math.random(13))
    anim:SetMultColour(0,0,0,1)
    trans:SetScale(.3,.3,.2)
    Tags(inst)
    return inst
end

local function SplashFraction(pos,num)
    num = num or math.random(10,15)
    for i = 1,num do
        local item = CreateFraction()
        item.Transform:SetPosition(pos:Get())
        item.Physics:SetVel(GetRandomMinMax(-10,10),GetRandomMinMax(-3,10),GetRandomMinMax(-10,10))
    end
end

local function PushItems(pos)
    local ents = TheSim:FindEntities(pos.x,pos.y,pos.z,5,nil,{"INLIMBO"})
    for _,item in pairs(ents)do
        if item and item.Physics then
            if item.components.inventoryitem then
                item.Transform:SetPosition((item:GetPosition()+Vector3(0,.01,0)):Get()) 
                item.components.inventoryitem:OnStartFalling()
                item:AddTag("NOVACUUM")
                item:DoTaskInTime(2, function() item:RemoveTag("NOVACUUM") end)
                local speed = 8 + (math.random() * 4)
                local vec = (item:GetPosition()-pos):Normalize() 
                item.Physics:SetVel(vec.x * speed, 10, vec.z * speed)
            end  
        end
    end
end

local function CreateBlastWave(pos)
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local snd = inst.entity:AddSoundEmitter()
    trans:SetPosition(pos:Get())
    SetAnim(inst,"griefseed_fx","",true)
    Tags(inst)
    trans:SetScale(5,5,3)

    inst:ListenForEvent("animover",inst.Remove)   
    inst:DoTaskInTime(.2,function()anim:PlayAnimation("blast")end) 
    inst:DoTaskInTime(0,function()PushItems(pos)end)
    snd:PlaySound("lw_griefseed/sfx/blast",nil)
    snd:PlaySound("lw_griefseed/sfx/glass",nil,.8)
    return inst
end

local function CreateGS(player)
    local inst = CreateEntity()
    inst.player = player
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local snd = inst.entity:AddSoundEmitter()
    SetAnim(inst,"griefseed_fx","gs_idle")
    trans:SetScale(2,2,2)
    Tags(inst)
    inst.counter = 0
    inst:StartThread(function ()
        Sleep(2.1)
        anim:PlayAnimation("gs_shake")
        anim:OverrideSymbol("griefseed","griefseed_fx","griefseed1")
        SplashFraction(inst:GetPosition())
        CreateBlastWave(inst:GetPosition())
        inst.player.HUD.puellaUI_sgdark:SetAlphaMinTarget(2,true)
        Sleep(2)
        anim:PlayAnimation("gs_shake")
        anim:OverrideSymbol("griefseed","griefseed_fx","griefseed2")
        SplashFraction(inst:GetPosition())
        CreateBlastWave(inst:GetPosition())
        inst.player.HUD.puellaUI_sgdark:SetAlphaMinTarget(3,true)
        Sleep(2)
        anim:PlayAnimation("gs_shake")
        anim:OverrideSymbol("griefseed","griefseed_fx","griefseed3")
        SplashFraction(inst:GetPosition())
        CreateBlastWave(inst:GetPosition())
        inst.player.HUD.puellaUI_sgdark:SetAlphaMinTarget(4,true)
        Sleep(2)
        anim:PlayAnimation("gs_pst")
        SplashFraction(inst:GetPosition(),50)
        CreateBlastWave(inst:GetPosition())
        Sleep(1)
        return true
    end)
    inst:StartThread(function()
        Sleep(.1)
        snd:PlaySound("lw_griefseed/sfx/shine1",nil,.5)
        Sleep(1)
        snd:PlaySound("lw_griefseed/sfx/shine2",nil,.55)
        Sleep(1)
        snd:PlaySound("lw_griefseed/sfx/shine1",nil,.6)
        Sleep(1)
        snd:PlaySound("lw_griefseed/sfx/shine2",nil,.65)
        Sleep(1)
        snd:PlaySound("lw_griefseed/sfx/shine1",nil,.7)
        Sleep(1)
        snd:PlaySound("lw_griefseed/sfx/shine2",nil,.75)
        Sleep(1)
        snd:PlaySound("lw_griefseed/sfx/shine1",nil,.8)
        Sleep(1)
        snd:PlaySound("lw_griefseed/sfx/shine2",nil,.85)
        return true
    end)
    inst:DoPeriodicTask(FRAMES,function()
        inst.counter = inst.counter + 1
        local a = -0.5*math.cos(inst.counter * 3.1416 / 15)+ .5 
        anim:SetAddColour(a,a,a,1)
    end)
    return inst
end

local function fn()
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local snd = inst.entity:AddSoundEmitter()

    Tags(inst)
    inst.player = GetPlayer() --

    inst.gs = CreateGS(inst.player)
    inst.gs.entity:SetParent(inst.entity)
    inst.gs.Transform:SetPosition(0,3,0)

    return inst
end

return Prefab("common/fx/griefseed_fx",fn,assets)
--,Prefab("c",fn,assets)