local assets= 
{
    Asset("ANIM", "anim/homura_bomb.zip"),
    Asset("ATLAS","images/inventoryimages/homura_bomb_bomb.xml"),
    Asset("ATLAS","images/inventoryimages/homura_bomb_flash.xml"),
    Asset("ATLAS","images/inventoryimages/homura_bomb_fire.xml"),
}
--  air 空中动画
--  swap_XXX手持通道
--  

local function quickset(prefab,name,desc_data,recipe)
    STRINGS.NAMES[prefab:upper()] = name 
    for cha,desc in pairs(desc_data)do
        STRINGS.CHARACTERS[cha:upper()].DESCRIBE[prefab:upper()] = desc
    end
    STRINGS.RECIPE_DESC[prefab:upper()] = recipe
end

if PUELLA_MOD_SETTINGS.LANGUAGE then
    quickset("HOMURA_BOMB_FLASH","flash bomb",
        {
            GENERIC = "flash bomb",
            --HOMURA_1 = "",
            --KYOUKO = "",
        },"")

    quickset("HOMURA_BOMB_FIRE","fire bomb",
        {
            GENERIC = "fire bomb",
            --HOMURA_1 = "",
            --KYOUKO = "",
        },"")

    quickset("HOMURA_BOMB_BOMB","bomb",
        {
            GENERIC = "bomb",
            --HOMURA_1 = "",
            --KYOUKO = "",
        },"")
else
    quickset("HOMURA_BOMB_FLASH","闪光弹",
        {
            GENERIC = "闪光弹",
            HOMURA_1 = "利用镁的燃烧来释放强光。",
            KYOUKO = "那个晓美焰总是喜欢用这些奇奇怪怪的玩意儿。",
            SAYAKA = "",
        },"放出刺眼的强光.")

    quickset("HOMURA_BOMB_FIRE","燃烧弹",
        {
            GENERIC = "燃烧弹",
            HOMURA_1 = "它可以点燃一大片区域",
            KYOUKO = "那个晓美焰总是喜欢用这些奇奇怪怪的玩意儿。",
        },"让火焰净化一切!")

    quickset("HOMURA_BOMB_BOMB","破片手雷",
        {
            GENERIC = "破片手雷",
            HOMURA_1 = "我研究了一晚上就把它给做出来了。",
            KYOUKO = "那个晓美焰总是喜欢用这些奇奇怪怪的玩意儿。",
        },"住手! 这根本就不是魔法.")
end


local ignore = .1

local function commonfn(prefabname)  --触地即爆炸
    local function onthrown(inst,thrower)
        inst:AddTag("thrown")
        inst:AddComponent('floatable')
        inst.AnimState:PlayAnimation(prefabname.."_air",true)
        inst.Physics:SetFriction(.2)

        inst.components.health:SetInvincible(true)
        inst.components.ignoretimemagic:Set(2)

        inst.GroundTask = inst:DoPeriodicTask(0, function()
            local pos = inst:GetPosition()
            if pos.y <= 0.2 then
                if inst.components.throwable.hitgroundfn then
                    inst.components.throwable.hitgroundfn(inst,thrower)
                end
                inst.GroundTask:Cancel() --保证只触发一次
            end
            
        end)
    end
    local function onequip(inst,owner)
        owner.AnimState:OverrideSymbol("swap_object", "homura_bomb", "swap_"..prefabname)
        owner.AnimState:Show("ARM_carry")
        owner.AnimState:Hide("ARM_normal")
    end
    local function onunequip(inst,owner)
        owner.AnimState:Hide("ARM_carry")
        owner.AnimState:Show("ARM_normal")
    end

    local function ondropped(inst)
        inst.components.health:SetInvincible(false)
    end

    local function onputininventory(inst)
        inst:RemoveComponent('floatable')
        inst:RemoveTag('thrown')
        inst.AnimState:PlayAnimation(prefabname.."")
        inst.Physics:SetFriction(.1)
        inst.components.health:SetInvincible(true)
        inst.components.ignoretimemagic:SetInventory()
        if inst.GroundTask then
            inst.GroundTask:Cancel();inst.GroundTask = nil
        end
    end

    local inst,trans,anim,snd = PUELLA_MOD_FN.CreateEntity()
    MakeInventoryPhysics(inst)
    PUELLA_MOD_FN.SetAnim(inst,'homura_bomb','homura_bomb',prefabname.."")

    inst:AddComponent("inspectable")

    inst:AddComponent("throwable")
    inst.components.throwable.onthrown = onthrown

    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem:SetOnDroppedFn(ondropped)
    inst.components.inventoryitem:SetOnPutInInventoryFn(onputininventory)
    inst.components.inventoryitem.atlasname = "images/inventoryimages/homura_bomb_"..prefabname..".xml"

    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(onequip)
    inst.components.equippable:SetOnUnequip(onunequip)
    inst.components.equippable.equipstack = true

    inst:AddComponent("stackable")
    inst.components.stackable.maxsize = TUNING.STACK_SIZE_LARGEITEM

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(1)
    inst.components.health.vulnerabletopoisondamage = 1
    inst.components.health.canmurder = false
    inst.components.health.canheal = false

    inst:ListenForEvent("death",function(inst) --火焰和aoe攻击会引爆炸弹
        if inst.components.throwable.hitgroundfn then
            inst.components.throwable.hitgroundfn(inst)
        end
    end)

    inst:AddComponent("combat")
    inst.components.combat.canbeattackedfn = function() return false end

    inst:AddComponent("reticule")
    inst.components.reticule.targetfn = function() return inst.components.throwable:GetThrowPoint() end
    inst.components.reticule.ease = true

    inst:AddComponent("ignoretimemagic")

    inst:AddTag("projectile")

    inst:ListenForEvent('hitwater', function() inst.components.throwable.hitgroundfn(inst) end)

    return inst
end

local function FlashBomb()
    local function hitfn(inst,thrower)  --致盲，消除仇恨，恐慌，驱散影怪
        inst.SoundEmitter:PlaySound('lw_homura/flashbang/flash')
        local pos = inst:GetPosition()
        do
            local v = GetPlayer()
            local angle1 = v.Transform:GetRotation()
            local angle2 = v:GetAngleToPoint(pos)
            local dist = math.sqrt(v:GetDistanceSqToPoint(pos))
            local delta = PUELLA_MOD_FN.DeltaAngle(angle1,angle2,"abs")
            if delta <= 75 then
                local time = 10
                if dist > 10 then
                    time = math.max(20 - dist, 0)
                end
                v.HUD.puellaUI_playerflash:SetBlind(time)
            else
                v.HUD.puellaUI_playerflash:SetBlind()
            end
        end
        local ents = TheSim:FindEntities(pos.x, pos.y, pos.z, 20, {'combat'}, {"FX", "NOCLICK", "DECOR", "INLIMBO",'player',})
        for k,v in pairs(ents) do
            v:PushEvent('homura: lightshock')
            if not v:HasTag("player")then
                v.components.combat.panictime = GetRandomMinMax(10,15)
                if  v.components.combat.panictask then
                    v.components.combat.panictask:Cancel();v.components.combat.panictask=nil
                end
                if not v:HasTag("homura: lightshock immue") then
                    v.components.combat.panictask = v:DoPeriodicTask(0,function()
                        v.components.combat.panictime = v.components.combat.panictime - FRAMES
                        v.components.combat.target = nil
                        if v.components.health then
                            v.components.health:DoFireDamage(0,nil,true)
                            --print('PANIC')
                        end
                        if v.components.combat.panictime < 0 then
                            v.components.combat.panictask:Cancel()
                            v.components.combat.panictask = nil
                        end
                    end)
                end
            end
        end
   
        SpawnPrefab("explode_small").Transform:SetPosition(pos:Get())  
        local function CreateLight()
            local inst = CreateEntity()
            inst.entity:AddTransform()
            inst.entity:AddLight()
            inst.Light:Enable(true)
            inst.Light:SetIntensity(.2)
            inst.Light:SetColour(1,1,1)
            inst.Light:SetFalloff(.5)
            inst.Light:SetRadius(.2)
            inst:DoTaskInTime(1,inst.Remove)
            return inst
        end
        CreateLight().Transform:SetPosition(GetPlayer():GetPosition():Get())
        inst:Remove()
    end

    local inst = commonfn("flash")
    inst.components.throwable.hitgroundfn = hitfn

    return inst
end

local function FireBomb() --强化版火狗
    local function hitfn(inst,thrower)
        local pos = inst:GetPosition()
        local ents = TheSim:FindEntities(pos.x, pos.y, pos.z, 5,nil, {"FX", "NOCLICK", "DECOR", "INLIMBO"})
        for k,v in pairs(ents)do
            if v.components.burnable then
                v.components.burnable:Ignite()
            end
        end
        inst.components.lootdropper:DropLoot(pos)
        inst:Remove()
    end
    local inst = commonfn("fire")
    local light = inst.entity:AddLight()
    inst.components.throwable.hitgroundfn = hitfn

    inst:AddComponent("lootdropper")
    inst.components.lootdropper.GenerateLoot = function(self) 
        local loot = {}
        for i = 1, math.random(8,10)do
            table.insert(loot, 'houndfire')
        end
        return loot
    end

    return inst
end

local function Bomb()  --正常的炸弹
    local inst = commonfn("bomb")
    local function hitfn(inst,thrower)
        inst.components.explosive.redirect_to_player = 1
        inst.components.explosive:OnBurnt()
        inst:Remove()
    end
    local function OnExplodeFn(inst)
        inst.SoundEmitter:PlaySound("dontstarve/common/blackpowder_explo")

        local explode = SpawnPrefab("explode_small")
        local pos = inst:GetPosition()
        explode.Transform:SetPosition(pos:Get())
        explode.AnimState:SetBloomEffectHandle( "shaders/anim.ksh" )
        explode.AnimState:SetLightOverride(1)
    end
    inst.components.throwable.hitgroundfn = hitfn

    inst:AddComponent("explosive")
    inst.components.explosive:SetOnExplodeFn(OnExplodeFn)
    inst.components.explosive.explosivedamage = 150
    inst.components.explosive.buildingdamage = 30
    inst.components.explosive.explosiverange = 3
    inst.components.explosive.lightonexplode = false
    inst.components.explosive.noremove = true
    
    return inst
end

return 
Prefab( "homura_bomb_flash" ,FlashBomb,assets),
Prefab( "homura_bomb_fire",FireBomb,assets),
Prefab( "homura_bomb_bomb",Bomb,assets)
