local L = PUELLA_MOD_SETTINGS.LANGUAGE 
STRINGS.NAMES.HOMURA_DETONATOR = L and "Detonator" or "雷管"
STRINGS.CHARACTERS.HOMURA_1.DESCRIBE.HOMURA_DETONATOR = L and "Explode on collision." or "这是炸弹不可或缺的组件"
STRINGS.RECIPE_DESC.HOMURA_DETONATOR1 = L and "Explode on collision." or "遇到撞击就会引爆"
STRINGS.RECIPE_DESC.HOMURA_DETONATOR2 = L and "A lame joke..." or "这是个冷笑话吗?"

local assets =
{
    Asset("ANIM", "anim/homura_detonator.zip"),
    Asset("ATLAS","images/inventoryimages/homura_detonator.xml"),
}

local prefabs =
{
    "explode_small"
}

local function OnExplodeFn(inst)
    inst.SoundEmitter:KillSound("hiss")
    inst.SoundEmitter:PlaySound("dontstarve/common/blackpowder_explo")

    local explode = SpawnPrefab("explode_small")
    local pos = inst:GetPosition()
    explode.Transform:SetPosition(pos.x, pos.y, pos.z)
    explode.AnimState:SetBloomEffectHandle( "shaders/anim.ksh" )
    explode.AnimState:SetLightOverride(1)
end

local function Explode(inst)
    if inst.components.health:IsDead() then
        if not inst.explode then
            inst.explode = true
            inst.components.explosive:OnBurnt()
        end
    end
end

local function fn(Sim)
    local inst = CreateEntity()
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    MakeInventoryPhysics(inst)
    MakeInventoryFloatable(inst, "idle_water", "idle")

    inst.AnimState:SetBank("homura_detonator")
    inst.AnimState:SetBuild("homura_detonator")
    inst.AnimState:PlayAnimation("idle")

    inst:AddComponent("stackable")
    inst.components.stackable.maxsize = TUNING.STACK_SIZE_SMALLITEM

    inst:AddComponent("inspectable")

    inst:AddComponent("explosive")
    inst.components.explosive:SetOnExplodeFn(OnExplodeFn)
    inst.components.explosive.explosivedamage = 20
    inst.components.explosive.buildingdamage = 6
    inst.components.explosive.lightonexplode = false
    inst.components.explosive.explosiverange = 2

    inst:AddComponent("appeasement")
    inst.components.appeasement.appeasementvalue = TUNING.WRATH_LARGE
    
    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.atlasname = "images/inventoryimages/homura_detonator.xml"

    inst:AddComponent('combat')
    inst.components.combat.canbeattackedfn = function() return false end

    inst:AddComponent('health')
    inst.components.health:SetMaxHealth(1)
    inst.components.health.vulnerabletopoisondamage = 1
    inst.components.health.canmurder = false
    inst.components.health.canheal = false

    inst:AddComponent("tradable")

    inst:ListenForEvent("healthdelta", Explode)

    return inst
end

return Prefab( "common/inventory/homura_detonator", fn, assets, prefabs) 

