local assets= 
{
    Asset("ANIM", "anim/homura_rpg.zip"),
    Asset("SOUNDPACKAGE", "sound/lw_homura.fev"),
    Asset("SOUND", "sound/lw_homura.fsb"), 

    Asset("ATLAS","images/inventoryimages/homura_rpg.xml"),
    Asset("ATLAS","images/inventoryimages/homura_rpg_ammo1.xml"),
    Asset("ATLAS","images/inventoryimages/homura_rpg_ammo2.xml"),
    Asset("ATLAS_BUILD","images/inventoryimages/homura_rpg.xml",256),
    Asset("ATLAS_BUILD","images/inventoryimages/homura_rpg_ammo1.xml",256),
    Asset("ATLAS_BUILD","images/inventoryimages/homura_rpg_ammo2.xml",256),

    Asset("ANIM", "anim/homura_weapon_ui_1.zip"),

    Asset("ANIM", "anim/player_homura_rpg.zip"),
}


local L = PUELLA_MOD_SETTINGS.LANGUAGE
local function quickset(prefab,name,desc_data,recipe)
    prefab = string.upper(prefab)
    STRINGS.NAMES[prefab] = name 
    for cha,desc in pairs(desc_data)do
        cha = string.upper(cha)
        STRINGS.CHARACTERS[cha].DESCRIBE[prefab] = desc
    end
    STRINGS.RECIPE_DESC[prefab] = recipe
end
if PUELLA_MOD_SETTINGS.LANGUAGE then
    quickset("homura_rpg","rocket launcher",
        {
            homura_1 = "Can easily shoot down an airplane!",
        },"Can easily shoot down an airplane!")
    quickset("homura_rpg_ammo1","rocket propelled grenade",
        {
            homura_1 = "A flying bomb!"
        },"A flying bomb!")
    quickset("homura_rpg_ammo2","FIM-92 Stinger ",
        {
            homura_1 = "It uses infrared guidance to track targets.",
        },"The nightmare of aircrafts.")

else
    quickset("homura_rpg","火箭发射器",
        {
            homura_1 = "我曾经听一个男的说这东西打飞机很准。",
        },"威力巨大的远程武器")
    quickset("homura_rpg_ammo1","火箭助推榴弹",
        {
            homura_1 = "这是一颗会飞的小炸弹。"
        },"一发入魂。")
    quickset("homura_rpg_ammo2","FIM-92毒刺导弹",
        {
            homura_1 = "采用红外制导，自动追踪",
        },"指哪儿打哪儿。")
    
end

local function MakeAmmo(num)
    local function fn()
        local inst,trans,anim,snd = PUELLA_MOD_FN.CreateEntity()
        MakeInventoryPhysics(inst)
        PUELLA_MOD_FN.SetAnim(inst,'homura_rpg', 'ammo'..num , 'idle')
        MakeInventoryFloatable(inst, "idle_water", "idle")

        inst:AddComponent("inspectable")
        inst:AddComponent("tradable")
        inst:AddComponent("inventoryitem")
        inst.components.inventoryitem.atlasname = "images/inventoryimages/homura_rpg_ammo"..num..".xml"
        inst:AddComponent('homura_ammo')
        inst:AddComponent("stackable")

        inst:AddTag('homura_rpg_ammo')
        inst.projectilename = 'homura_missile'..num
        return inst
    end
    return Prefab("homura_rpg_ammo"..num,fn,assets) 
end

local function OnThrown(inst,owner,target)
    inst.components.homura_projectile:OnTrow(inst,owner,target)
end

local function CreateFire()
    local inst = PUELLA_MOD_FN.CreateEntity()
    MakeInventoryPhysics(inst)
    PUELLA_MOD_FN.SetAnim(inst,'homura_rpg','smoke','anim',true)
    inst:AddTag("FX","NOCLICK","NOBLOCK")

    inst.AnimState:SetMultColour(1,195/255,86/255,.8)
    inst.AnimState:SetTime(math.random())
    inst.AnimState:SetBloomEffectHandle("shaders/anim.ksh")

    local s = GetRandomMinMax(.4,.8)
    inst.Transform:SetScale(s,s,s)
    inst:AddComponent("colourtweener")
    
    inst.components.colourtweener:StartTween({0,0,0,.5},.5)
    inst:DoTaskInTime(.6,function(inst)
        inst.AnimState:ClearBloomEffectHandle()
        inst.components.colourtweener:StartTween({0,0,0,0},.5,inst.Remove)
    end)

    local light = inst.entity:AddLight()
    light:Enable(true)
    light:SetRadius(.5)
    light:SetFalloff(.5)
    light:SetIntensity(.8)
    light:SetColour(1,0.75,0.25)

    inst:AddComponent('lighttweener')
    inst.components.lighttweener:StartTween(light, 0.1, 0.5, 0.8, nil, 0.6, function()light:Enable(false) end)

    return inst
end

local function UpdateTail(inst,tails)
    local x, y, z = inst.Transform:GetWorldPosition()
    for tail, _ in pairs(tails) do
        tail:ForceFacePoint(x, y, z)
    end
 
    local loop = math.random(1,5)
    for i = 1,loop do
        local speed = 5
        local tail = CreateFire()
        local rot = inst.Transform:GetRotation()
        tail.Transform:SetRotation(rot)
        rot = rot * DEGREES
        local offsangle = math.random() * 6.283
        local offsradius = GetRandomMinMax(0,.2)
        local hoffset = math.cos(offsangle) * offsradius
        local voffset = math.sin(offsangle) * offsradius
        tail.Transform:SetPosition(x + math.sin(rot) * hoffset, y + voffset, z + math.cos(rot) * hoffset)
        tail.Physics:SetMotorVel(speed * GetRandomMinMax(.2,.5), 0, 0)
        tails[tail] = true
        inst:ListenForEvent("onremove", function(tail) tails[tail] = nil end, tail)
        tail:ListenForEvent("onremove", function(inst)
            tail.Transform:SetRotation(tail.Transform:GetRotation() + GetRandomMinMax(-15,15))
        end, inst)
    end    
end

local function OnHit(radius, damage)
    local onhitfn = function(inst, owner, target)
        GetPlayer().components.playercontroller:ShakeCamera(inst, "FULL", 0.7, 0.02, .5, 40)
        local explode = SpawnPrefab("explode_small")
        explode.Transform:SetPosition(inst:GetPosition():Get())
        explode.AnimState:SetBloomEffectHandle( "shaders/anim.ksh" )
        explode.AnimState:SetLightOverride(1)

        inst.SoundEmitter:PlaySound("dontstarve/common/blackpowder_explo")
        inst.components.explosive.lightonexplode = not inst.components.homura_projectile.ice 
        inst.components.explosive.explosivedamage = damage
        inst.components.explosive.explosiverange = radius
        inst.components.explosive:OnBurnt()
    end

    return onhitfn
end

local function MakeCommonMissile(inst)
    inst.Physics:SetFriction(.5)
    inst.Transform:SetScale(0.8,0.8,1)
    inst:DoPeriodicTask(0,UpdateTail,nil,{})
 
    inst.components.projectile:SetLaunchOffset(Vector3(0,1.3,0))
    inst.components.projectile:SetHoming(true)

    inst:AddComponent("ignoretimemagic")
    inst.components.ignoretimemagic:SetProjectile()

    inst:AddComponent("acceleratingprojectile")
    inst.components.acceleratingprojectile.a = 1.1
    inst.components.acceleratingprojectile:StartUpdate()

    inst:AddComponent('homura_projectile')
    inst.components.homura_projectile.infinite_range = true

    inst:AddComponent("explosive")
    inst.components.explosive.redirect_to_player = true

    inst.components.projectile.onthrown = OnThrown

    inst:ListenForEvent("entitysleep", inst.Remove) 
end

local function missile1(inst) --榴弹
    MakeCommonMissile(inst)
    inst.components.projectile:SetHoming(false)

    inst.components.projectile:SetOnHitFn(OnHit(3, 300))
    inst.components.projectile:SetOnMissFn(OnHit(3, 300))
end

local function missile2(inst) --导弹
    MakeCommonMissile(inst)
    inst.components.projectile:SetHoming(true)

    inst.components.projectile:SetOnHitFn(OnHit(4, 500))
    inst.components.projectile:SetOnMissFn(OnHit(4, 500))
end

local function onequip(inst,owner)
    owner.AnimState:OverrideSymbol("swap_object","homura_rpg","swap_rpg")
    owner.AnimState:Show("ARM_carry") 
    owner.AnimState:Hide("ARM_normal") 
end

local function onunequip(inst,owner)
    owner.AnimState:ClearOverrideSymbol("swap_object")
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal") 
end

local function OnAcceptAmmo(inst, ammo)
    if ammo and ammo.prefab == 'homura_rpg_ammo1' then
        inst.components.homura_weapon.projectile = 'homura_missile1'
    elseif ammo and ammo.prefab == 'homura_rpg_ammo2' then
        inst.components.homura_weapon.projectile = 'homura_missile2'
    end
end

local function launcherfn()
    local inst = CreateEntity()
    inst.entity:AddTransform() 
    inst.entity:AddAnimState() 
    inst.entity:AddSoundEmitter()
    PUELLA_MOD_FN.SetAnim(inst,'homura_rpg','homura_rpg','idle')
    MakeInventoryPhysics(inst)
    MakeInventoryFloatable(inst, "idle_water", "idle")
    inst.Transform:SetScale(1.1,1.1,1.1)

    inst:AddComponent("inspectable")

    inst:AddComponent("container")
    inst.components.container.itemtestfn = function (inst,item) return item:HasTag('homura_weapon_buff') end
    inst.components.container:SetNumSlots(1)
    
    inst.components.container.widgetslotpos = {Vector3(2,0,0)}
    inst.components.container.widgetanimbank = "homura_weapon_ui_1"
    inst.components.container.widgetanimbuild = "homura_weapon_ui_1"
    inst.components.container.widgetpos = Vector3(0,100,0)
    inst.components.container.side_align_tip = 160
    inst.components.container.acceptsstacks = false
    inst.components.container.canbeopened = false
    inst.components.container.type = "homura_weapon"

    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.atlasname = "images/inventoryimages/homura_rpg.xml"
    if GetPlayer():HasTag("homura") then
        inst:AddComponent("equippable")
        inst.components.equippable:SetOnEquip(onequip)
        inst.components.equippable:SetOnUnequip(onunequip)

        inst:AddComponent("weapon")
        inst.components.weapon.onprojectilelaunch = function(inst, attacker, target)
            inst.components.homura_weapon:OnLaunch(attacker)
        end

        inst:AddComponent("homura_weapon")
        
        local weapon_data = {
            speed = 10, --射速
            range = 15,  --射程
            hitrange = 15, --必中射程
            min_hit_rate = 1, --最低命中率
            variety = 1.5,  --子弹最大偏角(pro)
            damage1 = 0,  ---攻击伤害
            damage2 = 17,  ---平a伤害
            maxammo = 1,
        }
        inst.components.homura_weapon.projectile = 'homura_missile1'
        inst.components.homura_weapon.projectile_prefabname = 'homura_rpg_ammo1'
        inst.components.homura_weapon:SetBasicPara(weapon_data)
        inst.components.homura_weapon.onacceptfn = OnAcceptAmmo
    end

    inst:AddComponent('named')
    
    return inst
end

return PUELLA_MOD_FN.MakeProjectile( "homura_missile1", "ammo1","homura_rpg",
 "pro",'火箭助推榴弹' ,10,nil, assets,missile1 ) ,
 PUELLA_MOD_FN.MakeProjectile( "homura_missile2", "ammo2","homura_rpg",
 "pro",'导弹' ,10,nil, assets,missile2 ) ,
Prefab( "common/inventory/homura_rpg" ,launcherfn,assets), 
MakeAmmo(1),MakeAmmo(2) --homura_rpg_ammo1~2

