local L = PUELLA_MOD_SETTINGS.LANGUAGE 

STRINGS.NAMES.HOMURA_PISTOL_SN = L and "Pistol" or "破旧的手枪"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.HOMURA_PISTOL_SN = L and "Often in battle, but never in name. [by google translation]" or "常常穿梭于战场, 但却从未留名"

local assets =
{
    Asset("ANIM","anim/homura_pistol_sn.zip"),
    Asset("ATLAS","images/inventoryimages/homura_pistol_sn.xml"),
}

local function onequip(inst, owner)
    inst.SoundEmitter:PlaySound('lw_homura/soulknight/equip', nil, 1)
    owner.AnimState:OverrideSymbol("swap_object", "homura_pistol_sn", "swap_item") 
    owner.AnimState:Show("ARM_carry")
    owner.AnimState:Hide("ARM_normal")
end
local function onunequip(inst, owner)
    inst.SoundEmitter:PlaySound('lw_homura/soulknight/unequip', nil, 1)
    owner.AnimState:Hide("ARM_carry") 
    owner.AnimState:Show("ARM_normal")
end

local function fn()
	local inst = CreateEntity()
    inst.entity:AddTransform() 
    inst.entity:AddAnimState() 
    inst.entity:AddSoundEmitter()
    MakeInventoryPhysics(inst)
    MakeInventoryFloatable(inst, "idle", "idle")

    inst.AnimState:SetBank('homura_pistol_sn')
    inst.AnimState:SetBuild('homura_pistol_sn')
    inst.AnimState:PlayAnimation('idle')

    inst:AddComponent("inspectable")

    inst:AddComponent("tradable")

    inst:AddComponent("inventoryitem")
    inst.components.inventoryitem.atlasname = "images/inventoryimages/homura_pistol_sn.xml"

    inst:AddComponent("equippable")
    inst.components.equippable:SetOnEquip(onequip)
    inst.components.equippable:SetOnUnequip(onunequip)

    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(3)
    inst.components.weapon:SetRange(6,8)
    inst.components.weapon:SetProjectile('homura_pistol_sn_projectile')
    
    inst:AddTag("irreplaceable")

	return inst
end

local function profn()
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local sound = inst.entity:AddSoundEmitter()
    
    MakeInventoryPhysics(inst)
    RemovePhysicsColliders(inst)

    trans:SetScale(1.3,1.3,1)
    
    anim:SetBank("pro")
    anim:SetBuild("homura_pistol")
    anim:PlayAnimation("anim")
    anim:SetOrientation(ANIM_ORIENTATION.OnGround)
    
    inst:AddTag("projectile")
    inst.persists = false
    
    inst:AddComponent("projectile")
    inst.components.projectile:SetSpeed(30)
    inst.components.projectile:SetHoming(false)
    inst.components.projectile:SetHitDist(2)
    inst.components.projectile:SetLaunchOffset(Vector3(1,1,0))
    inst.components.projectile:SetOnHitFn(inst.Remove)
    inst.components.projectile:SetOnMissFn(inst.Remove)
    
    return inst
end


return Prefab("homura_pistol_sn", fn, assets), Prefab("homura_pistol_sn_projectile", profn)
