local assets =
{
    Asset("ANIM", "anim/sayaka_heal.zip"),
    Asset("SOUNDPACKAGE", "sound/lw_sayaka.fev"),
    Asset("SOUND", "sound/lw_sayaka.fsb"),
}

local healingrange = 4
local magic = .7

local function onsave(inst,data)
    data.LeftHeal = inst.LeftHeal
    data.casterGUID = inst.caster and inst.caster.GUID
end

local function onload(inst,data)
    inst.LeftHeal = data.LeftHeal
end

local function onloadpostpass(ent,data)
    if data.casterGUID and ent[data.casterGUID] then
        inst.caster = ent[data.casterGUID]
    end
end

local function update(inst)
    inst.LeftHeal = inst.LeftHeal - 1
    if not inst.caster then 
        return 
    end

    local pos = inst:GetPosition()
    local ents = TheSim:FindEntities(pos.x,pos.y,pos.z, healingrange)
    for _,v in pairs(ents)do
        if v.components.health and not v.components.health:IsDead() and v.components.health:IsHurt() then
            if v.components.follower and v.components.follower.leader == inst.caster
            or v:HasTag("player") 
            or v.prefab == "chester" 
            or v.prefab == "glommer" 
            or v.prefab == "ro_bin" 
            or v.prefab == 'packim' then
                if not v.components.sayaka_healed then
                    v:AddComponent("sayaka_healed")
                end
                v.components.sayaka_healed:Heal()
                inst.caster.components.soulgem:DoDelta(magic,true)
            end
        end
    end

    if inst.LeftHeal == 0 then
        inst:Fade()
    end
end

local function fade(inst)
    inst.components.colourtweener:StartTween({0,0,0,0},1,function()
        inst:Remove()
    end)
end

local function fn()  --沙耶香的治疗法阵,持续时间10秒，共触发6次治疗
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local snd = inst.entity:AddSoundEmitter()
    local light = inst.entity:AddLight()

    light:Enable(true)
    light:SetIntensity(.5)--光强
    light:SetColour(80/255,80/255,255/255)
    light:SetFalloff(.5) --衰减
    light:SetRadius(3) --半径

    anim:SetMultColour(.8,0.8,1,1)
    anim:SetBank("sayaka_heal")
    anim:SetBuild("sayaka_heal")
    anim:PlayAnimation("enter")
    anim:PushAnimation("loop")
    anim:SetOrientation(ANIM_ORIENTATION.OnGround)
    anim:SetLayer(LAYER_BACKGROUND)
    anim:SetSortOrder(3)

    snd:PlaySound("lw_sayaka/sfx/heal",nil,.3)

    inst.LeftHeal = 6
    inst:DoPeriodicTask(1.5,update,1)
    inst.Fade = fade

    inst.OnSave = onsave
    inst.OnLoad = onload
    inst.OnLoadPostPass = onloadpostpass

    trans:SetScale(1,1,1)
    inst:AddTag("fx")

    inst:AddComponent("colourtweener")

    return inst
end

return 
Prefab("common/fx/sayaka_heal", fn, assets) 
