local function ShakeIfClose(inst)
    local player = GetClosestInstWithTag("player", inst, 40)
    if player then
        player.components.playercontroller:ShakeCamera(inst, "FULL", 0.7, 0.02, 2, 40)
    end
end

local states=
{
    State{

        name = "idle",
        tags = {"idle", "canrotate"},

        onenter = function(inst, playanim)
            inst.AnimState:PlayAnimation("idle", true)
        end,

    },

    State{

        name = "hit",
        tags = {"canrotate", "busy",},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("hit")
            inst.components.locomotor:StopMoving()           
        end,

        events=
        {
            EventHandler("animover", function(inst)inst.sg:GoToState("idle") end),
        },
    },    

    State{

        name = "walk",
        tags = {"walk","canrotate", "moving" },

        onenter = function(inst, playanim)
            inst.AnimState:PlayAnimation("walk", true)
            inst.components.locomotor:WalkForward()            
        end,
    }, 

    State{
        name = "pound",
        tags = {"attack", "canrotate","nointerrupt","busy"},

        onenter = function(inst, cb)
            inst.components.locomotor:StopMoving()
            inst.components.combat:StartAttack()
            inst.AnimState:PlayAnimation("pound")
        end,

        timeline=
        {
            --TimeEvent(17*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve_DLC001/creatures/bearger/swhoosh") end),
            TimeEvent(59*FRAMES, function(inst) 
                ShakeIfClose(inst)
                inst.components.combat:SetDefaultDamage(300)
                inst.components.groundpounder:GroundPound()
                inst.components.combat:SetDefaultDamage(200)
                inst.components.qw_stemspawner:OnPound()
                inst.CanGroundPound = false
                inst.SoundEmitter:PlaySound("dontstarve_DLC001/creatures/bearger/groundpound")
            end),
        },

        events=
        {
            EventHandler("animover", function(inst)inst.sg:GoToState("idle") end),
        },
    },  

    State{
        name = "attack",
        tags = {"attack", "canrotate","nointerrupt","busy"},

        onenter = function(inst, cb)
            inst.components.locomotor:StopMoving()
            inst.components.combat:StartAttack()
            inst.AnimState:PlayAnimation("attack")
        end,

        timeline=
        {
            TimeEvent(0*FRAMES, function(inst) inst.SoundEmitter:PlaySound("lw_witch/qw/atk") end),
            --TimeEvent(0*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve/creatures/spiderqueen/attack_grunt") end),
            --TimeEvent(28*FRAMES, function(inst) inst.SoundEmitter:PlaySound("dontstarve/creatures/spiderqueen/swipe") end),
            TimeEvent(20*FRAMES, function(inst) inst.components.combat:DoAttack() end),
        },

        events=
        {
            EventHandler("animover", function(inst)inst.sg:GoToState("idle") end),
        },
    },

    State{
        name = "death",
        tags = {"busy"},
        
        onenter = function(inst)
            inst.SoundEmitter:PlaySound("dontstarve/creatures/spiderqueen/die")
            inst.AnimState:PlayAnimation("death")
            inst.components.locomotor:StopMoving()
            inst.components.lootdropper:DropLoot(inst:GetPosition())  
            RemovePhysicsColliders(inst)         
        end,
        
    },

}

local event_handlers=
{

    EventHandler("locomote", function(inst) 
        if inst.components.locomotor:WantsToMoveForward() then
            if not inst.sg:HasStateTag("moving") and not inst.sg:HasStateTag("busy")then
                inst.sg:GoToState("walk")
            end
        else
            if not inst.sg:HasStateTag("idle") and not inst.sg:HasStateTag("busy") then
                inst.sg:GoToState("idle")
            end
        end
    end),

    EventHandler("attacked", function(inst, data) 
        if not inst.components.health:IsDead() and not inst.sg:HasStateTag("nointerrupt") and not inst.sg:HasStateTag("attack") then 
            if not inst.sg:HasStateTag('moving') or (data and data.damage > 50) then
                inst.sg:GoToState("hit") 
            end
        end 
    end),
    
    EventHandler("doattack", function(inst) 
        if not inst.components.health:IsDead() and not inst.sg:HasStateTag("attack") then 
            if inst.CanGroundPound then
                inst.sg:GoToState("pound")
            else
                inst.sg:GoToState("attack")
            end
        end 
    end),
    EventHandler("death", function(inst) inst.sg:GoToState("death") end),
}
local sg = StateGraph("witch_qw", states, event_handlers, "idle", {})


local L = PUELLA_MOD_SETTINGS.LANGUAGE 
STRINGS.NAMES.WITCH_QW = L and "Gertrud" or "蔷薇魔女"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.WITCH_QW = L and "That's scary." or "这是什么鬼？"
STRINGS.CHARACTERS.KYOUKO.DESCRIBE.WITCH_QW = L and "That's the true witch, not me..." or "明明这才是真正的魔女..."
STRINGS.CHARACTERS.SAYAKA.DESCRIBE.WITCH_QW = L and "She looks disgusting." or "呃，看起来好恶心。"
STRINGS.CHARACTERS.HOMURA_1.DESCRIBE.WITCH_QW = L and "She is cumbersome and can hardly hurt me." or "她很笨重，根本伤不到我。"


local assets=
{
    Asset("ANIM", "anim/witch_qw.zip"),
}
local prefabs = 
{
    "griefseed",  "qw_stem", "witch_qw_icon",
}

local function Retarget(inst)
    if not inst.components.health:IsDead() then
        local oldtarget = inst.components.combat.target
        local newtarget = FindEntity(inst, 10, function(guy) return inst.components.combat:CanTarget(guy) and guy:HasTag("character")end)

        if newtarget and newtarget ~= oldtarget then
            inst.components.combat:SetTarget(newtarget)
        end
    end
end

local function OnAttacked(inst, data)
    inst.components.combat:SetTarget(data.attacker)
    inst.components.combat:ShareTarget(data.attacker, 10, function(dude) 
        return dude:HasTag("witch_qw") and not dude.components.health:IsDead() 
    end, 2)
end

local function onhealthdelta(inst,data)
    local old,new = data.oldpercent, data.newpercent
    for _,stage in ipairs({0.25,0.50,0.75}) do
        if old ~= new and (old-stage)*(new-stage)<=0 then 
            inst.CanGroundPound = true
        end
    end
end

local function fn()
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local snd = inst.entity:AddSoundEmitter()
    local shadow = inst.entity:AddDynamicShadow()
    shadow:SetSize( 7, 3 )
   
    MakeCharacterPhysics(inst,1000, 1.5)
    trans:SetFourFaced()
    trans:SetScale(3,3,1)

    anim:SetBank("witch_qw")
    anim:SetBuild("witch_qw")

    MakeMediumBurnableCharacter(inst,'')

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetLoot({"griefseed"})

    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = 1
    inst.components.locomotor.runspeed = 1
 
    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(3000)
    inst.components.health.fire_damage_scale = 2
    inst.components.health:SetAbsorptionAmount(.15)

    inst:AddComponent("combat")
    inst.components.combat:SetRange(3.8,4.3)
    inst.components.combat:SetDefaultDamage(200)
    inst.components.combat.playerdamagepercent = .5
    inst.components.combat:SetAttackPeriod(3)
    inst.components.combat:SetRetargetFunction(3, Retarget)

    inst:AddComponent("inspectable")

    inst:AddComponent("groundpounder")
    inst.components.groundpounder.destroyer = true
    inst.components.groundpounder.damageRings = 3
    inst.components.groundpounder.destructionRings = 3
    inst.components.groundpounder.numRings = 2
    inst.components.groundpounder.ring_fx_scale = .7
    inst.components.groundpounder.noTags = {"FX", "NOCLICK", "DECOR", "INLIMBO","witch",}

    inst:AddTag("witch","witch_mn","witch_qw")
    inst:AddTag("epic")
    inst:AddTag("monster")
    inst:AddTag("hostile")

    inst:AddComponent("leader")

    inst:AddComponent("qw_stemspawner")
    inst:AddComponent("qw_flowerspawner")
    
    inst:ListenForEvent("attacked", OnAttacked)
    inst:ListenForEvent("healthdelta",onhealthdelta)
    inst:ListenForEvent('death',function(inst)
        for f,_ in pairs(inst.components.leader.followers)do
            inst:DoTaskInTime(math.random(),function()f.components.health:Kill("nopower")end)
        end
    end)

    inst.CanGroundPound = false
    inst.OnSave = function (inst,data)data.CanGroundPound = inst.CanGroundPound end
    inst.OnLoad = function (inst,data)if data then inst.CanGroundPound = data.CanGroundPound end end
    
    PUELLA_MOD_FN.SetSG(inst,sg)
    local brain = require "brains/witch_qw_brain"
    inst:SetBrain(brain)

    PUELLA_MOD_FN.WitchCommon(inst)

    return inst
end
return Prefab( "witch/witch_qw", fn, assets, prefabs)
