local L = PUELLA_MOD_SETTINGS.LANGUAGE 
STRINGS.NAMES.QW_STEM = L and "Thorn" or "荆棘"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.QW_STEM = L and "I should not use console." or "我就不该开控制台的。"
STRINGS.CHARACTERS.KYOUKO.DESCRIBE.QW_STEM = L and "My lance can easily cut it off." or "我的长枪可以轻松斩断这些荆棘。"
STRINGS.CHARACTERS.SAYAKA.DESCRIBE.QW_STEM = L and "It too rough." or "干干巴巴的，麻麻赖赖的，一点都不圆润。"
STRINGS.CHARACTERS.HOMURA_1.DESCRIBE.QW_STEM = L and "Don't stand too close when it drills out." or "这东西钻出来的时候别靠太近。"
STRINGS.NAMES.QW_MAN = L and "Anthony" or "蔷薇魔女的使魔"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.QW_MAN = L and "It's... kind of... pretty." or "这东西... 还挺漂亮的... 某种意义上。"
STRINGS.CHARACTERS.KYOUKO.DESCRIBE.QW_MAN = L and "It's just a minion." or "不过是一个小喽啰罢了。"
STRINGS.CHARACTERS.SAYAKA.DESCRIBE.QW_MAN = L and "This flower looks fragile." or "这朵花看起来挺弱的啊。"
STRINGS.CHARACTERS.HOMURA_1.DESCRIBE.QW_MAN = L and "You are a ADC,too?" or "你也是个远程？"

local assets=
{
    Asset("ANIM", "anim/qw_ghost.zip"),
    Asset("ANIM", "anim/qw_man.zip"),  
    Asset("ANIM", "anim/bramble_pmmm.zip"), 
}

local prefabs = {}

local function IsProjectile(weapon)
    if not weapon then return end
    return weapon:HasTag("projectile") or weapon.components.weapon.projectile or weapon.components.weapon.attackrange
end

local function OnBlocked(inst, data) 
    if not IsProjectile(data.weapon)
        and data.attacker and data.attacker.components.combat and not data.attacker:HasTag("witch")
        and ((data.damage and data.damage > 0) or (data.attacker.components.combat and data.attacker.components.combat.defaultdamage > 0)) then
        data.attacker.components.combat:GetAttacked(inst,8, nil, "thorns")
        inst.SoundEmitter:PlaySound("dontstarve_DLC002/common/armour/cactus")
    end
    if not inst.components.health:IsDead() then
        inst.AnimState:PlayAnimation("hit")
        inst.AnimState:PushAnimation("idle")
    end
    if data.attacker then
        inst.components.combat:SetTarget(data.attacker)
    end
end

local function qw_stem()
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local snd = inst.entity:AddSoundEmitter()
    MakeObstaclePhysics(inst, .3)
    anim:SetBank("bramble_"..math.random(1,3))
    anim:SetBuild("bramble_pmmm")
    --anim:SetBuild("qw_stem")
    anim:SetMultColour(.3,.3,.3,1)
    trans:SetScale(.4,.4,.5)
    
    anim:PlayAnimation("grow")
    anim:PushAnimation("idle")

    inst:AddTag("growing")
    inst:DoTaskInTime(1,function ()inst:RemoveTag("growing") end)

    inst.timer = GetRandomMinMax(20,40)
    inst:DoPeriodicTask(1,function()
        local leader = inst.components.follower.leader 
        if not leader then
            inst:PushEvent("death",{cause = "nopower"})
        else
            local dist = inst:GetPosition():Dist(leader:GetPosition())
            if dist > 20 then
                inst.timer = inst.timer - 1
            elseif dist > 10 then
                inst.timer = inst.timer - 2
            elseif dist > 3 then
                inst.timer = inst.timer - 3
            else
                inst.timer = inst.timer - 5
            end
            if inst.timer < 0 then
                inst:PushEvent("death",{cause = "nopower"})
            end
        end
    end)

    MakeLargeBurnableCharacter(inst,"")
    inst.components.burnable.canlight = true
    inst:RemoveComponent('propagator')
    MakeSmallPropagator(inst)

    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(40)
    inst.components.health:SetAbsorptionAmount(.4)
    
    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(1)
    inst.components.combat:SetRange(10)
    inst.components.combat.notags = {"witch"}
    inst.components.combat:SetHurtSound("dontstarve_DLC002/creatures/volcano_cactus/hit")

    inst:AddComponent("follower")

    inst:ListenForEvent("blocked", OnBlocked)
    inst:ListenForEvent("attacked", OnBlocked)
    inst:ListenForEvent("death", function(inst,data)
        inst.AnimState:PlayAnimation("wither") 
        inst:ListenForEvent("animover",inst.Remove)
    end)

    inst:AddComponent("inspectable")

    inst:AddTag("witch","witch_sm","witch_qw")
    inst:AddTag("epic")
    inst:AddTag("hostile")

    return inst
end

local function qw_ghost()   --暂时不用, 等哪天蔷薇魔女太弱了再加回来强化一下
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local snd = inst.entity:AddSoundEmitter()
    local light = inst.entity:AddLight()

    light:SetIntensity(.6)
    light:SetRadius(.5)
    light:SetFalloff(.6)
    light:Enable(true)
    light:SetColour(180/255, 195/255, 225/255)
    
    local brain = require "brains/ghostbrain"
    inst:SetBrain(brain)

    MakeAmphibiousGhostPhysics(inst, 1, .5)
    anim:SetBank("ghost")
    anim:SetBuild("qw_ghost")
    anim:PlayAnimation("idle", true)
    --trans:SetScale(.5,.5,.5)
    inst:AddTag("monster")
    inst:AddTag("hostile")
    inst:AddTag("ghost")
    inst:AddTag("noauradamage")
    inst:AddTag("witch")
    inst:AddTag("witch_qw")
    inst:AddTag("witch_sm")
    inst:AddTag("epic")

    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = TUNING.GHOST_SPEED
    inst.components.locomotor.runspeed = TUNING.GHOST_SPEED
    inst.components.locomotor.directdrive = true
    
    inst:SetStateGraph("SGqw_ghost")
    
    inst:AddComponent("sanityaura")
    inst.components.sanityaura.aura = -TUNING.SANITYAURA_MED
    
    inst:AddComponent("inspectable")
    
    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(500)
    inst.components.health:SetAbsorptionAmount(.25)
    
    inst:AddComponent("combat")
    inst.components.combat.defaultdamage = TUNING.GHOST_DAMAGE
    inst.components.combat.playerdamagepercent = TUNING.GHOST_DMG_PLAYER_PERCENT
    inst.components.combat.notags = {"witch"}
    
    inst:AddComponent("aura")
    inst.components.aura.radius = TUNING.GHOST_RADIUS
    inst.components.aura.tickperiod = TUNING.GHOST_DMG_PERIOD
    
    ------------------    
    --inst.SoundEmitter:PlaySound("dontstarve/ghost/ghost_howl_LP", "howl")---------

    return inst    
end

local function EquipWeapon(inst)
    if inst.components.inventory and not inst.components.inventory:GetEquippedItem(EQUIPSLOTS.HANDS) then
        local blowdart = CreateEntity()
        blowdart.entity:AddTransform()
        blowdart:AddComponent("weapon")
        blowdart:AddTag("sharp")
        blowdart.components.weapon:SetDamage(34)
        blowdart.components.weapon:SetRange(30)
        blowdart.components.weapon:SetProjectile("qw_projectile")
        blowdart:AddComponent("inventoryitem")
        blowdart.components.inventoryitem:SetOnDroppedFn(function(blowdart)blowdart:Remove()end)
        blowdart:AddComponent("equippable")
        blowdart.persists = false
        inst.components.inventory:Equip(blowdart)
    end
end

local function UpdateFn(inst)
    local leader = inst.witchqw
    if not leader or not leader:IsValid() then
        inst:PushEvent("death",{cause = "nopower"})
    else
        local dist = inst:GetPosition():Dist(leader:GetPosition())
        if dist > 20 then
            inst:PushEvent("death",{cause = "nopower"})
        end
    end
    inst.timer = inst.timer - 1
    if inst.timer < 0 and not inst:HasTag("abouttoatk") then
        inst:tryatkfn()
    end
end

local function DoAttack(inst)
    inst.AnimState:SetMultColour(1,1,1,1)
    if not inst.components.combat.target then
        inst.components.combat:SetTarget(GetPlayer())
    end
    inst.SoundEmitter:PlaySound("dontstarve/wilson/boomerang_throw")
    inst.components.combat:DoAttack()
    inst:RemoveTag("abouttoatk")
    inst.timer = 6 + 4 * math.random()
end

local function TryAttack(inst)
    inst:AddTag("abouttoatk")
    inst.components.colourtweener:StartTween({.7,.3,.3,1},3,inst.atkfn)
end

local function qw_man()  --蔷薇魔女的炮塔。距离魔女过远会消失，一旦被击杀，会被魔女重新召唤
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local snd = inst.entity:AddSoundEmitter()
    inst.entity:AddPhysics()
    --local light = inst.entity:AddLight()
    anim:SetBuild("qw_man")
    anim:SetBank("qw_man")
    anim:PlayAnimation("appear")
    anim:PushAnimation("idle",true)
    
    trans:SetScale(1.3,1.3,1.3)

    inst:AddTag("monster")
    inst:AddTag("hostile")
    inst:AddTag("witch","witch_qw","witch_sm")
    inst:AddTag("epic")

    inst:AddComponent("inspectable")

    inst:AddComponent("colourtweener")
    
    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(100)

    inst:AddComponent("inventory")
    
    inst:AddComponent("combat")
    inst.components.combat.defaultdamage = 34
    inst.components.combat.notags = {"witch"}

    inst:ListenForEvent("attacked",function ()
        if not inst.components.health:IsDead() then
            inst.AnimState:PlayAnimation("hit") 
            inst.AnimState:PushAnimation("idle",true)
        end
    end)

    inst:ListenForEvent("death",function ()
        if not inst.isdead then 
            inst.isdead = true
            inst.AnimState:PlayAnimation("disppear")  --我拼错了。。。
            inst:ListenForEvent("animover",inst.Remove)
        end
    end)
    
    inst:DoTaskInTime(0,EquipWeapon)
    inst:DoPeriodicTask(1,UpdateFn)
    inst.timer = 10
    inst.atkfn = DoAttack
    inst.tryatkfn = TryAttack

    return inst
end

return Prefab( "witch/qw_stem", qw_stem, assets, prefabs),
--Prefab("witch/qw_ghost",qw_ghost,assets,prefabs),
Prefab("witch/qw_man",qw_man,assets,prefabs)
