local function ShakeIfClose(inst)
    local player = GetClosestInstWithTag("player", inst, 40)
    if player then
        player.components.playercontroller:ShakeCamera(inst, "FULL", 0.4, 0.02, 2, 40)
    end
end

local function NeedControlTime()
    local timescale =  GetClock().puella_mod_timescale or 1
    if GetWorld():IsCave() then
        return false
    end
    if GetClock():IsNight() and timescale >= 1 then
        return true
    end
    if not GetClock():IsNight() and timescale <= 1 then
        return true
    end
end

local function ControlTime(vel, percent)
    local night_scale, light_scale = 1,1
    if vel then
        local night_segs = GetClock():GetNightSegs()
        if night_segs < 14 and night_segs > 0 then
            local light_segs = 16 - night_segs
            night_scale = night_segs/14
            light_scale = light_segs/2
        end
    end
    ---渐进---
    night_scale = Lerp(1, night_scale, percent or 1)
    light_scale = Lerp(1, light_scale, percent or 1)
    ----------
    local scale = GetClock():IsNight() and night_scale or light_scale
    GetClock().puella_mod_timescale = scale
    GetPlayer().HUD.controls.clock.anim:GetAnimState():SetDeltaTimeMultiplier(scale)
end

local TIMECONTROL_INTERNAL = 20
local CAST_COOLDOWN = 10
local DISPERSE_COOLDOWN = 10


local states=
{
    State{
        name = "default",
        onenter = function(inst)
            inst:RemoveFromScene()
        end,
        onexit = function(inst)
            inst:ReturnToScene()
        end,
    },
    State{

        name = "idle",
        tags = {"idle", "canrotate"},

        onenter = function(inst, playanim)
            if not inst.AnimState:IsCurrentAnimation('idle') then
                inst.AnimState:PlayAnimation("idle", true)
            end
        end,
    },

    State{
        name = "towerattack",
        tags = {"cast", "busy"},

        onenter = function(inst, para)
            inst.AnimState:PlayAnimation("summon_pre")
            inst.AnimState:PushAnimation("summon_loop")
            
            inst.last_towerattack_time = GetTime()
        end,

        timeline = {
            TimeEvent(1,function(inst)
                if inst.tower then
                    inst.tower:ChargeFn()
                end
            end),
            TimeEvent(5,function(inst)
                if inst.tower then
                    inst.tower:StartAttackBehavior()
                end
            end),
        },

        events={EventHandler("witch_sd: towerattack down", function(inst)inst.sg:GoToState("towerattack_pst")end),},
    },

    State{
        name = "towerattack_pst",
        tags = {"cast", "busy"},

        onenter = function(inst,para)
            inst.AnimState:PlayAnimation("summon_pst")
            inst.last_cast_time = GetTime()
            inst.last_towerattack_time = GetTime()
        end,

        events = {EventHandler("animover",function(inst)inst.sg:GoToState("idle")end)}
    },

    State{
        name = "timecontrol",
        tags = {"busy", "cast", "timecontrol"},
        onenter = function(inst)
            inst.last_timecontrol_time = GetTime()
            inst.AnimState:PlayAnimation('summon_pre')
            inst.AnimState:PushAnimation('summon_loop')
            inst.sg:SetTimeout(5)

            if inst.tower then
                local fx = SpawnPrefab('sd_lavaitem')
                fx.Transform:SetPosition(inst.tower:GetPosition():Get())
                inst.sg.statemem.fx = fx
            end
        end,
        ontimeout = function(inst)
            inst.sg:GoToState('timecontrol_pst')
        end,
        onexit = function(inst)
            inst.sg.statemem.fx:Fade()
        end,
        --events = {EventHandler('homura: lightshock', function(inst)inst.sg:GoToState('disperse')end)},
    },

    State{
        name = "timecontrol_pst",
        tags = {"busy", "cast", "timecontrol"},
        onenter = function(inst)
            inst.AnimState:PlayAnimation('summon_pst')
            inst.last_cast_time = GetTime()
            inst.last_timecontrol_time = GetTime()
            local fx = SpawnPrefab('sd_blastfx')
            if inst.tower then 
                fx:Play(inst.tower)
            else
                fx:Remove()
            end

            GetPlayer().components.talker:Say(PUELLA_MOD_STRINGS:Get(GetPlayer().prefab,"SD_TIMEMAGIC"))

            --inst.SoundEmitter:PlaySound('sd_timemagic_fx') --@---
            inst:StartThread(function()
                local tick = 0
                local total = 60
                while tick < total do
                    if inst.components.health:IsDead() then
                        return true
                    end
                    tick = tick+1
                    inst:ControlTimeFn(true,tick/total)
                    Sleep(0)
                end

                return true
            end)
        end,
        events = {EventHandler('animover',function(inst)inst.sg:GoToState('idle')end)},

    },

    State{
        name = "disperse",
        tags = {"busy"},
        onenter = function(inst)
            inst.AnimState:PlayAnimation('disperse')
            inst.components.health:SetInvincible(true)
            local fx = SpawnPrefab('sd_shadowfx')
            fx.Transform:SetPosition(inst:GetPosition():Get())
            fx.spawnprefabname = 'statue_transition'
            if inst.tower then
                inst.tower:UnchargeFn()
            end
        end,
        onexit = function(inst)
            inst.components.health:SetInvincible(false)
        end,
        events = {EventHandler('animover', function(inst)inst.sg:GoToState('merge')end)}
    },

    State{
        name = "merge",
        tags = {"busy"},
        onenter = function(inst)
            local centerpos = inst.tower and inst.tower:GetPosition() or GetPlayer():GetPosition()
            local offset = FindGroundOffset(centerpos, 6.283*math.random(), math.random(5,10), 10) or Vector3(1,0,1)
            inst.Transform:SetPosition((centerpos+offset):Get())
            inst.AnimState:PlayAnimation('merge')
            local fx = SpawnPrefab('sd_shadowfx')
            fx.Transform:SetPosition(inst:GetPosition():Get())
            fx.spawnprefabname = 'statue_transition_2'
        end,
        events = {EventHandler('animover',function(inst)inst.sg:GoToState('idle')end)},
    },

    State{
        name = "death",
        tags = {"busy"},
        
        onenter = function(inst)
            --inst.SoundEmitter:PlaySound("dontstarve/creatures/spiderqueen/die")
            inst.AnimState:PlayAnimation("death")
           
            inst.components.lootdropper:DropLoot(inst:GetPosition())  
            RemovePhysicsColliders(inst)         
        end,
        
    },

}

local event_handlers=
{
    EventHandler("attacked", function(inst)
        if not inst.components.health:IsDead() and not inst.sg:HasStateTag("busy") and not inst.sg:HasStateTag("attack") then 
            if inst:CanDisperse() then
                inst.sg:GoToState("disperse")
            end 
        end 
    end),

    EventHandler("death", function(inst) inst.sg:GoToState("death") end),
    EventHandler("witch_sd: controltime",function(inst) 
        if not inst.components.health:IsDead() and not inst.sg:HasStateTag("cast") then
            inst.sg:GoToState("timecontrol")
        end
    end),
    EventHandler("witch_sd: towerattack",function(inst)
        if not inst.components.health:IsDead() and not inst.sg:HasStateTag("cast") then
            inst.sg:GoToState("towerattack")
        end
    end),
    EventHandler("homura: lightshock",function(inst)
        if not inst.components.health:IsDead() then
            inst.sg:GoToState("disperse") --强制打断施法
        end
    end),
}
local sg = StateGraph("witch_sd", states, event_handlers, "idle", {})

--shadow_creatures_ground  shadowcreatures
--arm_loop  grab grab_pst hand_in hand_in_loop/ arm_scare hand_scare /scared_loop

local L = PUELLA_MOD_SETTINGS.LANGUAGE 
STRINGS.NAMES.WITCH_SD = L and "Elsa Maria" or "影之魔女"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.WITCH_SD = L and "That's scary." or "这是什么鬼？"
STRINGS.CHARACTERS.KYOUKO.DESCRIBE.WITCH_SD = L and "That's the true witch, not me..." or "哼, 我可以轻而易举地击败她"
STRINGS.CHARACTERS.SAYAKA.DESCRIBE.WITCH_SD = L and "She looks disgusting." or "邪恶又黑暗的家伙"
STRINGS.CHARACTERS.HOMURA_1.DESCRIBE.WITCH_SD = L and "She is cumbersome and can hardly hurt me." or "又是一位...绝望的少女吗？"


local assets=
{
    Asset("ANIM", "anim/witch_sd_tower.zip"),
    Asset("ANIM", "anim/witch_sd.zip"),
}

local function Retarget(inst)
    if not inst.components.health:IsDead() then
        local oldtarget = inst.components.combat.target
        local newtarget = FindEntity(inst, 15, function(guy)return inst.components.combat:CanTarget(guy) and guy:HasTag("player")end)
        
        if newtarget and newtarget ~= oldtarget then
            inst.components.combat:SetTarget(newtarget)
        end
    end
end

local function UpdateFn(inst)
    if inst:CanControlTime() and NeedControlTime() then
        inst:PushEvent('witch_sd: controltime')
    elseif inst:CanTowerAttack() then
        inst:PushEvent('witch_sd: towerattack')
    end
end

local function OnAttacked(inst, data)
    inst.components.combat:SetTarget(data.attacker)
end

local function fn()
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local snd = inst.entity:AddSoundEmitter()
    local shadow = inst.entity:AddDynamicShadow()
    shadow:SetSize( 7, 3 )
    --MakeObstaclePhysics(inst, .9)
    MakeCharacterPhysics(inst,1000, 1.5)
    trans:SetTwoFaced()
    trans:SetScale(.6,.6,.6)

    anim:SetBank("witch_sd")
    anim:SetBuild("witch_sd")
    anim:SetMultColour(1,1,1,0.7)

    inst:AddTag("witch","witch_mn","witch_sd","monster","hostile")

    inst.last_timecontrol_time = 0
    inst.CanControlTime = function(inst) 
        return (GetTime() - inst.last_timecontrol_time > TIMECONTROL_INTERNAL)
        and not inst.components.health:IsDead() and not inst.sg:HasStateTag('cast')
        and inst.components.health:IsHurt()
        and inst:CanCast()
    end
    inst.last_towerattack_time = 0
    inst.CanTowerAttack = function(inst)
        return (true)
        and not inst.components.health:IsDead() and not inst.sg:HasStateTag('cast')
        and inst:CanCast()
    end
    inst.last_cast_time = 0
    inst.CanCast = function(inst)
        return GetTime() - inst.last_cast_time > CAST_COOLDOWN
    end
    inst.last_disperse_time = 0
    inst.CanDisperse = function(inst) --do return false end --@--
        return GetTime() - inst.last_disperse_time > DISPERSE_COOLDOWN 
        and math.random() < 0.333
    end

    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetLoot({"griefseed"})
 
    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(3000)
    inst.components.health.fire_damage_scale = 0.25
    inst.components.health:SetAbsorptionAmount(.15)

    inst:AddComponent("combat")
    inst.components.combat:SetRetargetFunction(5, Retarget)

    inst:ListenForEvent("attacked", OnAttacked)

    inst:AddComponent("inspectable")

    inst:AddComponent("timer")

    inst:AddComponent("leader")
    
    inst:AddTag('homura: lightshock immue')
    inst:ListenForEvent('homura: lightshock',function()
        inst.last_cast_time = GetTime()
        inst.components.health:DoDelta(-50)
    end)
    inst:ListenForEvent('onremove',function() ControlTime(false) end)

    inst:DoPeriodicTask(1, UpdateFn)

    inst.OnSave = function (inst,data)
        data.towerGUID = inst.tower and inst.tower.GUID
        return {data.towerGUID}
    end 

    inst.OnLoadPostPass = function(inst,ents,data)
        print('guid->',data.towerGUID)
        if data and data.towerGUID and ents[data.towerGUID] then
            print('ent->',ents[data.towerGUID])
            inst.tower = ents[data.towerGUID].entity
            ents[data.towerGUID].entity.witch = inst
        end
    end

    inst.ControlTimeFn = function(inst,vel) ControlTime(vel) end

    inst:AddComponent('sd_shadowspawner')
  
    PUELLA_MOD_FN.SetSG(inst,sg)
    
    inst:ListenForEvent('onremove',function()
        if inst.tower and inst.tower:IsValid() then
            inst.tower:DestroyFn()
        end
    end)

    PUELLA_MOD_FN.WitchCommon(inst)

    return inst
end

local function TowerFn()
    local inst,trans,anim,snd = PUELLA_MOD_FN.CreateEntity()
    PUELLA_MOD_FN.SetAnim(inst,'witch_sd_tower','witch_sd_tower','idle')
    anim:Hide('firemarker')
    anim:SetLightOverride(.4)
    MakeObstaclePhysics(inst, .9)

    local shadow = inst.entity:AddDynamicShadow()
    shadow:SetSize( 7, 3 )

    local light = inst.entity:AddLight()

    inst:AddComponent('lighttweener')
    do 
        local old_update = inst.components.lighttweener.OnUpdate
        inst.components.lighttweener.OnUpdate = function(self,dt)
            old_update(self,dt)
            self.inst.AnimState:SetLightOverride(self.light:GetIntensity())
        end
    end

    inst:AddComponent('follower')
    inst:AddComponent('combat')
    trans:SetScale(1.5,1.5,2)

    inst.ChargeFn = function(inst)
        if inst.chargefx then
            return
        end
        snd:PlaySound('dontstarve/cave/nightmare_full')
        inst.chargefx = SpawnPrefab('sd_charge_fx')
        local follower = inst.chargefx.entity:AddFollower()
        follower:FollowSymbol(inst.GUID,'firemarker',0 ,0, 0)
        inst.chargefx.followerdata = {inst.GUID,'firemarker',0,0,0}
        ---(light, rad, intensity, falloff, colour, time, callback)
        inst.components.lighttweener:StartTween(light,15, .95, .9, {1,0,0}, 2.5, function ()
            inst.charged = true
        end)
    end

    inst.UnchargeFn = function(inst)
        if inst.chargefx then
            inst.chargefx:Remove()
            inst.chargefx = nil
        end
        inst.components.lighttweener:StartTween(light,2, .4, .5 ,{1,.8,.8}, 5 ,function()
            inst.charged = false
        end)
        snd:PlaySound('dontstarve/cave/nightmare_end')
        inst.inattack = false
        if inst.witch then
            inst.witch:PushEvent('witch_sd: towerattack down')
        end
    end
 
    inst.AttackFn = function(inst)
        if not inst.chargefx or not inst.charged then
            inst:ChargeFn()
        end
        local ball = SpawnPrefab('sd_ball')
        ball.caster = inst
        ball.instantshoot = true
        local follower = ball.entity:AddFollower()
        follower:FollowSymbol( inst.GUID,'firemarker',-0,0,0 )
    end

    inst.StartAttackBehavior = function(inst)
        if inst.inattack then
            return
        end
        inst.inattack = true
        local phase = GetWorld():IsCave() and 'night' or GetClock():GetPhase()
        if phase == 'day' then
            local shoot = math.random(3,5)
            local interval = 3
            for i = 1,shoot do
                inst:DoTaskInTime(i*interval,inst.AttackFn)
            end
            inst:DoTaskInTime(shoot*interval+1,inst.UnchargeFn)
        elseif phase == 'dusk' then
            local shoot = math.random(5,7)
            local interval = 2
            for i = 1,shoot do
                inst:DoTaskInTime(i*interval,inst.AttackFn)
            end
            inst:DoTaskInTime(shoot*interval+1,inst.UnchargeFn)
        elseif phase == 'night' then
            if math.random(2) == 1 then
                local shoot = math.random(10,15)
                local interval = 1
                for i = 1,shoot do
                    inst:DoTaskInTime(i*interval,inst.AttackFn)
                end
                inst:DoTaskInTime(shoot*interval+1,inst.UnchargeFn)  
            else
                for i = 1,3 do
                    for ii = 1,3 do
                        inst:DoTaskInTime(i*2+ii*0.3,inst.AttackFn)
                    end
                end
                inst:DoTaskInTime(11,inst.UnchargeFn)
            end
        end
    end

    inst.TimecontrolChargeFn = function(inst)
        --
    end

    --[[
    local function ringfx()
        local inst = CreateEntity()
        local trans = inst.entity:AddTransform()
        local anim = inst.entity:AddAnimState()
        local snd = inst.entity:AddSoundEmitter()
        PUELLA_MOD_FN.SetAnim(inst,"griefseed_fx","griefseed_fx","blast",false,true)

        trans:SetScale(5,5,5)
        anim:SetMultColour(0.7,0.2,0.2,0.5)

        inst:ListenForEvent("animover",inst.Remove)   

        return inst
    end
    --]]
    inst.DestroyFn = function(inst)
        anim:PlayAnimation("")
        if inst.charged then
            inst:UnchargeFn()
        end

        inst:ListenForEvent("animover",inst.Remove)
        inst:Remove()--@--
    end

    ----
    inst:DoTaskInTime(0,function()
        if not inst.witch then
            local witch = SpawnPrefab('witch_sd')
            inst.witch = witch
            witch.tower = inst
            witch.sg:GoToState('merge')
            --witch.Transform:SetPosition(inst:GetPosition():Get())
        end
    end)

    inst.OnSave =  function(inst, data)
        return {inst.GUID}
    end

    return inst
end

return Prefab( "witch/witch_sd", fn, assets),
Prefab('witch_sd_tower',TowerFn,assets)--,Prefab('cc',TowerFn,assets)