require("stategraphs/commonstates")
local SGtentacle = nil
do
local events = 
{
    CommonHandlers.OnAttack(),
    CommonHandlers.OnAttacked(),
    CommonHandlers.OnDeath(),
    EventHandler("despawn", function(inst) inst.sg:GoToState("despawn") end),
}

local actionhandlers = {}

local states = 
{
    State{
        name = "idle",
        tags = {"idle", "canrotate"},

        onenter = function(inst, playanim)
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("idle_slow",true)
        end,
    },

    State{
        name = "spawn",
        tags = {"busy"},

        onenter = function(inst, playanim)
            inst.SoundEmitter:PlaySound("dontstarve_DLC002/creatures/quacken/tentacle_emerge")
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("enter")
        end,
        
        events={EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),},
    },

    State{
        name = "despawn",
        tags = {"busy"},

        onenter = function(inst, playanim)
            inst.SoundEmitter:PlaySound("dontstarve_DLC002/creatures/quacken/tentacle_submerge")
            inst.Physics:Stop()
            inst.AnimState:PlayAnimation("exit")
        end,
        
        events={EventHandler("animover", function(inst) inst:Remove() end),},
    },

    State{
        name = "death",
        tags = {"busy"},
        
        onenter = function(inst)
            inst.AnimState:PlayAnimation("exit")
            if inst.components.lootdropper then
                inst.components.lootdropper:DropLoot()
            end
        end,
    },

    State{
        name = "hit",
        tags = {"busy", "hit"},
        
        onenter = function(inst)
            inst.AnimState:PlayAnimation("hit")
            inst.SoundEmitter:PlaySound("dontstarve_DLC002/creatures/quacken/hit")
        end,
        
        events ={EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),},
    },

    State{
        name = "attack",
        tags = {"busy", "attack", "canrotate"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("attack")
            inst.components.combat:StartAttack()
        end,

        timeline =
        {
            TimeEvent(17*FRAMES, function(inst) 
                inst.components.combat:DoAttack() 
                inst.SoundEmitter:PlaySound("dontstarve_DLC002/creatures/quacken/attack")
            end),
            TimeEvent(20*FRAMES, function(inst) inst.sg:RemoveStateTag("attack") end),
        },

        events={EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),},
    },
}
SGtentacle = StateGraph("sd_tentacle_big", states, events, "idle", actionhandlers)
end
--重建sg是为了防止播放音效及释放海浪

local SGshadowhand
do
local events = 
{
    CommonHandlers.OnAttack(),
    --CommonHandlers.OnAttacked(),
    CommonHandlers.OnDeath(),
    EventHandler("dissipate", function(inst) inst.sg:GoToState("dissipate") end),
}
local actionhandlers = {}
local states =
{
    State{
        name = "spawn",
        tags = {"idle"},
        onenter = function(inst)
            inst.AnimState:PlayAnimation("hand_in")
        end,
        events={EventHandler("animover", function(inst) inst.sg:GoToState("idle") end),},
    },
    State{
        name = "idle",
        tags = {"idle"},
        onenter = function(inst)
            inst.AnimState:PlayAnimation('hand_in_loop',true)
        end,
    },

    State{
        name = "attack",
        tags = {"busy"},
        onenter = function(inst)
            inst.AnimState:PlayAnimation("grab")
            inst.AnimState:PushAnimation("grab_pst",false)
        end,
        timeline=
        {
            TimeEvent(17*FRAMES, function(inst) 
                inst.components.combat:DoAttack() 
                inst.SoundEmitter:PlaySound("dontstarve_DLC002/creatures/quacken/attack")
            end),
            TimeEvent(20*FRAMES, function(inst) inst.sg:RemoveStateTag("attack") end),
        },
        events={EventHandler("animqueueover", function(inst) inst.sg:GoToState("idle") end),},
    },

    State{
        name = "dissipate",
        tags = {"death","busy",},
        onenter = function(inst)
            inst.AnimState:PlayAnimation("hand_scare")
            if inst.components.playerprox then
                inst:RemoveComponent("playerprox")
            end
            if inst.arm then
                inst.arm.AnimState:PlayAnimation("arm_scare")
            end
        end,
        events={EventHandler("animover", function(inst) inst:Remove() end),},
    },

}
SGshadowhand = StateGraph("sd_shadowhand", states, events, "spawn", actionhandlers)
end

--shadow_creatures_ground  shadowcreatures
--  grab grab_pst hand_in hand_in_loop/ hand_scare /scared_loop

--shadow_skittish  shadowcreatures
--disappear  idle_loop
local assets =  
{
    Asset("ANIM", "anim/shadow_skittish.zip"),
} 


local L = PUELLA_MOD_SETTINGS.LANGUAGE 
STRINGS.NAMES.WITCH_SD = L and "Elsa Maria" or "影之魔女"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.WITCH_SD = L and "That's scary." or "这是什么鬼？"
STRINGS.CHARACTERS.KYOUKO.DESCRIBE.WITCH_SD = L and "That's the true witch, not me..." or "明明这才是真正的魔女..."
STRINGS.CHARACTERS.SAYAKA.DESCRIBE.WITCH_SD = L and "She looks disgusting." or "呃，看起来好恶心。"
STRINGS.CHARACTERS.HOMURA_1.DESCRIBE.WITCH_SD = L and "She is cumbersome and can hardly hurt me." or "她很笨重，根本伤不到我。"

local function AddDarkness(inst,stack)
    if not inst.components.witchsd_darkness_debuff and inst.components.combat then
        inst:AddComponent('witchsd_darkness_debuff')
    end
    inst.components.witchsd_darkness_debuff:AddDebuff(stack)
end

local function SetDarknessAttack(inst,stack)
    inst:ListenForEvent('onhitother',function(i,data) AddDarkness(data.target,stack) end)
end

local function skittish()  --影子小鬼
    local inst = SpawnPrefab('shadowskittish')
    inst.prefab = 'sd_skittish'
    MakeCharacterPhysics(inst, 100, .5)
    RemovePhysicsColliders(inst)
    
    inst:RemoveComponent('transparentonsanity')
    inst:RemoveComponent('playerprox')
    inst.AnimState:PlayAnimation("idle_loop", true)
    inst.AnimState:SetMultColour(1,1,1,0)
    inst.AnimState:SetScale((-1)^math.random(2),1)
    inst.Transform:SetScale(.7,.7,.7)
    inst.deathtask:Cancel()
    inst:RemoveTag("NOCLICK")

    inst:AddTag("witch","witch_sm","witch_sd","hostile")

    inst:AddComponent('health')
    inst.components.health:SetMaxHealth(60)

    inst:AddComponent('combat')

    inst:AddComponent('follower')

    inst.fadein = inst:DoPeriodicTask(0,function()
        local _,_,_,a = inst.AnimState:GetMultColour()
        inst.AnimState:SetMultColour(1,1,1,a+FRAMES)
        if a >= 0.5 then 
            inst.fadein:Cancel()
        end
    end)

    inst:ListenForEvent('death',function() 
        inst.death = true
        inst.fadein:Cancel()
        inst.AnimState:PlayAnimation('disappear')
    end)

    inst:DoTaskInTime(20+math.random(20),function(inst)if not inst.death then inst.components.health:Kill()end end)
    
    return inst
end


        --inst.AnimState:PlayAnimation("grab")
        --inst.AnimState:PushAnimation("grab_pst", false)

local function MakeArm(hand)
    local inst = SpawnPrefab('shadowhand_arm')
    inst.persists = true
    inst:RemoveAllEventCallbacks()
    inst.hand = hand
    hand.arm  = inst

    inst:ListenForEvent("onremove",function()inst:Remove()end,hand)
    inst.components.stretcher:SetStretchTarget(hand)

    inst:DoPeriodicTask(0,function()
        if inst.hand then
            inst:FacePoint(inst.hand:GetPosition())
        end
    end)
    inst:DoTaskInTime(0,function()
        if inst.hand then
            inst.Transform:SetPosition(inst.hand:GetPosition():Get())
        end
    end)
    ---arm_scare   arm_loop
    return inst
end

local function shadowhand()
    local inst = CreateEntity()
    inst:AddTag("shadowhand")
    
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.entity:AddPhysics()
    inst.entity:AddLightWatcher()
    inst.LightWatcher:SetLightThresh(.2)
    inst.LightWatcher:SetDarkThresh(.19)

    MakeCharacterPhysics(inst, 10, .1)
    RemovePhysicsColliders(inst)

    inst.dissipatefn = Dissipate
    inst:ListenForEvent( "daytime", inst.dissipatefn, GetWorld())
    
    inst.AnimState:SetBank("shadowcreatures")
    inst.AnimState:SetBuild("shadow_creatures_ground")
    inst.AnimState:SetOrientation( ANIM_ORIENTATION.OnGround )
    inst.AnimState:SetLayer( LAYER_BACKGROUND )
    inst.AnimState:SetSortOrder( 3 )
    inst.AnimState:PlayAnimation("hand_in")
    inst.AnimState:PushAnimation("hand_in_loop", true)

    inst:AddComponent("playerprox")
    inst.components.playerprox:SetDist(2,6)----------
    inst.components.playerprox:SetOnPlayerNear(Retreat)-------
    inst.components.playerprox:SetOnPlayerFar(Regroup)------

    inst:AddComponent("locomotor")
    inst.components.locomotor.walkspeed = 2
    inst.components.locomotor.directdrive = true
    inst.components.locomotor.slowmultiplier = 1
    inst.components.locomotor.fastmultiplier = 1

    inst:AddComponent("sanityaura")
    --inst.components.sanityaura.aura = -TUNING.SANITYAURA_MED

    inst.OnRemoveEntity = function(inst)inst.SoundEmitter:KillAllSounds()end
    return inst
end

local function tentacle_big() --外周触手墙(原始:海妖触手)
    local inst = SpawnPrefab('kraken_tentacle')
    MakeObstaclePhysics(inst, .6)

    inst.Transform:SetScale(.7,.7,.7)

    inst.AnimState:SetMultColour(0,0,0,0.7)

    local hiden_symbol = {'column','droplet','droplet_single','ripple2','sidesplash','splash',
    'splash_cone','tentacle_fade',}
    for _,v in pairs(hiden_symbol)do
        inst.AnimState:OverrideSymbol(v,'kyouko','')
    end

    local minimap = inst.entity:AddMiniMapEntity()
    minimap:SetIcon("")

    inst:RemoveTag("kraken")
    inst:RemoveTag("epic")
    inst:AddTag('witch','witch_sm','witch_sd','hostile')
 
    PUELLA_MOD_FN.SetSG(inst,SGtentacle)

    inst.components.health:SetMaxHealth(300)
    inst.components.health:SetAbsorptionAmount(.25)

    inst.components.combat:SetRange(4)
    inst.components.combat:SetDefaultDamage(20)
    inst.components.combat:SetAttackPeriod(6)
    inst.components.combat:SetAreaDamage(4)
    inst.components.combat.notags = {'witch'}

    inst:RemoveComponent("lootdropper")

    inst:AddComponent('follower')

    SetDarknessAttack(inst,20)

    return inst
end

local function tentacle_mid() --普通触手
    local inst = SpawnPrefab('tentacle')
    inst:AddTag('witch')
    inst:AddTag('witch_sd')
    inst:AddTag('witch_sm')
    inst.AnimState:SetMultColour(0,0,0,0.7)

    inst:RemoveComponent('lootdropper')
    inst:AddComponent("lootdropper") --sg内没有组件检测
    inst:RemoveComponent('freezable')

    inst.components.combat:SetDefaultDamage(15)
    inst.components.combat:SetAreaDamage(4)
    inst.components.combat.notags = {'witch'}

    inst:AddComponent('follower')

    SetDarknessAttack(inst,20)
    
    return inst
end

local function tentacle_small() --小触手
    local inst = SpawnPrefab('tentacle_pillar_arm')

    inst:AddTag('witch')
    inst:AddTag('witch_sd')
    inst:AddTag('witch_sm')
    inst:AddTag('hostile')
    inst.persists = true
    inst.AnimState:SetMultColour(0,0,0,0.7)

    inst:AddComponent('follower')

    inst.components.combat:SetDefaultDamage(10)
    inst.components.combat:SetAreaDamage(3)
    inst.components.combat.notags = {'witch'}

    SetDarknessAttack(inst,20)
    return inst
end

local function attackroot()
    local inst = SpawnPrefab('deciduous_root')
    inst.AnimState:SetMultColour(0,0,0,.7)
    inst:RemoveTag('birchnutroot')

    inst.components.combat:SetAreaDamage(3.7)
    inst.components.combat:SetDefaultDamage(10)
    inst.components.combat.notags = {'witch'}

    SetDarknessAttack(inst,20)

    return inst
end

local function Retarget(inst)
    if not inst.components.health:IsDead()  then
        local oldtarget = inst.components.combat.target
        local newtarget = FindEntity(inst, 10, function(guy) 
            if inst.components.combat:CanTarget(guy) then
                return guy:HasTag("character")
            end
        end)
        if newtarget and newtarget ~= oldtarget then
            inst.components.combat:SetTarget(newtarget)
        end
    end
end

local function OnAttacked(inst, data)
    inst.components.combat:SetTarget(data.attacker)
    inst.components.combat:ShareTarget(data.attacker, 20, function(dude) 
        return dude:HasTag("witch_sd") and not dude.components.health:IsDead() 
    end, 2)
end


return Prefab( "sd_skittish", skittish, assets),
Prefab("sd_tentacle_big",tentacle_big, assets),
Prefab("sd_tentacle_mid",tentacle_mid, assets),
Prefab("sd_tentacle_small",tentacle_small, assets),
Prefab("sd_attackroot",attackroot,assets),
Prefab("sd_hand",shadowhand, assets)
