local AnimSpeedData = {
    [10] = 0.75,
    [11] = 0.75,
}

local function SetTVSymbol(inst,num)
    inst.AnimState:ClearOverrideSymbol("anim")
    if num then
        inst.AnimState:OverrideSymbol("anim","witch_tv".. num ,"tv")
        inst.AnimState:SetDeltaTimeMultiplier(AnimSpeedData[num] or 1)
    else
        inst.AnimState:OverrideSymbol("anim","witch_tv_nosignal", "tv")
        inst.AnimState:SetDeltaTimeMultiplier(1)
    end
end

local function MakeTeleportFx(pos, stage)
    local inst, trans, anim = PUELLA_MOD_FN.CreateEntity()
    anim:SetBank('wilson')
    anim:SetBuild('player_wagstaff_pmmm')
    anim:PlayAnimation(({'teleport_in','teleport_out','teleport_finish'})[stage])
    anim:SetBloomEffectHandle("shaders/anim.ksh")
    anim:Hide('flash01')
    anim:Hide('glow01')
    anim:SetLayer(LAYER_WORLD_CEILING)
    trans:SetPosition(pos:Get())

    inst:AddTag('NOCLICK','NOBLOCK','FX','ignoretimemagic')
    inst:ListenForEvent('animover', inst.Remove)

    return inst
end


local states=
{   
    State{
        name = "still", --静止，地面
        tags = {"idle","canrotate"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("still")
            inst.Light:Enable(false)
        end,
    },
    State{
        name = "nosignal",  --雪花屏
        tags = {"idle","canrotate"},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("funnyidle2")
            inst.Light:Enable(true)
            SetTVSymbol(inst)
        end,
        events = 
        {
            EventHandler("animover",function(inst)
                --if inst:HasTag("TV")then
                if 1 then
                    inst.sg:GoToState("funnyidle")
                else
                    inst.sg:GoToState("nosignal")
                end
            end),
        },
    },
    State{
        name = "funnyidle", 
        tags = {"idle","canrotate"},

        onenter = function(inst) 
            local function isSame(a, b)
                if a == b then return true end
                if (a == 10 or a == 11) and (b == 10 or b == 11) then return true end
            end
            local new = inst.channelID
            --while new == inst.channelID do
            while isSame(new, inst.channelID) do
                new = math.random(inst.MAX_TVSYMBOL_ID)
            end
            --@--debug
            --new = 10
     
            inst.channelID = new
            inst.AnimState:PlayAnimation("funnyidle2",true)
            inst.Light:Enable(true)
            SetTVSymbol(inst,inst.channelID)
            
            --连续动画处理
            if new == 10 or new == 11 then
                inst.sg:GoToState('funnyidle_long', 10)
                return
            end
        end,

        timeline=  --共10.6秒，318帧
        {
            TimeEvent(3*FRAMES, function (inst) SetTVSymbol(inst) end),
            TimeEvent(6*FRAMES, function (inst) SetTVSymbol(inst,inst.channelID) end),
            TimeEvent(10*FRAMES, function (inst) SetTVSymbol(inst) end),
            TimeEvent(15*FRAMES, function (inst) SetTVSymbol(inst,inst.channelID) end),
            
            TimeEvent(305*FRAMES, function (inst) SetTVSymbol(inst) end),
            TimeEvent(310*FRAMES, function (inst) SetTVSymbol(inst,inst.channelID) end),
            TimeEvent(315*FRAMES, function (inst) SetTVSymbol(inst) end),

            TimeEvent(316*FRAMES, function (inst) inst.sg:SetTimeout(GetRandomMinMax(10, 20))end)
        },
        ontimeout= function (inst)
            inst.sg:GoToState("funnyidle")
        end,
    },

    State{
        name = "funnyidle_long", 
        tags = {"idle","canrotate"},
        onenter = function(inst, id)
            if id == 10 then
                inst.sg.statemem.fadein = true
                inst.sg.statemem.fadeout = false
            end
            if id == 11 then
                inst.sg.statemem.fadein = false
                inst.sg.statemem.fadeout = true
            end
            inst.sg.statemem.id = id
            inst.channelID = id

            inst.AnimState:PlayAnimation("funnyidle2",true)
            inst.Light:Enable(true)
            SetTVSymbol(inst,inst.channelID)
        end,

        timeline=
        {
            TimeEvent(3*FRAMES, function (inst) if inst.sg.statemem.fadein then SetTVSymbol(inst) end end),
            TimeEvent(6*FRAMES, function (inst) SetTVSymbol(inst,inst.channelID) end),
            TimeEvent(10*FRAMES, function (inst) if inst.sg.statemem.fadein then SetTVSymbol(inst) end end),
            TimeEvent(15*FRAMES, function (inst) SetTVSymbol(inst,inst.channelID) end),
            
            TimeEvent(305*FRAMES, function (inst) if inst.sg.statemem.fadeout then SetTVSymbol(inst) end end),
            TimeEvent(310*FRAMES, function (inst) SetTVSymbol(inst,inst.channelID) end),
            TimeEvent(315*FRAMES, function (inst) if inst.sg.statemem.fadeout then SetTVSymbol(inst) end end),

            TimeEvent(316*FRAMES, function (inst) 
                if inst.sg.statemem.fadeout then 
                    inst.sg:SetTimeout(GetRandomMinMax(10, 20))
                else
                    inst.sg:GoToState('funnyidle_long', inst.channelID + 1)
                end 
            end),
        },
        ontimeout= function (inst)
            inst.sg:GoToState("funnyidle")
        end,
    },
    State{
        name = 'teleport_in',
        tags = {'busy','teleport'},

        onenter = function(inst,pos)
            MakeTeleportFx(inst:GetPosition(),1)
            inst.sg.statemem.pos = pos
            inst:AddTag('ignoretimemagic')
            inst.SoundEmitter:PlaySound('lw_witch/tv/teleport',nil, 0.7)
        end,
        timeline = {
            TimeEvent(15*FRAMES, function(inst) inst:Hide() inst.components.tv_ballspawner:Hide() end),
            TimeEvent(16*FRAMES, function(inst) inst.sg:GoToState('teleport_out',inst.sg.statemem.pos) end),
        },
        onexit = function(inst)
            inst:RemoveTag('ignoretimemagic')
        end,
    },

    State{
        name = 'teleport_out',
        tags = {'busy','teleport'},

        onenter = function(inst,pos)
            inst.Transform:SetPosition(pos:Get())
            MakeTeleportFx(inst:GetPosition(),1)
            inst.sg:SetTimeout(15*FRAMES)
            
            inst:AddTag('ignoretimemagic')
        end,
        ontimeout = function(inst)
            inst:Show()
            inst.components.tv_ballspawner:Show()
            inst.sg:GoToState('funnyidle')
        end,
        onexit = function(inst)
            inst:RemoveTag('ignoretimemagic')
        end,
    },
    
    State{
        name = "death",
        tags = {"busy","idle","death"},
        
        onenter = function(inst)
            inst.AnimState:PlayAnimation("death")
            inst.components.lootdropper:DropLoot(inst:GetPosition())  
            for k,_ in pairs(inst.components.tv_ballspawner.minion)do
                k:PushEvent("remove")
            end
            if inst.floattask then
                inst.floattask:Cancel()
            end
            MakeInventoryPhysics(inst)
            inst.Light:Enable(false)
            --inst:Remove()
        end,
        timeline = 
        {
            TimeEvent(1*FRAMES,function(inst)
                SpawnPrefab('shock_machines_fx').Transform:SetPosition(inst:GetPosition():Get())
                inst.SoundEmitter:PlaySound("dontstarve_DLC002/creatures/jellyfish/electric_land")
            end)
        },
        
    },

}

local event_handlers=
{
    --EventHandler("attacked",function (inst)
    --    if not inst.sg:HasStateTag("attack")and not inst.sg:HasStateTag("death")then
    --        inst.sg:GoToState("hit")
    --    end
    --end),
    EventHandler("death", function (inst) inst.sg:GoToState("death")end),
    EventHandler("teleport",function(inst,data)
        if not inst.sg:HasStateTag('death') and not inst.sg:HasStateTag('teleport') and data and data.pos then
            inst.sg:GoToState('teleport_in',data.pos)
        end
    end),

}
local sg = StateGraph("witch_tv", states, event_handlers, "nosignal", {})


local L = PUELLA_MOD_SETTINGS.LANGUAGE 
STRINGS.NAMES.WITCH_TV = L and "Kirsten" or "箱之魔女"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.WITCH_TV = L and "That's... scary." or "这是什么鬼？"
STRINGS.CHARACTERS.KYOUKO.DESCRIBE.WITCH_TV = L and "That's the witch, not me..." or "明明这才是真正的魔女..."

local assets=
{
    Asset("ANIM", "anim/witch_tv_nosignal.zip"),
    Asset("ANIM", "anim/witch_tv.zip"),
    Asset("ANIM", "anim/witch_tv_hit.zip"),
}

local MAX_TVSYMBOL_ID = 1
while softresolvefilepath('anim/witch_tv'..MAX_TVSYMBOL_ID..'.zip') do
    table.insert(assets, Asset("ANIM", "anim/witch_tv"..MAX_TVSYMBOL_ID..".zip"))
    MAX_TVSYMBOL_ID = MAX_TVSYMBOL_ID + 1
end

local function OnAttacked(inst, data)
    --inst:Remove()
    inst.components.combat:SetTarget(data.attacker)
    if not inst.floattask then
        local maxheight = .6
        local rate = 0.1
        inst.floattask = inst:DoPeriodicTask(0,function()
            local pos = inst:GetPosition()
            inst.heightx = inst.heightx + rate
            local y = (math.sin(inst.heightx)+1)*maxheight/2 --print(y)
            inst.Transform:SetPosition(pos.x,y,pos.z)
            if inst.components.combat.target == nil then
                inst.Transform:SetPosition(pos.x,0,pos.z)
                inst.floattask:Cancel();inst.floattask =nil;inst.heightx = 0
            end
        end)
    end
    inst.damagevalue = 3
    inst.AnimState:OverrideSymbol("summon","witch_tv_hit","hit"..math.random(6)-1)
end

local function hitfxtask(inst)
    if not inst.components.health:IsHurt()then return end
    if inst.damagevalue < -GetRandomMinMax(10*30,20*30)*inst.components.health:GetPercent() then
        inst.damagevalue = 3
        inst.AnimState:OverrideSymbol("summon","witch_tv_hit","hit"..math.random(6)-1)
    --elseif inst.damagevalue > 0 then
    elseif inst.damagevalue <= 0 then
        inst.AnimState:ClearOverrideSymbol("summon")
    end      
    inst.damagevalue = inst.damagevalue - 1
end

local function onhealthdelta(inst,data)
    local p = data and data.newpercent
    local oldstate = inst.state
    if p then
        if  p <= 0.2 then
            inst.state = 3
        elseif p <= 0.5 then
            inst.state = 2
        else
            inst.state = 1
        end
    end
    if oldstate ~= inst.state then
        inst:AngryFn()
    end
end

local function onangry(inst)
    inst.components.tv_ballspawner:ForceShoot()
end

local function sparktask(inst)
    inst.sparktask = inst:DoTaskInTime(GetRandomMinMax(3,5),sparktask)
    if GetSeasonManager():IsRaining() or inst:GetIsWet() then
        inst.components.health:DoDelta(-0.1,false,"water")
        local spark = SpawnPrefab("sparks")
        spark.Transform:SetPosition(inst:GetPosition():Get())
    end
end

local function fn()
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local snd = inst.entity:AddSoundEmitter()
    local shadow = inst.entity:AddDynamicShadow()
    local light = inst.entity:AddLight()
    shadow:SetSize( 7, 3 )
   
    MakeCharacterPhysics(inst,500, 1.5)
    trans:SetFourFaced()
    trans:SetScale(1.2,1.2,1.2)

    light:Enable(false)
    light:SetIntensity(.5)--光强
    light:SetColour(.5,.5,.5)
    light:SetFalloff(.5) --衰减
    light:SetRadius(3) --半径

    anim:SetBank("witch_tv")
    anim:SetBuild("witch_tv")
    
    inst:AddComponent("lootdropper")
    inst.components.lootdropper:SetLoot({"griefseed",})
 
    inst:AddComponent("health")
    inst.components.health:SetMaxHealth(4000)
    inst.components.health:SetAbsorptionAmount(.15)
    inst.components.health.fire_damage_scale = 0

    inst:AddComponent("combat")
    inst.components.combat:SetDefaultDamage(1024)
    inst.components.combat.hurtsound = "dontstarve/creatures/bishop/hurt"

    inst:AddComponent("inspectable")

    inst:AddComponent("timer")

    inst:AddComponent("tv_ballspawner")

    inst:AddTag("witch","witch_mn","witch_tv")
    inst:AddTag("epic")
    inst:AddTag("hostile")
    inst:AddTag('canignorearmor')

    inst.state = 1
    inst.AngryFn = onangry

    inst.damagevalue = 0
    inst.hitfxtask = inst:DoPeriodicTask(0,hitfxtask)

    inst.MAX_TVSYMBOL_ID = MAX_TVSYMBOL_ID

    PUELLA_MOD_FN.SetSG(inst,sg)

    inst:ListenForEvent("healthdelta",onhealthdelta)
    inst:ListenForEvent("attacked", OnAttacked)

    inst.sparktask = inst:DoTaskInTime(0,sparktask)

    inst.heightx = 0

    PUELLA_MOD_FN.WitchCommon(inst, function(inst) return inst.components.combat.target == GetPlayer() end)

    return inst
end

return Prefab( "witch/witch_tv", fn, assets)
