local states=
{
    State{

        name = "idle",
        tags = {"idle",},

        onenter = function(inst)
            inst.AnimState:PlayAnimation("anim", true)
        end,

    },

    State{
        name = "attack",
        tags = {"attack"},

        onenter = function(inst)
            inst.sg:SetTimeout(0.1)
            inst.circle_atk_fn(inst)
        end,

        onexit = function (inst)
            --inst.sg:GoToState("remove")
        end,
    },

    State{
        name = "attack_pre",
        tags = {"attack",},
        
        onenter = function(inst)
            inst.sg:GoToState("attack")
            --inst.SoundEmitter:PlaySound(inst.atksound)
            --inst.AnimState:PlayAnimation("atk_pre")        
        end,
        
        onexit = function (inst)
        end,
        events =
        {
            --EventHandler("animover", function(inst)inst.sg:GoToState("attack") end),
        },
        
    },
    
    State{
        name = "remove",
        tags = {"busy","idle","death"},
        
        onenter = function(inst) 
            if inst.hitfx_fn then
                inst:hitfx_fn()
            end
            if inst.center then
                inst.center.components.tv_ballspawner.minion[inst] = nil
            end
            inst:Remove()
        end,   
    },
}

local event_handlers=
{

    EventHandler("remove", function(inst) inst.sg:GoToState("remove") end),
    EventHandler("attack", function(inst)
        if not inst.sg:HasStateTag("attack")and not inst.sg:HasStateTag("death")then
            inst.sg:GoToState("attack_pre")
        end
    end),

}
local sg = StateGraph("witch_tv_ball", states, event_handlers, "idle", {})

local assets=
{
    Asset("ANIM", "anim/witch_tv_ball.zip"),
    Asset("ANIM", "anim/witch_tv_red.zip"),
    Asset("ANIM", "anim/witch_tv_green.zip"),
    Asset("ANIM", "anim/witch_tv_blue.zip"),
    Asset("ANIM", "anim/frozen_shatter.zip"),
    Asset("ANIM", "anim/bishop_attack.zip"), 
}
local prefabs = 
{
}
local ice_dmg = 20
local ice_range1,ice_range2 = 2, 2.5
local ice_colour = {0,0,1}

local fire_dmg = 20
local fire_range1,fire_range2 = 2, 2.5
local fire_colour = {1,0,0}

local poison_dmg = 5
local poison_range1,poison_range2 = 2, 4
local poison_colour = {0,1,0}

local zap_dmg,zap_dmg2 = 2.5,30
local zap_range1,zap_range2 = 2, 2.5
local zap_colour = {1,1,1}

local simply_dmg = 50
local simply_range1,simply_range2 = 3, 4

local function SetLight(inst,colour)
    inst.entity:AddLight()
    inst.Light:Enable(false)
    inst.Light:SetIntensity(.5)--光强
    inst.Light:SetColour(colour[1],colour[2],colour[3])
    inst.Light:SetFalloff(.5) --衰减
    inst.Light:SetRadius(1) --半径
end
------------------------------------------------------
local function PushAOE(inst,range,fn)
    local pos = inst:GetPosition()
    local ents = TheSim:FindEntities(pos.x,0,pos.z,range,nil,{"INLIMBO","witch"})
    for k,v in pairs(ents) do 
        if v ~= inst then
            fn(v)
        end
    end
end

local function HitBoat(targ,dmg)
    local boathealth = targ.components.driver and targ.components.driver.vehicle and targ.components.driver.vehicle.components.boathealth
    if boathealth and boathealth:IsInvincible() == false then
        if dmg == 0 then
            return true
        end
        boathealth:DoDelta(-dmg, "combat", "NIL")
        targ:PushEvent("boatattacked",{damage = dmg})
        return true
    end
    if targ.components.health and targ.components.health:IsInvincible()then
        return true 
    end
end

local function CreateTail(bank, build)
    local inst = CreateEntity()

    inst:AddTag("FX")
    inst:AddTag("NOCLICK")
    inst.persists = false

    inst.entity:AddTransform()
    inst.entity:AddAnimState()

    MakeInventoryPhysics(inst)
    inst.Physics:ClearCollisionMask()

    inst.AnimState:SetBank(bank)
    inst.AnimState:SetBuild(build)
    inst.AnimState:PlayAnimation("tail")

    inst.AnimState:SetFinalOffset(-1)

    inst:ListenForEvent("animover", inst.Remove)

    return inst
end

local function UpdateTail(inst, bank, build, tails,tailfn)  
    if inst:IsAsleep() then
        inst:Remove()
        inst:PushEvent("forceremove")
    end
    local x, y, z = inst.Transform:GetWorldPosition()
    for tail, _ in pairs(tails) do
        tail:ForceFacePoint(x, y, z)
    end
    --if not GetWorld():HasTag("homuraTag_pause") then
        local speed = inst.components.acceleratingprojectile:GetPhysicsSpeed()
        if speed == 0 then 
            return
        end
        local tail = CreateTail(bank, build)
        local rot = inst.Transform:GetRotation()
        tail.Transform:SetRotation(rot)
        rot = rot * DEGREES
        local offsangle = math.random() * 6.283
        local offsradius = GetRandomMinMax(.2,.4)
        local hoffset = math.cos(offsangle) * offsradius
        local voffset = math.sin(offsangle) * offsradius
        tail.Transform:SetPosition(x + math.sin(rot) * hoffset, y + voffset, z + math.cos(rot) * hoffset)
        tail.Physics:SetMotorVel(speed * GetRandomMinMax(.2,.5), 0, 0)
        tails[tail] = true
        inst:ListenForEvent("onremove", function(tail) tails[tail] = nil end, tail)
        tail:ListenForEvent("onremove", function(inst)
            tail.Transform:SetRotation(tail.Transform:GetRotation() + GetRandomMinMax(-15,15))
        end, inst)
        tail:ListenForEvent("forceremove",function()tail:Remove()end,inst)
        if tailfn then
            tailfn(tail)
        end
    --end
end
----------------------------------------------------------
local function blue_circle_atk(inst)
    local function Freeze(targ)
        local dmg = ice_dmg
        if not targ:HasTag("player")then
            dmg = dmg * 3
        end
        if HitBoat(targ,dmg)then
            return
        end
        local freezable = targ.components.freezable
        if not targ.components.health or targ.components.health:IsDead() or targ.components.health:IsInvincible() then
            return
        end
        if freezable and not targ.components.health:IsDead()  then
            if freezable:IsFrozen()then
                if targ.components.combat then
                    targ.components.combat:GetAttacked(inst.center, dmg, inst)
                end
                freezable:Unfreeze()
            else
                freezable:Freeze()
            end
        end
    end
    inst:PushEvent("remove")
    PushAOE(inst,ice_range1,Freeze)
end

local function simply_circle_atk(inst) ----------
end

----------------------------------------------------------------------------------------

local function blue_circle(inst)
    local function Freeze(targ)     --为附近物体增加冰冻层数，若存在被冰冻物体，触发攻击
        if HitBoat(targ,0)then
            return
        end
        local freezable = targ.components.freezable
        if freezable then
            if freezable:IsFrozen() and not targ.components.health:IsDead() then
                inst:PushEvent("attack")     
            else
                freezable:AddColdness(3)
                --freezable:SpawnShatterFX()
            end
        end
    end
    PushAOE(inst,ice_range1,Freeze)
end

local function green_circle(inst)   --施加中毒
    local function Poison(targ)
        local equipslots = targ.components.inventory and targ.components.inventory.equipslots
        if equipslots then
            for k,v in pairs (equipslots) do
                if v.components.equippable and v.components.equippable:IsPoisonGasBlocker() then
                    return
                end     
            end
        end
        if targ.components.health and targ.components.health.DoGasDamage then
            targ.components.health:DoGasDamage(5)            
            targ:PushEvent("poisondamage")   
            if targ:HasTag("player") then 
                targ.components.talker:Say(GetString(targ.prefab, "ANNOUNCE_GAS_DAMAGE"))
            end
        end
        if targ.components.poisonable then
            targ.components.poisonable:Poison(true)
        end
    end
    PushAOE(inst,poison_range1,Poison)
end

local function lightning_circle(inst)  --造成微弱的闪电伤害，控场
    local dmg = zap_dmg  
    local function Zap(targ)
        if GetMoistureManager():IsEntityWet(targ) then
            dmg = dmg * 2
        elseif targ.components.moisture then
            dmg = dmg * (targ.components.moisture.moisture/66+1)
        end
        if not targ:HasTag("player")then
            dmg = dmg * 3
        end
        if targ.components.inventory and targ.components.inventory:IsInsulated()then
            return
        end
        if targ.components.health and(targ.components.health:IsDead() or targ.components.health:IsInvincible())then
            return
        end
        if targ.components.combat then
            targ.components.combat:GetAttacked(inst.center,dmg,inst,'electric')
        end
    end
    PushAOE(inst,zap_range1,Zap)
end

local function red_circle(inst) --
end

local function simply_circle(inst) --怎么设计？
end


--------------------------------------------------------------------------------------

local function blue_fly_atk(inst)
    local function Freeze(targ) --将目标冰冻，对已冰冻目标造成真实伤害
        local dmg = ice_dmg * inst.components.acceleratingprojectile:GetSpeedRate(.5)
        if HitBoat(targ,dmg)then
            return
        end
        if not targ:HasTag("player")then
            dmg = dmg * 3
        end
        if not targ.components.health then
            return
        end
        local freezable = targ.components.freezable
        if freezable and not targ.components.health:IsDead() then
            if freezable:IsFrozen()then
                targ.components.combat:GetAttacked(inst.center, dmg, inst)
            else
                if targ:HasTag('sayaka_lunge') then
                    targ.components.combat:GetAttacked(inst.center,0.01,inst) --为了触发减cd判定
                else
                    freezable:AddColdness(100)
                end
            end
        end
    end
    PushAOE(inst,ice_range2,Freeze)
    inst:PushEvent("remove")
end

local function red_fly_attack(inst)
    local function Burn(targ) --将目标点燃，并造成真实伤害
        local dmg = fire_dmg * inst.components.acceleratingprojectile:GetSpeedRate(.5)
        if HitBoat(targ,dmg)then
            return
        end
        if not targ:HasTag("player")then
            dmg = dmg * 3
        end
        local immue = false
        if targ.components.burnable and not targ.components.burnable:IsBurning() then
            targ.components.burnable:Ignite()
        end
        if targ:HasTag("player")then
            if targ.components.health.fire_damage_scale <= 0 then
                immue = true
            elseif targ.components.moisture.moisture > 5 then
                immue = true
                targ.components.moisture:DoDelta(-5)
            end
        elseif GetMoistureManager():IsEntityWet(targ) then
            immue = true
        end

        if immue == false and targ.components.combat then
            targ.components.combat:GetAttacked(inst.center,dmg,inst)
        end
    end
    PushAOE(inst,fire_range2,Burn)
    inst:PushEvent("remove")
end

local function green_fly_atk(inst)
    local function Poison(targ) --施加中毒效果，造成少量真实伤害
        local dmg = poison_dmg * inst.components.acceleratingprojectile:GetSpeedRate(.25)
        if not targ:HasTag("player")then
            dmg = dmg * 3
        end
        if targ.components.poisonable and not targ:HasTag('sayaka_lunge') then
            targ.components.poisonable:Poison(true)
        end
        if targ.components.combat then
            targ.components.combat:GetAttacked(inst.center, dmg, inst)
        end
    end
    PushAOE(inst,poison_range2,Poison)
    inst:PushEvent("remove")
end

local function lightning_fly_atk(inst)
    local function Zap(targ)       --电击
        local dmg = zap_dmg2 * inst.components.acceleratingprojectile:GetSpeedRate(.25)
        if GetMoistureManager():IsEntityWet(targ) then
            dmg = dmg * 2
        elseif targ.components.moisture then
            dmg = dmg * (targ.components.moisture.moisture/66+1)
        end
        if not targ:HasTag("player")then
            dmg = dmg * 3
        end
        if targ.components.inventory and targ.components.inventory:IsInsulated() then
            return
        end
        if not targ.components.health or targ.components.health:IsInvincible() then
            return
        end
        if not targ.components.combat then
            return
        end
        targ.components.combat:GetAttacked(inst.center, dmg, inst, 'electric')
    end
    PushAOE(inst,zap_range2,Zap)
    inst:PushEvent("remove")
end
        
local function simply_fly_atk(inst)
    local function Fn(targ)       --简单粗暴的破坏
        local dmg = simply_dmg * inst.components.acceleratingprojectile:GetSpeedRate(1)
        if HitBoat(targ,dmg)then
            return
        end
        if targ.components.workable and targ.components.workable.workleft > 0 and targ.components.workable.action ~= ACTIONS.NET then
            targ.components.workable:Destroy(inst)
        end
        if targ.components.combat then
            targ.components.combat:GetAttacked(inst.center,dmg,inst)
        end
    end
    PushAOE(inst,simply_range2,Fn)
    inst:PushEvent("remove")
end
-----------------------------------------------

local function Homing(inst)
    if inst.components.magicballmanager.circle then
        return 
    end
    local current = inst:GetPosition()
    local dest = inst.components.projectile.target and inst.components.projectile.target:GetPosition()
    if not dest then return end
    local destangle = inst:GetAngleToPoint(dest)    
    local oldangle = inst.Transform:GetRotation()
    local deltaangle = PUELLA_MOD_FN.DeltaAngle(oldangle,destangle) --0~360
    local speed = inst.components.acceleratingprojectile:GetSpeedRate(1)
    local rate = speed < 1.2 and 0.8 or 0.6 --低速状态下追踪能力更强
    if deltaangle < 45 then
        deltaangle = deltaangle * rate
    elseif deltaangle > 315 then
        deltaangle = (deltaangle-360) * rate
    end
    inst.Transform:SetRotation(oldangle+deltaangle)
end

local function OnLaunch(inst,attacker,target)
    if inst.circle_task then
        inst.circle_task:Cancel()
        inst.circle_task = nil
    end
    if inst.fly_taskfn then
        inst.fly_task = inst:DoPeriodicTask(FRAMES,inst.fly_taskfn)
    end
end

local function common(build,tailfn,notail)
    local inst = CreateEntity()
    local trans = inst.entity:AddTransform()
    local anim = inst.entity:AddAnimState()
    local snd = inst.entity:AddSoundEmitter()
    MakeInventoryPhysics(inst)
    RemovePhysicsColliders(inst)

    anim:SetBank(build)
    anim:SetBuild(build)
    anim:PlayAnimation("anim",true)

    inst:AddComponent("projectile")
    inst.components.projectile.homing = false
    inst.components.projectile:SetSpeed(10)
    --inst.components.projectile:hitdist = 1.5
    inst:DoPeriodicTask(1/6.01,Homing) --弱追踪能力，可以使用闪现躲避

    if not notail then
        inst:ListenForEvent("startperiodictask",function()
            inst:DoPeriodicTask(FRAMES,UpdateTail,nil,build,build,{},tailfn)
            inst.Light:Enable(true)
        end)
    end

    inst:AddComponent("ignoretimemagic")
    inst.components.ignoretimemagic:Set(0)

    inst:AddComponent("weapon")
    inst.components.weapon:SetDamage(0)

    inst:AddComponent("acceleratingprojectile")
    inst.components.acceleratingprojectile.onlaunch = OnLaunch

    inst:AddTag("NOCLICK")
    inst:AddTag("FX")
    inst.persists = false

    PUELLA_MOD_FN.SetSG(inst,sg)

    --inst:ListenForEvent("entitysleep", inst.Remove) 

    return inst
end

local function bluefn()  
    local function fn(tail)
        tail.AnimState:SetBank("witch_tv_blue")
        tail.AnimState:SetBuild("witch_tv_blue")
        tail.AnimState:OverrideSymbol("snow0", "witch_tv_blue", "snow"..math.random(4))
        tail.AnimState:SetAddColour(.5,.5,1,1)
        tail.Transform:SetScale(.7,.7,.7)
        local pos = tail:GetPosition()
        tail.Transform:SetPosition((pos+Vector3(0,1.3,0)):Get())
    end  
    local inst = common("witch_tv_blue",fn) 
    --inst.AnimState:SetAddColour(.5,.5,1,1)
    inst.components.projectile:SetOnMissFn(blue_fly_atk)
    inst.components.projectile:SetOnHitFn(blue_fly_atk)

    SetLight(inst,ice_colour)

    inst.circle_atk_fn = blue_circle_atk

    inst.hitfx_fn = function (inst)
        local pos = inst:GetPosition()
        SpawnPrefab("witch_tv_blue_hit").Transform:SetPosition((pos+Vector3(0,.5,0)):Get())
    end

    inst:ListenForEvent("startperiodictask",function ()
        inst.circle_task = inst:DoPeriodicTask(30*FRAMES,blue_circle)
    end)

    inst:AddComponent("burnable")
    inst.components.burnable.onignite = function (inst) inst:PushEvent("remove") end

    inst:RemoveTag("NOCLICK")

    inst.sound = 'lw_witch/tv/ice'

    return inst
end


local function zapfn()   
    local function fn(tail)
        tail.AnimState:SetBloomEffectHandle("shaders/anim.ksh")
    end 
    local function lightning_normal(inst)
        local rod = FindEntity(inst,zap_range1,nil,{"lightningrod"},{"dead"})
        if rod then
            rod:PushEvent("lightningstrike", {rod=rod})
            inst:PushEvent("remove")
        end
    end
    local inst = common("bishop_attack",fn,true) 
    inst.AnimState:PlayAnimation("idle",true)
    inst.AnimState:SetBloomEffectHandle("shaders/anim.ksh")
    inst.Transform:SetScale(.8,.8,1)

    SetLight(inst,zap_colour)

    inst.components.projectile:SetOnMissFn(lightning_fly_atk)
    inst.components.projectile:SetOnHitFn(lightning_fly_atk)

    inst:ListenForEvent("startperiodictask",function ()
        inst.circle_task = inst:DoPeriodicTask(45*FRAMES,lightning_circle)
        inst.normal_task = inst:DoPeriodicTask(0.1,lightning_normal)
    end)

    inst.sound = 'lw_witch/tv/zap'
    
    return inst
end

local function redfn()
    local function fn(tail)
        tail.AnimState:SetBloomEffectHandle("shaders/anim.ksh")
    end
    local function fire_normal(inst)
        local pos = inst:GetPosition()
        local ents = TheSim:FindEntities(pos.x,pos.y,pos.z,4,nil,{"INLIMBO"})
        for _,v in pairs(ents)do
            if v.prefab == "splash_snow_fx" then
                inst:PushEvent("remove")
                return
            end
        end
    end

    local inst = common("witch_tv_red",fn) 
    inst.AnimState:SetBloomEffectHandle("shaders/anim.ksh")
    inst.components.projectile:SetOnMissFn(red_fly_attack)
    inst.components.projectile:SetOnHitFn(red_fly_attack)

    SetLight(inst,fire_colour)

    inst.hitfx_fn = function (inst)
        local pos = inst:GetPosition()
        SpawnPrefab("witch_tv_red_hit").Transform:SetPosition((pos+Vector3(0,1,0)):Get())
    end

    inst:AddComponent("burnable")
    inst.components.burnable.nofx = true
    inst.components.burnable.onextinguish = function (inst) inst:PushEvent("remove") end

    MakeSmallPropagator(inst)
    inst.components.propagator.damagerange = 3

    inst:ListenForEvent("startperiodictask",function ()
        inst.components.burnable:Ignite()
        inst.components.propagator:StartSpreading()
    end)
       
    inst:RemoveTag("NOCLICK")
    inst:RemoveTag("FX")
    inst.sound = 'lw_witch/tv/fire'

    return inst
end

local function greenfn()  
    local function fn(tail)
        tail.AnimState:SetMultColour(.2,1,0,1)
    end  
    local inst = common("witch_tv_green",fn) 
    inst.AnimState:SetMultColour(.2,1,0,1)
    inst.components.projectile:SetOnMissFn(green_fly_atk)
    inst.components.projectile:SetOnHitFn(green_fly_atk)

    SetLight(inst,poison_colour)

    inst.hitfx_fn = function (inst)   
        local pos = inst:GetPosition()
        SpawnPrefab("witch_tv_green_hit").Transform:SetPosition((pos+Vector3(0,1,0)):Get())
    end

    inst:ListenForEvent("startperiodictask",function ()
        inst.circle_task = inst:DoPeriodicTask(30*FRAMES,green_circle)
    end)

    inst.sound = 'lw_witch/tv/wind'
    return inst
end

local function simplyfn()
    local function fn(tail)
        -- body
    end
    local inst = common("bishop_attack",fn)
    inst.AnimState:PlayAnimation("idle")
    inst.components.projectile:SetOnMissFn(simply_fly_atk)
    inst.components.projectile:SetOnHitFn(simply_fly_atk)
    return inst
end

local function commonhit(bank)
    local inst = CreateEntity()
    inst.entity:AddTransform()
    inst.entity:AddAnimState()
    inst.entity:AddSoundEmitter()
    inst.AnimState:SetBank(bank)
    inst.AnimState:SetBuild(bank)
    inst:AddTag("FX")
    inst:AddTag("NOCLICK")
    inst.persists = false
    inst:ListenForEvent("animover", inst.Remove)
    return inst
end

local function greenhit()
    local inst = commonhit("witch_tv_green")
    inst.AnimState:PlayAnimation("hit"..math.random(2))
    inst.AnimState:SetMultColour(.2, 1, 0, 1)
    --inst.AnimState:SetFinalOffset(-1)
    inst.Transform:SetScale(1.5,1.5,1)
    return inst
end

local function redhit()
    local inst = commonhit("witch_tv_red")
    inst.AnimState:PlayAnimation("hit")
    inst.AnimState:SetBloomEffectHandle("shaders/anim.ksh")
    inst.Transform:SetScale(1.5,1.5,1)
    return inst
end

local function bluehit() 
    local inst = commonhit("frozen_shatter")
    inst.AnimState:PlayAnimation("small")
    for i = 1,13 do
        inst.AnimState:OverrideSymbol("dfs"..i,"witch_tv_blue","snow"..math.random(4))
    end
    return inst
end

return 

Prefab("witch_tv_green_hit",greenhit,assets,nil),
Prefab("witch_tv_blue_hit",bluehit,assets,nil),
Prefab("witch_tv_red_hit",redhit,assets,nil),
Prefab("witch_tv_simple",simplyfn,assets,prefabs),
Prefab("witch_tv_blue",bluefn,assets,prefabs),
Prefab("witch_tv_red",redfn,assets,prefabs),
Prefab("witch_tv_zap",zapfn,assets,prefabs),
Prefab("witch_tv_green",greenfn,assets,prefabs)
