local function ShowBoat(inst, vel)
    if inst.components.driver and inst.components.driver:GetIsDriving() then
        if vel then
            inst.components.driver.vehicle:Show()
        else -- if not inst.components.driver.vehicle.components.drivable.donothideonmounted then
            inst.components.driver.vehicle:Hide()
        end
    end
end
local function CheckWeaponCanRangeAttack(inst)
    local weapon = inst.components.combat:GetWeapon()
    return weapon and weapon.components.weapon:CanRangedAttack()
end

local function GetWeapon(inst)
    return inst.components.combat:GetWeapon()
end

local function AddExtraFx(inst, prefabname, offset, lockdir, postfn) --offset传入格式(半径,高度,是否反向)
    local fx = SpawnPrefab(prefabname)
    local pos = inst:GetPosition()
    local angle = inst.Transform:GetRotation()*DEGREES
    local dir = offset.z < 0 and -1 or 1
    local radius = offset.x
    local height = offset.y

    if lockdir then --特效只出现在人物的上、下、左、右
        local down_vec = TheCamera:GetDownVec()
        local right_vec = TheCamera:GetRightVec()
        local facing = inst.AnimState:GetCurrentFacing()

        if facing == FACING_UP then
            offset = down_vec*radius*dir + Vector3(0,height,0)
        elseif facing == FACING_DOWN then
            offset = down_vec*(-radius)*dir + Vector3(0,height,0)
        elseif facing == FACING_RIGHT then
            offset = right_vec*radius*dir + Vector3(0,height,0)
        else
            offset = right_vec*(-radius)*dir + Vector3(0,height,0)
        end
    else
        offset = Vector3(math.cos(angle)*radius*dir, height, -math.sin(angle)*radius*dir)
    end

    local down_mult = TheCamera:GetDownVec():Dot(offset)*dir --求点积,如果为正,则人物覆盖特效

    fx.Transform:SetRotation(inst.Transform:GetRotation())
    fx.Transform:SetPosition((offset+pos):Get())
    if down_mult > 0 then 
        fx.AnimState:SetLayer(LAYER_WORLD_BACKGROUND)
    else
        fx.AnimState:SetLayer(LAYER_WORLD_CEILING)
    end

    if postfn then
        postfn(inst, fx)
    end
end

local function updateSG(inst) 

    local old_doattack = inst.sg.sg.events['doattack'].fn
    inst.sg.sg.events['doattack'] = EventHandler('doattack', function(inst,data,...)
        --print('receive doattack event')
        if inst:HasTag('homura') and not inst.components.health:IsDead() then
            local item = inst.components.combat:GetWeapon()
            local target = inst.components.combat.target
            if not item then
                inst.sg:GoToState('attack')
                return
            end
            --有武器
            if item.components.weapon.projectile or item:HasTag('projectile') then
                inst.components.combat:AddDamageModifier("HOMURA",0)
            else
                inst.components.combat:AddDamageModifier("HOMURA",-0.25)
            end

            if item.prefab == "homura_magicwave_launcher" then
                inst.sg:GoToState("homura_magicalwave") 
                return
            end
            --@--
            if item.prefab == 'homura_pistol_sn' then
                inst.sg:GoToState('homura_pistol_pre')
            end

            if item.prefab == "homura_rpg" and item.components.weapon.projectile then
                if target and target:GetDistanceSqToInst(inst) <= 4*4 then
                    inst.components.talker:Say(PUELLA_MOD_SETTINGS.LANGUAGE and "It's too close!" or "太近了!")
                    inst.components.combat:SetTarget(nil)
                    inst.sg:GoToState("idle")
                else
                    inst.sg:GoToState("homura_rpg")
                end
                return
            end

            if item.components.homura_focus then
                if item.components.homura_focus:ShouldFocus(target) then
                    print('go in to focus')
                    inst.sg:GoToState('homura_focus', item)
                    return
                elseif item.components.homura_focus:IsFocusing() then
                    print('dont do any thing')
                    return
                elseif --item.components.homura_focus:IsFocused() and 
                        item.components.weapon:CanRangedAttack() then
                    print('now i can attack')
                    if item.prefab == 'homura_pistol' then
                        if not inst.sg:HasStateTag('homura_pistol_atk') then
                            inst.sg:GoToState('homura_pistol_pre')
                        end
                    elseif item.prefab == 'homura_gun' then
                        if not inst.sg:HasStateTag('homura_gun_atk') then
                            inst.sg:GoToState('homura_gun_pre')
                        end
                    elseif item.prefab == 'homura_hmg' then
                        if not inst.sg:HasStateTag('homura_hmg_atk') then
                            inst.sg:GoToState('homura_gun_pre') --共用同一个动画
                        end
                    end
                    return
                end
            end
        end
       
        old_doattack(inst,data,...)
    end)


    inst.sg.sg.states.homura_magicalwave = 
    State{ 
        name = "homura_magicalwave",
        tags = {"attack", "notalking", "abouttoattack", "busy"},
        
        onenter = function(inst)
            
            inst.AnimState:PlayAnimation("atk")  -----
            inst.AnimState:Show("ARM_carry")
            inst.AnimState:Hide("ARM_normal") 
            inst.SoundEmitter:PlaySound("dontstarve/wilson/attack_nightsword") ------
            
            if inst.components.combat.target then
                inst.components.combat:BattleCry()
                if inst.components.combat.target and inst.components.combat.target:IsValid() then
                    inst:FacePoint(Point(inst.components.combat.target.Transform:GetWorldPosition()))
                end
            end

            inst.sg.statemem.target = inst.components.combat.target
            inst.components.combat:StartAttack()
            inst.components.locomotor:Stop()
            
        end,
        
        timeline=
        {
            TimeEvent(8*FRAMES, function(inst) 
                inst.components.combat:DoAttack(inst.sg.statemem.target) 
                if GetWeapon(inst) and GetWeapon(inst).components.weapon.magicfn then
                    GetWeapon(inst).components.weapon.magicfn(inst)
                end
                inst.sg:RemoveStateTag("abouttoattack") 
            end),
            TimeEvent(15*FRAMES, function(inst) 
                inst.sg:RemoveStateTag("busy")
            end),               
            TimeEvent(18*FRAMES, function(inst)
                inst.sg:RemoveStateTag("attack")
            end),                
        },
        
        events=
        {
            EventHandler("animover", function(inst)
                inst.sg:GoToState("idle")
            end ),
        },

        onexit= function (inst)
            inst.AnimState:Show("ARM_normal")
            inst.AnimState:Hide("ARM_carry") 
        end,
    }  

    inst.sg.sg.states.homura_focus = 
    State{
        name = 'homura_focus',
        tags = {"notalking","busy","homura_focus"},

        onenter = function(inst, weapon)
            ShowBoat(inst, true)
            local target = inst.components.combat.target
            inst.sg.statemem.target = target
            print(target or "NO TARGET")
            inst.sg.statemem.weapon = weapon
            inst.components.locomotor:Stop()
            if weapon and weapon.prefab == 'homura_pistol' then
                inst.AnimState:PlayAnimation('hand_shoot')
                inst.SoundEmitter:PlaySound('lw_homura/sfx/load_up',nil,0.7)
            else
                inst.AnimState:PlayAnimation('homura_gun_pre')
                inst.SoundEmitter:PlaySound('lw_homura/sfx/load_down',nil,0.7)
            end
            weapon.components.homura_focus.to_focustarget = target
            weapon.components.homura_focus.focustarget = nil
            inst:PushEvent('homura_focus_start',{target = target})

        end,

        timeline = {
            --和抬枪前摇播放相同的动画, 但在结束时暂停或loop
            TimeEvent(11*FRAMES,function(inst)
                if inst.sg.statemem.weapon and inst.sg.statemem.weapon.prefab == 'homura_pistol' then
                    inst.AnimState:Pause() --极致偷懒...
                end
            end),
            TimeEvent(12*FRAMES,function(inst)
                if inst.sg.statemem.weapon and inst.sg.statemem.weapon.prefab ~= 'homura_pistol' then
                    inst.AnimState:PlayAnimation('homura_gun_focus',true)
                end
            end),
        },

        onupdate = function(inst)
            local target = inst.sg.statemem.target
            if target == inst.components.combat.target then --攻击目标与锁定目标相同
                inst.components.combat:SetTarget(nil)  --清除, 防止exit后自动发动攻击
            elseif inst.components.combat.target then
                inst.sg.statemem.changetarget = true --由于切换目标而重新进入state
                inst.sg:GoToState('homura_focus',inst.sg.statemem.weapon)
            end
            
            if target and target:IsValid() and not target:HasTag("INLIMBO")and not target.components.health:IsDead()then
                inst:ForceFacePoint(target:GetPosition())
            else
                print('target lost')
                inst.sg.statemem.weapon:PushEvent('homura_focus_abrupt')
                inst.sg:GoToState('idle')--@--
            end
        end,


        events = 
        {
            EventHandler('homura_focus_down',function(inst) --接收ui发出的事件 (理想情况下所有逻辑处理都应该在sg内实现, 但是我懒得改了)
                inst.sg:RemoveStateTag('busy')
                inst.sg.statemem.down = true
                local weapon = inst.components.combat:GetWeapon()
                if weapon and weapon.prefab == 'homura_pistol' then
                    inst.sg:GoToState('homura_pistol_atk')
                elseif weapon and weapon.prefab == 'homura_gun' then
                    inst.sg:GoToState('homura_gun_atk')
                elseif weapon and weapon.prefab == 'homura_hmg' then
                    inst.sg:GoToState('homura_hmg_atk')
                end
                inst:PushEvent('doattack',{target = inst.sg.statemem.target,weapon = inst.sg.statemem.weapon})
            end),
            EventHandler('unequip',function(inst)
                inst.sg:GoToState('item_in') --手动进入卸下状态(因为带了busy标签)
            end),
        },    

        onexit = function(inst, newstate) --statemem真好用!
            
            if inst.sg.statemem.down then 
                inst.components.combat:SetTarget(inst.sg.statemem.target)
                inst.sg.statemem.weapon:PushEvent('homura_focus_down')
                print('focus down')
            elseif inst.sg.statemem.changetarget then
                inst.sg.statemem.weapon:PushEvent('homura_focus_abrupt')
                print('change target')
            else
                inst:PushEvent('homura_focus_abrupt')
                inst.sg.statemem.weapon:PushEvent('homura_focus_abrupt')
                print('focus abrupt')
            end
            inst.AnimState:Resume()
            ShowBoat(inst, false)
        end,
    }

    inst.sg.sg.states.homura_pistol_pre = 
    State{
        name = 'homura_pistol_pre',
        tags = {'attack','notalking','abouttoattack','busy'},

        onenter = function(inst)
            --ShowBoat(inst, true)
            inst.AnimState:PlayAnimation("hand_shoot")
            inst.SoundEmitter:PlaySound('lw_homura/sfx/load_up',nil,.5)

            if inst.components.combat.target then
                inst.components.combat:BattleCry()
                if inst.components.combat.target and inst.components.combat.target:IsValid() then
                    inst:FacePoint(Point(inst.components.combat.target.Transform:GetWorldPosition()))
                end
            end
            inst.sg.statemem.target = inst.components.combat.target
            inst.components.combat:StartAttack()
            inst.components.locomotor:Stop()            
        end,
        
        timeline=
        {
            TimeEvent(11*FRAMES, function(inst)
                --inst.AnimState:Pause()
                inst.sg:GoToState('homura_pistol_atk',inst.sg.statemem.target)
                --此处sg即结束
            end),
            TimeEvent(17*FRAMES, function(inst) 
                inst.components.combat:DoAttack(inst.sg.statemem.target) 
                inst.sg:RemoveStateTag("abouttoattack")
            end),            
            TimeEvent(20*FRAMES, function(inst)
                inst.sg:RemoveStateTag("attack")
            end),          
        },
        
        events=
        {
            --EventHandler("animover", function(inst)
            --    inst.sg:GoToState("idle")
            --end ),
        },
        --onexit = function(inst) ShowBoat(inst, false) end,
    }
    --手枪
    inst.sg.sg.states.homura_pistol_atk = 
    State{
        name = 'homura_pistol_atk',
        tags = {'abouttoattack','attack','busy','notalking','homura_pistol_atk'},

        onenter = function (inst,p)
            --ShowBoat(inst, true)
            if not inst.AnimState:IsCurrentAnimation('hand_shoot')then
                inst.AnimState:PlayAnimation('hand_shoot')
            end
            inst.components.combat:StartAttack()
            inst.AnimState:SetTime(11*FRAMES)
            inst.sg.statemem.target = p or inst.components.combat.target
            inst.sg.statemem.gun = inst.components.combat:GetWeapon()
        end,

        timeline = {
            --17帧攻击
            TimeEvent(6*FRAMES, function(inst) 
                inst.components.combat:DoAttack(inst.sg.statemem.target) 
                inst.sg:RemoveStateTag("abouttoattack") 
                local weapon = inst.sg.statemem.gun
                if weapon and weapon.components.homura_weapon then
                    inst.SoundEmitter:PlaySound(weapon.components.homura_weapon:GetSound())
                else
                    inst.SoundEmitter:PlaySound('lw_homura/pistol/normal', nil, .6)
                end
                --@?--
                AddExtraFx(inst, 'homura_gun_fire', Vector3(1.1,1.1,0), true, function(inst,fx) fx:SetRandomScale(0.6,0.8) end)
            end),            
            TimeEvent(6*FRAMES, function(inst)
                inst.sg:RemoveStateTag("attack")
            end),
            TimeEvent(18*FRAMES, function(inst)
                local target = inst.components.combat.target
                if (target and target:IsValid() and not target.components.health:IsDead() and target == inst.sg.statemem.target)then
                    if CheckWeaponCanRangeAttack(inst) then
                        inst.sg:GoToState('homura_pistol_atk',inst.sg.statemem.target)
                        if GetWeapon(inst).components.homura_weapon then
                            GetWeapon(inst).components.homura_weapon:EnterCSMode()
                        end
                    else
                        --inst.sg:GoToState('idle') 
                    end
                end
            end),
        }, 

        events = {
            EventHandler('animover',function(inst)
                inst.sg:GoToState('idle')
            end),
        },
        --onexit = function(inst) ShowBoat(inst, false) end,
    }
    --步枪 前摇
    inst.sg.sg.states.homura_gun_pre = 
    State{
        name = 'homura_gun_pre',
        tags = {"attack", "notalking", "abouttoattack", "busy"},

        onenter = function(inst)
            ShowBoat(inst, true)
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation('homura_gun_pre')
            inst.SoundEmitter:PlaySound('lw_homura/sfx/load_down',nil,.7)
            inst.sg:SetTimeout(0.4)
            inst.sg.statemem.target = inst.components.combat.target
        end,

        ontimeout = function(inst) 
            if GetWeapon(inst) and GetWeapon(inst).prefab == 'homura_gun' then
                inst.sg:GoToState('homura_gun_atk', inst.sg.statemem.target)
            else
                inst.sg:GoToState('homura_hmg_atk', inst.sg.statemem.target)
            end
        end,
        --events = {EventHandler('animover',function(inst)inst.sg:GoToState('homura_gun_atk', inst.sg.statemem.target)end)},

        onexit = function(inst) ShowBoat(inst, false) end,
    }

    inst.sg.sg.states.homura_gun_atk = 
    State{
        name = 'homura_gun_atk',
        tags = {"attack", "notalking", "abouttoattack", "busy", "homura_gun_atk"},

        onenter = function(inst, target)
            ShowBoat(inst, true)
            inst.AnimState:OverrideSymbol('player_homura_gun_fire','homura_gun_fire','fire')
            inst.components.locomotor:Stop()
            inst.components.combat:StartAttack()
            inst.AnimState:PlayAnimation('homura_gun_loop')
            inst.sg.statemem.target = inst.components.combat.target or target
            inst.sg.statemem.moveback = function(inst, l)
                local l = l/30
                local pos = inst:GetPosition()
                local facing = inst.Transform:GetRotation()
                local angle = (facing + 270)*DEGREES
                local targetpos = pos + Vector3(l*math.sin(angle),0,l*math.cos(angle))
                inst.Physics:Teleport(targetpos:Get())
            end
        end,

        onupdate = function(inst)
            if inst.sg.statemem.target and inst.sg.statemem.target:IsValid() then
                inst:ForceFacePoint(inst.sg.statemem.target:GetPosition())
            elseif inst.components.combat.target and inst.components.combat.target:IsValid() then
                inst.sg.statemem.target = inst.components.combat.target
            end

            local weapon = inst.components.combat:GetWeapon() 
            if not (weapon and weapon.components.weapon:CanRangedAttack()) then
                inst.sg.statemem.exit = true
                inst.sg:GoToState('homura_gun_pst')
                return
            end
        end,

        timeline = {
            TimeEvent(1*FRAMES, function(inst)
                --if not CheckWeaponCanRangeAttack(inst) then
                --    inst.sg:GoToState('homura_gun_pst')
                --end
            end),
            TimeEvent(2*FRAMES, function(inst)
                if not CheckWeaponCanRangeAttack(inst) then
                    inst.sg:GoToState('homura_gun_pst')
                    return
                end
                AddExtraFx(inst, 'homura_gun_light', Vector3(2,0,0))
                inst.components.combat:DoAttack(inst.sg.statemem.target)
                inst.sg.statemem.moveback(inst, 0.4)
                if GetWeapon(inst).components.homura_weapon then
                    inst.SoundEmitter:PlaySound(GetWeapon(inst).components.homura_weapon:GetSound())
                end
            end),
            TimeEvent(3*FRAMES, function(inst)
                inst.sg:RemoveStateTag('busy')
                inst.sg:RemoveStateTag('attack')
                inst.sg:RemoveStateTag('abouttoattack')
            end),
            TimeEvent(4*FRAMES, function(inst)
                if not CheckWeaponCanRangeAttack(inst) then
                    inst.sg:GoToState('homura_gun_pst')
                elseif not TheInput:IsControlPressed(CONTROL_ATTACK) and not TheInput:IsControlPressed(CONTROL_PRIMARY) then
                    inst.sg:GoToState('homura_gun_pst')
                elseif not(inst.sg.statemem.target and inst.sg.statemem.target:IsValid() and not 
                    inst.sg.statemem.target.components.health:IsDead()) then
                    inst.sg:GoToState('homura_gun_pst')
                else
                    local weapon = inst.components.combat:GetWeapon()
                    if weapon and weapon.components.homura_weapon then
                        weapon.components.homura_weapon:EnterCSMode()
                    end

                    inst.sg:GoToState('homura_gun_atk', inst.sg.statemem.target)
                end
            end),
        },
        onexit = function(inst)
            if inst.sg.statemem.exit then
                inst.components.combat:SetTarget(nil)
            end
            ShowBoat(inst, false)
        end,
    }
    --机枪
    inst.sg.sg.states.homura_hmg_atk = 
    State{
        name = 'homura_hmg_atk',
        tags = {"attack", "notalking", "abouttoattack", "busy", "homura_hmg_atk"},

        onenter = function(inst, target)
            ShowBoat(inst, true)
            inst.AnimState:OverrideSymbol('player_homura_gun_fire','homura_gun_fire','fire')
            inst.components.locomotor:Stop()
            inst.components.combat:StartAttack()
            inst.AnimState:PlayAnimation('homura_gun_loop', true)
            inst.sg.statemem.vol = GetWeapon(inst).components.homura_weapon.buffed_data.silent and 0.1 or 0.7
            --print("volume: "..inst.sg.statemem.vol)
            --@--用同一个音效?
            inst.SoundEmitter:PlaySound('lw_homura/hmg/pst', 'SG -> homura_hmg_atk -> pst', inst.sg.statemem.vol)
            inst.sg.statemem.target = inst.components.combat.target or target
            inst.sg.statemem.moveback = function(inst, l)
                local l = l/30
                local pos = inst:GetPosition()
                local facing = inst.Transform:GetRotation()
                local angle = (facing + 270)*DEGREES
                local targetpos = pos + Vector3(l*math.sin(angle),0,l*math.cos(angle))
                inst.Physics:Teleport(targetpos:Get())
            end
            inst.sg.statemem.exittick = 3 --至少射出3发子弹才会结束sg
        end,

        onupdate = function(inst)
            if inst.sg.statemem.target and inst.sg.statemem.target:IsValid()then
                if inst.sg.statemem.target.components.health:IsDead() then
                    inst.sg:GoToState('homura_gun_pst')
                    return
                else
                    inst:ForceFacePoint(inst.sg.statemem.target:GetPosition())
                end
            elseif inst.components.combat.target and inst.components.combat.target:IsValid() then
                inst.sg.statemem.target = inst.components.combat.target
            end

            if not CheckWeaponCanRangeAttack(inst) then
                inst.sg:GoToState('homura_gun_pst')
                return
            end

            if inst.sg.statemem.exittick == 0 then --射完前3发子弹后, 移除标签, 播放循环音效
                inst.sg:RemoveStateTag('attack')
                inst.sg:RemoveStateTag('busy')
                if not inst.SoundEmitter:PlayingSound('SG -> homura_hmg_atk -> loop') then
                    inst.SoundEmitter:PlaySound('lw_homura/hmg/loop', 'SG -> homura_hmg_atk -> loop', inst.sg.statemem.vol)
                end
            elseif inst.sg.statemem.exittick < 0 then --不按住攻击键则退出sg
                if not TheInput:IsControlPressed(CONTROL_ATTACK) and not TheInput:IsControlPressed(CONTROL_PRIMARY) then
                    inst.sg:GoToState('homura_gun_pst')
                    return
                end

                local weapon = GetWeapon(inst)
                if weapon and weapon.components.homura_weapon then
                    weapon.components.homura_weapon:EnterCSMode()
                end
            end
            AddExtraFx(inst, 'homura_gun_light', Vector3(2,0,0))
            inst.components.combat:DoAttack(inst.sg.statemem.target)
            inst.sg.statemem.moveback(inst, 0.3)

            inst.sg.statemem.exittick = inst.sg.statemem.exittick - 1
            inst.components.combat:SetTarget(nil)
        end,

        onexit = function(inst)
            inst.components.combat:SetTarget(nil)
            inst.SoundEmitter:KillSound('SG -> homura_hmg_atk -> loop')
            ShowBoat(inst, false)
        end,
    }

    inst.sg.sg.states.homura_gun_pst = 
    State{
        name = 'homura_gun_pst',
        tags = {"notalking"},
        onenter = function(inst)
            ShowBoat(inst, true)
            inst.AnimState:PlayAnimation('homura_gun_pst') 
            inst.sg:SetTimeout(5/30)
        end,
        ontimeout = function(inst) inst.sg:GoToState('idle')end,
        onexit = function(inst) ShowBoat(inst, false) end,
    }

    inst.sg.sg.states.homura_rpg = 
    State{
        name = "homura_rpg",
        tags = {"attack", "notalking", "abouttoattack"},
        
        onenter = function(inst)
            ShowBoat(inst, true)
            inst.sg.statemem.target = inst.components.combat.target
            --inst.sg.statemem.targetpos = inst.sg.statemem.target:GetPosition()

            inst.components.combat:StartAttack()
            inst.components.locomotor:Stop()
            inst.AnimState:PlayAnimation("homura_rpg")

            if inst.components.combat.target then
                if inst.components.combat.target and inst.components.combat.target:IsValid() then
                    inst:FacePoint(inst.components.combat.target:GetPosition())
                end
            end
        end,
        
        timeline=
        {
            TimeEvent(1*FRAMES,function(inst) inst.SoundEmitter:PlaySound("lw_homura/sfx/rpg")end),
           
            TimeEvent(11*FRAMES, function(inst)
                inst.sg:RemoveStateTag("abouttoattack")
                if inst.sg.statemem.target:IsValid() then
                    inst.components.combat:DoAttack(inst.sg.statemem.target)
                elseif inst.sg.statemem.targetpos then --目标无法攻击, 则直接向前发射
                    --
                end
                AddExtraFx(inst, 'homura_gun_light', Vector3(1,0,0))
                AddExtraFx(inst, 'homura_rpg_smoke', Vector3(0.5,1,-1))
            end),
            TimeEvent(20*FRAMES, function(inst) inst.sg:RemoveStateTag("attack") end),
            TimeEvent(21*FRAMES, function(inst) inst.sg:GoToState('idle') end),
        },
        
        events=
        {
            --EventHandler("animover", function(inst)inst.sg:GoToState("idle")end),
        },
        onexit = function(inst) ShowBoat(inst, false) end,
    }

    --瞄准动作带有busy标签,但可以被行走动作打断
    local old_locomote = inst.sg.sg.events['locomote'].fn
    inst.sg.sg.events['locomote'] = EventHandler('locomote', function(inst,data,...)
        if inst.sg.currentstate and inst.sg.currentstate.name == 'homura_focus' then
            inst.sg:RemoveStateTag('busy')
        end
        old_locomote(inst,data,...)
    end)
end

local function SetHomuraSG(inst)
    inst.AnimState:AddOverrideBuild("player_pistol_pmmm")
    updateSG(inst)
    inst:ListenForEvent('pmmm: force update sg',updateSG)
    inst:ListenForEvent("dismountboat",updateSG)
    inst:ListenForEvent("mountboat",updateSG)
end

return SetHomuraSG
     
