local UIAnim = require "widgets/uianim"
local Widget = require "widgets/widget"
local Badge = require "widgets/badge"

local utilfn = require "widgets/puellaUI_sgdark_util"

local vig = Class(Widget, function(self, owner)

	Widget._ctor(self, "vig_grief")
	
	self.owner = owner
	self:SetClickable(false)

	self.img = self:AddChild(Image("images/fx.xml", "ice_over.tex"))
	self.img:SetTint(0,0,0,1)
	self.img:SetEffect( "shaders/uifade.ksh" )
    self.img:SetHAnchor(ANCHOR_MIDDLE)
    self.img:SetVAnchor(ANCHOR_MIDDLE)
    self.img:SetScaleMode(SCALEMODE_FILLSCREEN)
    self.img:SetClickable(false)

	self.shine = self:AddChild(UIAnim())
    self.shine:GetAnimState():SetBuild("vig")
    self.shine:GetAnimState():SetBank("vig")
    for i = 1,4 do self.shine:GetAnimState():OverrideSymbol('vein0'..i,'','') end
    self.shine:GetAnimState():PlayAnimation("insane", true)
    self.shine:SetHAnchor(ANCHOR_MIDDLE)
    self.shine:SetVAnchor(ANCHOR_MIDDLE)
    self.shine:SetScaleMode(SCALEMODE_FIXEDSCREEN_NONDYNAMIC)
    self.shine:SetClickable(false)

    self.img:Hide()
    self.shine:Hide()
    
    self.alpha_min = 1
    self.alpha_min_target = 1
    self.lock_alpha_time = 0

    self.inst:ListenForEvent('soulgemdelta',function(i,data)
    	if data.newpercent >= 1 then
    		self.img:Show()
    		self.shine:Show()
    		self:SetAlphaMinTarget(1)
    	elseif data.newpercent < 1 and self.lock_alpha_time <= 0 then
    		self.img:Hide()
    		self.shine:Hide()
    		self.alpha_min_target = 1
    	end
    end,owner)

    self.shadows = { }

    if self.owner:HasTag("puellamagi") and self.owner.components.soulgem then
    	self.inst:DoTaskInTime(1,function()self:Spawner1()end)
    	self.inst:DoTaskInTime(2,function()self:Spawner2()end)
    	self.inst:DoTaskInTime(3,function()self:Spawner3()end)
    end

    self.mindist = 50
    self.alpha = 0.5
    self.checkdistance = true
    self:SetScale(1.6)

    self.inst:ListenForEvent('soulgemdelta',function(i,data)self:SetSpawnRate(data)end,owner)
end)

function vig:SetAlphaValue(num)
	self.alpha_min_target = num
	self.lock_alpha_time = 0.1
	self:StartUpdating()
end

function vig:SetAlphaMinTarget(targ,tomax)
	if self.lock_alpha_time > 0 then
		return
	end
	local alpha_mins = {.7,.5,.3,0}
	local old_target = self.alpha_min_target
	local new_target = alpha_mins[targ]
	if tomax and new_target < old_target then
		return
	end
	self.alpha_min_target = new_target
	self:StartUpdating()
end

function vig:OnUpdate(dt)
	self.lock_alpha_time = self.lock_alpha_time - dt
	local lspeed = dt*2
	self.alpha_min = (1 - lspeed) * self.alpha_min + lspeed *self.alpha_min_target
	self.img:SetAlphaRange(self.alpha_min,1)
	
	if self.alpha_min >= .99 then
		self.img:Hide()
		self:StopUpdating()
	else
		self.img:Show()
	end
end

function vig:CheckDistance(dist,pos)
	for k,v in pairs(self.shadows)do
		local pos2 = v:GetPosition()
		if pos2:Dist(pos) <= dist then
			return false
		end
	end
	return true
end

local function IsInOval(a,b,x,y)
	return (x/a)^2 + (y/b)^2 <= 1
end

local function Minus()
	if math.random() > .5 then return -1 else	return 1 end
end

local function GetRandom(v)
	-- y = x^0.5
	-- y = ax + b  -- 斜率a与上一个函数相切
	-- y = -(1-x)^0.5 + c
	local x1 = v
	local y1 = v^0.5
	local k = 0.5*v^(-0.5)

	local x2 = 1-v
	local y2 = y1+k*(x2-x1)

	local c = y2+(1-x2)^0.5

	local max = y1+y2

	local x = math.random()
	if x <= v then
		return (x^0.5)/max
	elseif x >= 1-v then
		return ( -(1-x)^0.5+c )/max
	else
		return (Remap(x,x1,x2,y1,y2))/max
	end
end

function vig:GetSpawnPos()
	--广义极坐标
	local a,b = 600,450
	local ran = GetRandom(0.1)
	local r = Remap(ran,0,1,0.5,1)
	local theta = math.random()*2*PI
	local pos = Point(a*math.cos(theta)*r,b*math.sin(theta)*r,0)

	return pos
end

function vig:SpawnShadow()
	local pos = self:GetSpawnPos()
	local newshadow = utilfn.PickUpRandomShadow(self.owner)
	if not newshadow then return end
	local GUID = newshadow.inst.entity:GetGUID()

	newshadow:SetPosition(pos:Get())
	newshadow:SetRotation(math.random(360))
	newshadow.inst.alpha = self.alpha
	newshadow:Hide()
	newshadow.inst:DoTaskInTime(0,function()newshadow:Show()end)
	newshadow.inst:DoTaskInTime(30,function(inst)inst:Remove();self.shadows[GUID]=nil;self:RemoveChild(newshadow)end)

	self.shadows[GUID] = self:AddChild(newshadow)
	self.inst:ListenForEvent('onremove',function()self.shadows[GUID] = nil end, newshadow)
	
	return newshadow,GUID
end

function vig:Spawner1()
	if self.spawner1_enabled then self:SpawnShadow(0) end
	self.inst:DoTaskInTime(GetRandomMinMax(3,7),function()self:Spawner1()end)
end

function vig:Spawner2()
	if self.spawner2_enabled then self:SpawnShadow(.25) end
	self.inst:DoTaskInTime(GetRandomMinMax(3,7),function()self:Spawner2()end)
end

function vig:Spawner3()
	if self.spawner3_enabled then self:SpawnShadow(.5) end
	self.inst:DoTaskInTime(GetRandomMinMax(2,6),function()self:Spawner3()end)
end

function vig:EndBurst()
	if self.burst_task then
		self.burst_task:Cancel()
		self.burst_task = nil
	end
	self.mindist = 30
end
function vig:StartBurst()
	self:EndBurst()
	self.burst_task = self.inst:DoPeriodicTask(0.5,function()self:SpawnShadow(.5)end)
	self.mindist = 0
end

function vig:StartBurstAfterDie() 
	self.inst:DoPeriodicTask(0,function()self:SpawnShadow(.5)end)
end

function vig:Clear()
	for _,v in pairs(self.shadows)do
		if not v.clear then v.clear = true; utilfn.Clear(v) end
	end
end
function vig:SetSpawnRate(data)
	local old,new = data.oldpercent,data.newpercent
	self.spawner1_enabled = new > .85 and true or false
	self.spawner2_enabled = new > .90 and true or false
	self.spawner3_enabled = new > .95 and true or false
	if old < 1 and new >= 1 then
		self:StartBurst()
	elseif old >=1 and new < 1 then
		self:EndBurst()
	end
	if old >= 0.7 and new < 0.7 then
		self:Clear()
	end
	if new < 0.5 then
		self:Clear()
	end

	self.alpha = Remap(new,0.7,1,0.7,1)
	for _,v in pairs(self.shadows)do
		v.inst.alpha = self.alpha
	end
end

return vig